<?php
declare(strict_types=1);

/**
 * webEdition CMS
 *
 * $Rev: 2fcfd49ac317 $
 * $Author: Marc Kraemer <marc.kraemer@aldente-it.de> $
 * $Date: 2020-07-06 14:53 +0200 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
class we_chooser_multiDirTemplateAndDefault extends we_chooser_multiDir{
	/** @var string */
	private $defaultName = '';

	/** @var int[] */
	private $defaultArr = [];

	/** @var string */
	protected $tmplSelectName = '';

	/** @var  int[] */
	protected $tmplArr = [];

	/** @var array<int,int> */
	protected $mustTemplateIDs = [];

	/** @var array<int,string> */
	protected $mustPaths;

	/** @var bool */
	public $create = false;

	/**
	 * @param int[] $ids
	 * @param string $cmd_del
	 * @param string $addbut
	 * @param int[] $ws
	 * @param int[] $tmplcsv
	 * @param string $tmplSelectName
	 * @param int[] $mustTemplateIDs
	 * @param array<int,int> $tmplWs
	 * @param string $defaultName
	 * @param int[] $defaultArr
	 * @param string $fields
	 * @param string $table
	 * @param string $css
	 */
	public function __construct(array $ids, string $cmd_del, string $addbut, array $ws = [], array $tmplcsv = [], string $tmplSelectName = '', array $mustTemplateIDs = [
		], array $tmplWs = [], string $defaultName = '', array $defaultArr = [
		], string $fields = 'Path', string $table = FILE_TABLE, string $css = ''){
		$this->defaultName = $defaultName;
		$this->defaultArr = $defaultArr;
		parent::__construct($ids, $cmd_del, $addbut, $ws, $fields, $table, $css);
		$this->tmplSelectName = $tmplSelectName;
		$this->mustTemplateIDs = $mustTemplateIDs;
		$this->tmplArr = $tmplcsv;
		$this->mustPaths = id2pathA($this->mustTemplateIDs, TEMPLATES_TABLE);

		$this->lines = 3;
	}

	function getRootLine(int $lineNr): string{

		switch($lineNr){
			case 0:
				return '<tr>
	<td class="chooserFileIcon" data-contenttype="' . we_base_ContentTypes::FOLDER_DOCUMENT . '"></td>
	<td class="' . $this->css . '">/</td>
	<td>' . ((($this->isEditable && $this->cmd_del) || $this->CanDelete) ?
					we_html_button::create_button(we_html_button::TRASH, "javascript:" . $this->getJsSetHot() . ($this->extraDelFn ?: "") . ";we_cmd('" . $this->cmd_del . "','0');") :
					"") . '</td>
</tr>';
			default:
				return $this->getLine($lineNr);
		}
	}

	private static function getTemplateSelector(string $idname, int $id, string $path): string{
		$weSuggest = we_gui_suggest::getInstance();
		$textname = $idname . 'Name';

		$weSuggest->setAcId('Template')
			->setContentType([we_base_ContentTypes::TEMPLATE])
			->setInput($textname, $path)
			->setMaxResults(10)
			->setResult($idname, (string) $id)
			->setSelector(we_gui_suggest::DocSelector)
			->setTable(TEMPLATES_TABLE)
			->setWidth(0)
			->setSelectButton(we_html_button::create_button(we_html_button::SELECT, "javascript:we_cmd('we_selector_document',{id:document.we_form.elements['" . $idname . "'].value,table:WE().consts.tables.TEMPLATES_TABLE,targetID:'" . $idname . "',targetName:'" . $textname . "',jsCmd:'setHot',filter:WE().consts.contentTypes.TEMPLATE,open_doc:true})"));
		//$weSuggest->setIsDropFromTree(true);// deactivated
		return $weSuggest->getHTML();
	}

	function getLine(int $lineNr): string{

		//$editable = $this->isEditable();
		switch($lineNr){
			case 0:
				return parent::getLine(0);
			case 1:
				$checkbox = we_html_forms::checkbox((string) $this->ids[$this->nr], in_array($this->ids[$this->nr], $this->defaultArr), $this->defaultName . '_' . $this->nr, g_l('weClass', '[standard_workspace]'));
				return '<tr><td></td><td>' . $checkbox . '</td><td style="width:50px;"></td></tr>';
			case 2:
				if($this->create){
					$but = we_html_button::create_button('fa:btn_add_template,fa-plus,fa-lg fa-file-code-o', "javascript:we_cmd('we_object_class_createTemplate','0','" . $this->nr . "','" . we_base_WE::$transaction . "')");
				} else {
					$but = we_html_button::create_button(we_html_button::VIEW, "javascript:we_cmd('we_object_file_preview','0','" . ($this->tmplArr[$this->nr] ?? '') . "','" . we_base_WE::$transaction . "')");
				}
				$nr = !empty($this->tmplArr[$this->nr]) ? (int) $this->tmplArr[$this->nr] : 0;
				$path = id2path($nr, TEMPLATES_TABLE);
				if($this->isEditable){
					return '<tr><td></td><td>' . we_html_element::span(['class' => "small"], we_html_element::b(g_l('weClass', '[template]') . ':')) . we_html_element::br() . self::getTemplateSelector($this->tmplSelectName . "_" . $this->nr, $nr, $path) . '</td><td style="vertical-align:bottom">' . $but . '</td></tr>';
				}
				return '<tr><td></td><td class="' . $this->css . '">' . we_html_element::span(['class' => "small"], we_html_element::b(g_l('weClass', '[template]') . ':')) . we_html_element::br() . $path . we_html_element::hidden($this->tmplSelectName . "_" . $this->nr, (string) $nr) . '" /></td><td style="vertical-align:bottom">' . $but . '</td></tr>';
			default:
				return '';
		}
	}

}
