<?php
declare(strict_types=1);

/**
 * webEdition CMS
 *
 * $Rev: 94db540e0f59 $
 * $Author: Marc Kraemer <marc.kraemer@aldente-it.de> $
 * $Date: 2020-09-03 10:29 +0200 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
class we_dialog_customerEIWizard extends we_dialog_base{
	const CSV_DELIMITER = ';';
	const CSV_ENCLOSE = '';
	const CSV_LINEEND = 'windows';
	const THE_CHARSET = 'UTF-8';
	const SELECTION_MANUAL = 'manual';
	const SELECTION_FILTER = 'filter';
	const EXPORT_SERVER = 'server';
	const EXPORT_LOCAL = 'local';
	const ART_IMPORT = 1;
	const ART_EXPORT = 2;

	/** @var int */
	private $step = 0;

	/** @var int */
	private $mode = self::ART_IMPORT;

	public function __construct(){
		parent::__construct();
	}

	public function getHTML(int $mode = 0, int $step = 0): string{
		$this->mode = $mode;
		$this->step = max(1, $step);
		return $this->getHTMLFrameset();
	}

	protected function getHTMLFrameset(): string{
		$stepCont = $this->getHTMLStep($this->step);
		return we_html_tools::getHtmlTop(g_l('modules_customer', '[export_title]'), we_html_element::jsScript(WE_JS_MODULES_DIR . 'customer/EIWizard.js', '', [
					'id' => 'loadVarEIWizard',
					'data-wizzard' => setDynamicVar([
						'art' => $this->mode,
						'type' => ''
				])]) . $this->jsCmd->getCmds(), we_html_element::body(
					['id' => 'weMainBody', 'class' => 'weDialogBody', 'onload' => 'self.focus();']
					,
		 we_html_element::form(['name' => we_base_constants::WE_FORM, 'method' => 'post'],
												 we_html_element::exIFrame('edbody', $stepCont, 'background-color:white;position:absolute;top:0;bottom:45px;left:0;right:0;') .
						we_html_element::exIFrame('footer', self::getHTMLFooter($this->mode, $this->step), 'position:absolute;height:45px;bottom:0;left:0;right:0;overflow: hidden;padding-top:2ex;background-color:grey;')
					)
		));
	}

	private function getHTMLStep(int $step = 0): string{
		switch($this->mode){
			case self::ART_EXPORT:
				switch($step){
					case 0:
					case 1:
						return $this->getHTMLExportStep1();
					case 2:
						return $this->getHTMLExportStep2();
					case 3:
						return $this->getHTMLExportStep3();
					case 4:
						return $this->getHTMLExportStep4();
					case 5:
						$this->getHTMLExportStep5();
						return '';
					default:
						return '';
				}
			case self::ART_IMPORT:
				switch($step){
					case 0:
					case 1:
						return $this->getHTMLImportStep1();
					case 2:
						return $this->getHTMLImportStep2();
					case 3:
						return $this->getHTMLImportStep3();
					case 4:
						return $this->getHTMLImportStep4();
					case 5:
						return $this->getHTMLImportStep5();
					default:
						return '';
				}
			default:
				return '';
		}
	}

	private function getHTMLExportStep1(): string{
		$type = we_base_request::string('type', we_exim_ExIm::TYPE_XML);

		$parts = (new we_html_multiIconBoxContent())
			->add(new we_html_multiIconBoxContentLine(g_l('modules_customer', '[generic_export]'), implode(we_html_element::br(), [
					we_html_forms::radiobuttonV(we_exim_ExIm::TYPE_XML, $type, 'type', g_l('modules_customer', '[gxml_export]'), true, '', '', false, g_l('modules_customer', '[txt_gxml_export]'), we_html_tools::TYPE_HELP),
					we_html_forms::radiobuttonV(we_exim_ExIm::TYPE_CSV, $type, 'type', g_l('modules_customer', '[csv_export]'), true, '', '', false, g_l('modules_customer', '[txt_csv_export]'), we_html_tools::TYPE_HELP),
					we_html_forms::radiobuttonV(we_exim_ExIm::TYPE_ODS, $type, 'type', g_l('modules_customer', '[ods_export]'), true, '', '', false, g_l('modules_customer', '[txt_ods_export]'), we_html_tools::TYPE_HELP)])
		));

		return self::getHiddensExport($this->step) .
			(new we_html_multiIconBox($parts))->marginLeft()->headline(g_l('modules_customer', '[export_step1]'))->get()
		;
	}

	private function getHTMLExportStep2(): string{
		$selection = we_base_request::string('selection', self::SELECTION_FILTER);

		$parts = (new we_html_multiIconBoxContent())
			->add((new we_html_multiIconBoxContentLine('', implode(we_html_element::br(), [
					we_html_forms::radiobuttonV(self::SELECTION_FILTER, $selection, 'selection', g_l('modules_customer', '[filter_selection]')),
					we_html_tools::formElementTable(g_l('modules_customer', '[customer]'), $this->getHTMLCustomer()),
					we_html_forms::radiobuttonV(self::SELECTION_MANUAL, $selection, 'selection', g_l('modules_customer', '[manual_selection]')),
					$this->getHTMLCustomerFilter()
				])))->noline());

		return self::getHiddensExport($this->step) .
			(new we_html_multiIconBox($parts))->marginLeft()->headline(g_l('modules_customer', '[export_step2]'))->get()
		;
	}

	private function getHTMLExportStep3(): string{
		//set defaults
		$type = we_base_request::string('type', we_exim_ExIm::TYPE_XML);
		$filename = we_base_request::_(we_base_request::FILE, 'filename', 'weExport_' . date('Y-m-d_H:i'));
		$export_to = we_base_request::string('export_to', self::EXPORT_SERVER);
		$path = we_base_request::_(we_base_request::FILE, 'path', '/');
		//$cdata = we_base_request::bool('cdata', true);

		$csv_delimiter = we_base_request::_(we_base_request::RAW_CHECKED, 'csv_delimiter', self::CSV_DELIMITER);
		$csv_enclose = we_base_request::_(we_base_request::RAW_CHECKED, 'csv_enclose', self::CSV_ENCLOSE);
		$csv_lineend = we_base_request::string('csv_lineend', self::CSV_LINEEND);
		$csv_fieldnames = we_base_request::bool('csv_fieldnames', true);


		$customers = $this->getExportCustomers();
		$parts = new we_html_multiIconBoxContent();
		if($customers){
			//set variables in top frame
			$parts->add((new we_html_multiIconBoxContentLine(g_l('modules_customer', '[filename]'), we_html_tools::textInput('filename', $filename)))->space(we_html_multiIconBox::SPACE_MED2));

			switch($type){
				case we_exim_ExIm::TYPE_ODS:
					break;
				/* case we_exim_ExIm::TYPE_XML:
				  $parts->add((new we_html_multiIconBoxContentLine(g_l('modules_customer', '[cdata]'), implode(we_html_element::br(), [
				  we_html_forms::radiobutton('1', $cdata, 'cdata', g_l('modules_customer', '[export_xml_cdata]'), true),
				  we_html_forms::radiobutton('0', !$cdata, 'cdata', g_l('modules_customer', '[export_xml_entities]'), true)
				  ]
				  )))->space(we_html_multiIconBox::SPACE_MED2));
				  break; */
				case we_exim_ExIm::TYPE_CSV:
					$file_encoding = (new we_html_select(['name' => 'csv_lineend', 'class' => 'defaultfont', 'style' => 'width: 254px;']))
						->addOptions(we_exim_ExportCSV::getCSVLineEnds())
						->selectOption($csv_lineend);

					$fileformattable = implode(we_html_element::br(), [
						g_l('modules_customer', '[csv_lineend]') . '<br/>' . $file_encoding->getHtml(),
						$this->getHTMLChooser('csv_delimiter', $csv_delimiter, we_exim_ExportCSV::getCSVDelimeters(), g_l('modules_customer', '[csv_delimiter]')),
						$this->getHTMLChooser('csv_enclose', $csv_enclose, we_exim_ExportCSV::getCSVQuotes(), g_l('modules_customer', '[csv_enclose]')),
						we_html_forms::checkbox('1', $csv_fieldnames, 'csv_fieldnames', g_l('modules_customer', '[csv_fieldnames]'))
					]);

					$parts->add((new we_html_multiIconBoxContentLine(g_l('modules_customer', '[csv_params]'), $fileformattable))->space(we_html_multiIconBox::SPACE_MED2));
			}

			$parts
				->add((new we_html_multiIconBoxContentLine(g_l('modules_customer', '[export_to]'), ''))->noline())
				->add((new we_html_multiIconBoxContentLine(we_html_element::div(['class' => 'default'], we_html_forms::radiobuttonV(self::EXPORT_SERVER, $export_to, 'export_to', g_l('modules_customer', '[export_to_server]'))),
																																																																																					we_html_element::br() .
						we_html_tools::formElementTable(g_l('modules_customer', '[path]'), $this->formFileChooser('path', $path, '', we_base_ContentTypes::FOLDER_COMMON))))
					->space(we_html_multiIconBox::SPACE_MED2)
					->noline()
				)
				->add((new we_html_multiIconBoxContentLine(we_html_forms::radiobuttonV(self::EXPORT_LOCAL, $export_to, 'export_to', g_l('modules_customer', '[export_to_local]')), ''))
					->space(we_html_multiIconBox::SPACE_MED2)
					->noline()
			);
		} else {
			$parts->add((new we_html_multiIconBoxContentLine('Fehler', we_html_element::b(g_l('wysiwyg', '[selection_invalid]'))))
					->space(we_html_multiIconBox::SPACE_MED2));
			$this->jsCmd->addCmd('do_back', self::ART_EXPORT);
		}

		return self::getHiddensExport($this->step) .
			(new we_html_multiIconBox($parts, 'weExportWizard'))->marginLeft()->headline(g_l('modules_customer', '[export_step3]'))->get()
		;
	}

	private function getHTMLExportStep4(): string{
		$path = urldecode(we_base_request::_(we_base_request::FILE, 'path'));
		$filename = urldecode(we_base_request::_(we_base_request::FILE, 'filename'));
		switch(we_base_request::string('export_to', self::EXPORT_SERVER)){
			case self::EXPORT_LOCAL:
				we_base_file::insertIntoCleanUp(TEMP_DIR . '/' . $filename);
				$message = we_html_element::span(['class' => 'defaultfont'], g_l('modules_customer', '[export_finished]') . we_html_element::br(2) .
						g_l('modules_customer', '[download_starting]', false, true) .
						we_html_element::a(['href' => WEBEDITION_DIR . 'we_cmd.php?we_cmd[0]=we_showMod&mod=customer&pnt=eibody&art=' . self::ART_EXPORT . '&step=5&exportfile=' . $filename,
							'download' => $filename], g_l('modules_customer', '[download]'))
				);
				$this->jsCmd->addCmd('reload_frame', [
					'cmd' => '',
					'step' => 5,
					'exportfile' => $filename
				]);
				return we_html_tools::getHtmlTop(g_l('modules_customer', '[export_title]'), $this->jsCmd->getCmds() .
						we_html_element::meta(['http-equiv' => 'refresh', 'content' => '2; url=' . WEBEDITION_DIR . 'we_cmd.php?we_cmd[0]=we_showMod&mod=customer&pnt=eibody&art=' . self::ART_EXPORT . '&step=5&exportfile=' . $filename]), we_html_element::body([
							'class' => 'weDialogBody'], we_html_tools::dialogLayout($message, g_l('modules_customer', '[export_step4]'))
						)
				);
			default:
			case self::EXPORT_SERVER:
				$message = we_html_element::span(['class' => 'defaultfont'], g_l('modules_customer', '[export_finished]') . we_html_element::br(2) .
						g_l('modules_customer', '[server_finished]') . '<br/>' .
						rtrim($path, '/') . '/' . $filename
				);
				$this->jsCmd->addCmd('reload_frame', ['cmd' => '', 'step' => 5]);
				return we_html_tools::dialogLayout($message, g_l('modules_customer', '[export_step4]'));
		}
	}

	private function getHTMLExportStep5(): string{
		if(($filename = we_base_request::_(we_base_request::FILE, 'exportfile'))){
			if(file_exists(TEMP_PATH . $filename) // Does file exist?
				&& !preg_match('%p?html?%i', $filename) && stripos($filename, 'inc') === false && !preg_match('%php3?%i', $filename)){ // Security check
				session_write_close();
				$size = filesize(TEMP_PATH . $filename);

				header('Pragma: public');
				header('Expires: 0');
				header('Cache-control: private, max-age=0, must-revalidate');
				header('Content-Type: application/octet-stream');
				header('Content-Disposition: attachment; filename="' . trim(htmlentities($filename)) . '"');
				header('Content-Description: Customer-Export');
				header('Content-Length: ' . $size);

				readfile(TEMP_PATH . $filename);

				exit;
			}
		}
		header('Location: ' . WEBEDITION_DIR . 'we_cmd.php?we_cmd[0]=we_showMod&mod=customer&pnt=cmd&step=99&error=download_failed');
		exit;
	}

	private static function getHiddensImport(int $step): string{
		$filename = we_base_request::_(we_base_request::FILE, 'filename');
		$import_from = we_base_request::string('import_from', self::EXPORT_SERVER);
		$type = we_base_request::string('type', we_exim_ExIm::TYPE_XML);
		$xml_from = we_base_request::_(we_base_request::RAW, 'xml_from', 0);
		$xml_to = we_base_request::_(we_base_request::RAW, 'xml_to', 1);
		$dataset = we_base_request::_(we_base_request::RAW, 'dataset', '');
		$csv_delimiter = we_base_request::_(we_base_request::RAW_CHECKED, 'csv_delimiter', self::CSV_DELIMITER);
		$csv_enclose = we_base_request::_(we_base_request::RAW_CHECKED, 'csv_enclose', self::CSV_ENCLOSE);
		$csv_lineend = we_base_request::string('csv_lineend', self::CSV_LINEEND);
		$the_charset = we_base_request::string('the_charset', self::THE_CHARSET);

		$csv_fieldnames = we_base_request::bool('csv_fieldnames');

		$source = we_base_request::_(we_base_request::FILE, 'source', '/');
		$hidden = [
			'pnt' => 'eibody',
			'step' => $step,
			'art' => self::ART_IMPORT,
		];
		switch($step){
			case 1:
				return we_html_element::hiddens(array_merge($hidden, [
						($filename ? 'filename' : '') => $filename,
						'source' => $source,
						'import_from' => $import_from,
						'xml_from' => $xml_from,
						'xml_to' => $xml_to,
						'dataset' => $dataset,
						'csv_delimiter' => $csv_delimiter,
						'csv_enclose' => $csv_enclose,
						'csv_lineend' => $csv_lineend,
						'the_charset' => $the_charset,
						'csv_fieldnames' => (string) $csv_fieldnames
				]));

			case 2:
				return we_html_element::hiddens(array_merge($hidden, [
						'type' => $type,
						($filename ? 'filename' : '') => $filename,
						'xml_from' => $xml_from,
						'xml_to' => $xml_to,
						'dataset' => $dataset,
						'csv_delimiter' => $csv_delimiter,
						'csv_enclose' => $csv_enclose,
						'csv_lineend' => $csv_lineend,
						'the_charset' => $the_charset,
						'csv_fieldnames' => (string) $csv_fieldnames
				]));

			case 3:
			case 99:
				return we_html_element::hiddens(array_merge($hidden, [
						'type' => $type,
						'source' => $source,
						'step' => 3,
						($filename ? 'filename' : '') => $filename,
						'import_from' => $import_from,
						'dataset' => $dataset]));

			case 4:
				return we_html_element::hiddens(array_merge($hidden, [
						'type' => $type,
						'source' => $source,
						'import_from' => $import_from,
						'dataset' => $dataset,
						'xml_from' => $xml_from,
						'xml_to' => $xml_to,
						'csv_delimiter' => $csv_delimiter,
						'csv_lineend' => $csv_lineend,
						'the_charset' => $the_charset,
						'csv_fieldnames' => (string) $csv_fieldnames,
						'cmd' => self::ART_IMPORT,
						($filename ? 'filename' : '') => $filename,
						'csv_enclose' => ($csv_enclose === '"' ? '"' : $csv_enclose)
				]));
		}
		return '';
	}

	private static function getHiddensExport(int $step): string{
		$type = we_base_request::string('type', we_exim_ExIm::TYPE_XML);
		$selection = we_base_request::string('selection', self::SELECTION_FILTER);
		$export_to = we_base_request::string('export_to', self::EXPORT_SERVER);
		$path = urldecode(we_base_request::_(we_base_request::FILE, 'path', '/'));
		$filename = we_base_request::_(we_base_request::FILE, 'filename');
		//$cdata = we_base_request::int('cdata', 1);

		$customers = implode(',', we_base_request::intList('customers', []));

		$csv_delimiter = we_base_request::_(we_base_request::RAW_CHECKED, 'csv_delimiter', self::CSV_DELIMITER);
		$csv_enclose = we_base_request::_(we_base_request::RAW_CHECKED, 'csv_enclose', self::CSV_ENCLOSE);
		$csv_lineend = we_base_request::string('csv_lineend', self::CSV_LINEEND);
		$csv_fieldnames = we_base_request::bool('csv_fieldnames', true);

		$filter_count = we_base_request::int('filter_count', 0);
		$filter = '';
		$fields_names = ['fieldname', 'operator', 'fieldvalue', 'logic'];
		for($i = 0; $i < $filter_count; $i++){
			foreach($fields_names as $field){
				$varname = 'filter_' . $field . '_' . $i;
				if(($f = we_base_request::string($varname, we_base_request::NOT_VALID_S)) !== we_base_request::NOT_VALID_S){
					$filter .= we_html_element::hidden($varname, $f);
				}
			}
		}
		$hidden = [
			'pnt' => 'eibody',
			'step' => $step,
			'art' => self::ART_EXPORT,
			'type' => $type,
			'selection' => $selection,
		];
		switch($step){
			case 1:
				return we_html_element::hiddens(array_merge($hidden, [
						'export_to' => $export_to,
						'path' => $path,
						//'cdata' => $cdata,
						'customers' => $customers,
						($filename ? 'filename' : '') => $filename,
						'csv_delimiter' => $csv_delimiter,
						'csv_enclose' => $csv_enclose,
						'csv_lineend' => $csv_lineend,
						'csv_fieldnames' => $csv_fieldnames,
						'filter_count' => $filter_count])) .
					$filter;

			case 2:
				return we_html_element::hiddens(array_merge($hidden, [
						'export_to' => $export_to,
						'path' => $path,
						//'cdata' => $cdata,
						'customers' => $customers,
						($filename ? 'filename' : '') => $filename,
						'csv_delimiter' => $csv_delimiter,
						'csv_enclose' => $csv_enclose,
						'csv_lineend' => $csv_lineend,
						'csv_fieldnames' => $csv_fieldnames,
						'filter_count' => $filter_count]));

			case 3:
				return we_html_element::hiddens(array_merge($hidden, [
						'customers' => $customers,
						'filter_count' => $filter_count,
						'cmd' => self::ART_EXPORT
					])) .
					$filter;

			case 4:
				return we_html_element::hiddens(array_merge($hidden, [
						'export_to' => $export_to,
						'path' => $path,
						//'cdata' => $cdata,
						'customers' => $customers,
						($filename ? 'filename' : '') => $filename,
						'csv_delimiter' => $csv_delimiter,
						'csv_enclose' => $csv_enclose,
						'csv_lineend' => $csv_lineend,
						'csv_fieldnames' => $csv_fieldnames,
						'filter_count' => $filter_count,
						'cmd' => self::ART_EXPORT
					])) .
					$filter;
		}
		return '';
	}

	private function getHTMLImportStep1(): string{
		$type = we_base_request::string('type', we_exim_ExIm::TYPE_XML);

		$generic = new we_html_table(['class' => 'default withSpace'], 2, 1);
		$generic->setCol(0, 0, we_html_forms::radiobuttonV(we_exim_ExIm::TYPE_XML, $type, 'type', g_l('modules_customer', '[gxml_import]'), true, '', '', false, g_l('modules_customer', '[txt_gxml_import]'), 0, 430));
		$generic->setCol(1, 0, we_html_forms::radiobuttonV(we_exim_ExIm::TYPE_CSV, $type, 'type', g_l('modules_customer', '[csv_import]'), true, '', '', false, g_l('modules_customer', '[txt_csv_import]'), 0, 430));

		$parts = (new we_html_multiIconBoxContent())
			->add((new we_html_multiIconBoxContentLine('', $generic->getHTML()))->space(we_html_multiIconBox::SPACE_SMALL)->noline()
		);

		return self::getHiddensImport($this->step) .
			(new we_html_multiIconBox($parts))->marginLeft()->headline(g_l('modules_customer', '[import_step1]'))->get()
		;
	}

	private function getHTMLImportStep2(): string{
		$import_from = we_base_request::string('import_from', self::EXPORT_SERVER);
		$source = we_base_request::_(we_base_request::FILE, 'source', '/');
		$type = we_base_request::string('type', '');

		$fileUploader = new we_fileupload_ui_base('upload', $type === 'XML' ? [we_base_ContentTypes::XML] : [we_base_ContentTypes::TEXT]);
		$fileUploader->setCmd('cmdFileselectOnclick', 'set_radio_importFrom,,1');
		$fileUploader->setGenericFileName(TEMP_DIR . we_fileupload::REPLACE_BY_UNIQUEID . ($type === we_exim_ExIm::TYPE_CSV ? '.csv' : '.xml'));

		$parts = new we_html_multiIconBoxContent();
		$js = $fileUploader->getJs();
		$css = $fileUploader->getCss();

		$table = (new we_html_table(['class' => 'default withSpace'], 2, 2))
			->setCol(0, 0, we_html_forms::radiobuttonV(self::EXPORT_SERVER, $import_from, 'import_from', g_l('modules_customer', '[server_import]'), true, 'defaultfont'), [
				'colspan' => '2'])
			->setCol(1, 1, $this->formFileChooser('source', $source, 'set_radio_importFrom,0', ($type == we_exim_ExIm::TYPE_XML ? we_base_ContentTypes::XML : '')));

		$parts->add((new we_html_multiIconBoxContentLine('', $table->getHtml()))->space(we_html_multiIconBox::SPACE_SMALL)->noline());

		//upload table
		$tmptable = (new we_html_table(['class' => 'default withSpace'], 1, 1))
			->setCol(0, 0, $fileUploader->getHTML(), ['style' => 'vertical-align:middle;'])
			->getHtml();

		$table = (new we_html_table(['class' => 'default withSpace'], 2, 2))
			->setCol(0, 0, we_html_forms::radiobuttonV(self::EXPORT_LOCAL, $import_from, 'import_from', g_l('modules_customer', '[upload_import]'), true, 'defaultfont'), [
				'colspan' => '2'])
			->setCol(1, 1, $tmptable);

		$parts->add((new we_html_multiIconBoxContentLine('', $table->getHTML()))->space(we_html_multiIconBox::SPACE_MED)->noline());

		return self::getHiddensImport($this->step) . $css . $js .
			(new we_html_multiIconBox($parts))->marginLeft()->headline(g_l('modules_customer', '[import_step2]'))->get()
		;
	}

	private function getHTMLImportStep3(): string{
		$import_from = we_base_request::string('import_from', self::EXPORT_SERVER);
		$source = we_base_request::_(we_base_request::FILE, 'source', '/');
		$type = we_base_request::string('type', '');

		if($import_from == self::EXPORT_LOCAL){
			$fileUploader = new we_fileupload_resp_base();
			try{
				$filename = $fileUploader->commitUploadedFile();
			} catch (we_exception_file $e){
				$filename = '';
			}
		} else {
			$filename = $source;
		}
		$filesource = $filename ? WE_DOCUMENT_ROOT . $filename : '';

		$parts = new we_html_multiIconBoxContent();

		switch(is_file($filesource) && is_readable($filesource) ? $type : 'INVALID_FILE'){
			case 'INVALID_FILE':
				$this->step = 99;
				$parts->add((new we_html_multiIconBoxContentLine('', we_html_tools::alertAttentionBox(g_l('modules_customer', '[missing_filesource]'), we_html_tools::TYPE_ALERT, 570)))->noline());
				break;
			case we_exim_ExIm::TYPE_CSV:
				$line = we_base_file::loadLine($filesource, 0, 4096);
				$charset = mb_detect_encoding($line, ['UTF-8', 'ISO-8859-15', 'ISO-8859-1'], true) ?: '';
				$charCount = count_chars($line, 0);

				$csv_delimiters = we_exim_ExportCSV::getCSVDelimeters();
				$csv_encloses = we_exim_ExportCSV::getCSVQuotes();
				$max = 0;
				$csv_delimiter = '';
				foreach(array_keys($csv_delimiters) as $char){
					'@phan-var string $char';
					$ord = ord($char);
					if($charCount[$ord] > $max){
						$csv_delimiter = $char;
						$max = $charCount[$ord];
					}
				}
				//leave max
				$csv_enclose = '';
				foreach(array_keys($csv_encloses) as $char){
					'@phan-var string $char';
					$ord = ord($char);
					if($charCount[$ord] > $max){
						$csv_enclose = $char;
						$max = $charCount[$ord];
					}
				}
				$r = $charCount[ord("\r")];
				$n = $charCount[ord("\n")];
				$csv_lineend = ($r > 0 && $r == $n ? we_exim_ExportCSV::LINEEND_WINDOWS : ($r > 0 ? we_exim_ExportCSV::LINEEND_MAC : we_exim_ExportCSV::LINEEND_UNIX));
				$csv_fieldnames = (strpos($line, 'Username') !== false);


				$file_encoding = (new we_html_select(['name' => 'csv_lineend', 'class' => 'defaultfont', 'style' => 'width: 254px;']))
					->addOptions(we_exim_ExportCSV::getCSVLineEnds())
					->selectOption($csv_lineend);

				$importCharset = we_html_tools::textInput('the_charset', ($charset === 'ASCII' ? 'ISO8859-1' : $charset), 255, [], 'text', '100px');
				$importCharsetChooser = we_html_tools::select('ImportCharsetSelect', we_base_charsetHandler::inst()->getCharsetsForTagWizzard(), 1, ($charset === 'ASCII' ? 'ISO8859-1' : $charset), false, [
						'onchange' => "we_cmd('selectCharset_onchange', {select: this})"], true, 160, 'defaultfont', false);
				$import_Charset = '<table class="default"><tr><td>' . $importCharset . '</td><td>' . $importCharsetChooser . '</td></tr></table>';

				$fileformattable = (new we_html_table(['style' => 'margin-top:10px;'], 5, 1))
					->setCol(0, 0, g_l('modules_customer', '[csv_lineend]') . we_html_element::br() . $file_encoding->getHtml())
					->setCol(1, 0, g_l('modules_customer', '[import_charset]') . we_html_element::br() . $import_Charset)
					->setCol(2, 0, $this->getHTMLChooser('csv_delimiter', $csv_delimiter, $csv_delimiters, g_l('modules_customer', '[csv_delimiter]')))
					->setCol(3, 0, $this->getHTMLChooser('csv_enclose', $csv_enclose, $csv_encloses, g_l('modules_customer', '[csv_enclose]')))
					->setCol(4, 0, we_html_forms::checkbox('1', $csv_fieldnames, 'csv_fieldnames', g_l('modules_customer', '[csv_fieldnames]')));

				$parts->add((new we_html_multiIconBoxContentLine(g_l('modules_customer', '[csv_params]'), $fileformattable->getHtml()))->space(we_html_multiIconBox::SPACE_MED2));
				break;
			case we_exim_ExIm::TYPE_XML:
				//invoke parser
				$xp = new we_xml_parser($filesource);
				$xmlWellFormed = ($xp->parseError === '');
				$hasChildNode = false;
				if($xmlWellFormed){
					// Node-set with paths to the child nodes.
					/* $node_set = */$xp->evaluate("*/child::*");
					$children = $xp->nodes[$xp->root]['children'];

					$recs = [];
					foreach($children as $key => $value){
						$flag = true;
						for($k = 1; $k < ($value + 1); $k++){
							if(!$xp->hasChildNodes($xp->root . '/' . $key . '[' . $k . ']')){
								$flag = false;
							}
						}
						if($flag){
							$recs[$key] = $value;
						}
					}
					$isSingleNode = (count($recs) === 1);
					$hasChildNode = (!empty($recs));
					if($hasChildNode){
						$rcdSelect = new we_html_select([
							'name' => 'we_select',
							'class' => 'defaultfont',
							(($isSingleNode) ? 'disabled' : 'style') => '',
							'onchange' => 'selectWeSelect_doOnselect(this)']
						);
						$optid = 0;
						$firstItem = $firstOptVal = '';
						foreach($recs as $value => $text){
							'@phan-var string $text';
							if($optid == 0){
								$firstItem = $value;
								$firstOptVal = $text;
							}
							$rcdSelect->addOption($text, (string) $value);
							if(isset($recs['rcd']) && $text == $recs['rcd']){
								$rcdSelect->selectOption($value);
							}
							$optid++;
						}

						$tblSelect = (new we_html_table([], 1, 7))
							->setCol(0, 1, $rcdSelect->getHtml())
							->setColAttributes(0, 2, ['width' => '20'])
							->setCol(0, 3, g_l('modules_customer', '[num_data_sets]'))
							->setCol(0, 4, we_html_tools::textInput('xml_from', '1', 5, [], 'text', '30px', false, ($isSingleNode && ($firstOptVal == 1))))
							->setCol(0, 5, g_l('modules_customer', '[to]'))
							->setCol(0, 6, we_html_tools::textInput('xml_to', $firstOptVal, 5, [], 'text', '30px', false, ($isSingleNode && ($firstOptVal == 1))));

						$tblFrame = (new we_html_table([], 3, 2))
							->setCol(0, 0, ($isSingleNode) ? we_html_tools::alertAttentionBox(g_l('modules_customer', '[well_formed]') . ' ' . g_l('modules_customer', '[select_elements]'), we_html_tools::TYPE_INFO, 570) :
								we_html_tools::alertAttentionBox(g_l('modules_customer', '[xml_valid_1]') . ' ' . $optid . ' ' . g_l('modules_customer', '[xml_valid_m2]'), we_html_tools::TYPE_INFO, 570), [
								'colspan' => '2', 'class' => 'defaultfont'])
							->setColAttributes(1, 0, ['colspan' => '2'])
							->setCol(2, 1, $tblSelect->getHtml());

						$_REQUEST['dataset'] = $firstItem;
						$parts->add((new we_html_multiIconBoxContentLine('', $tblFrame->getHtml()))->noline());
					}
				}
				if(!($xmlWellFormed && $hasChildNode)){
					$parts->add((new we_html_multiIconBoxContentLine('', we_html_tools::alertAttentionBox(g_l('modules_customer', (!$xmlWellFormed) ? '[not_well_formed]' : '[missing_child_node]'), we_html_tools::TYPE_ALERT, 570)))->noline());
					$this->step = 99;
				}
				break;
		}

		$_REQUEST['filename'] = $filename;
		return self::getHiddensImport($this->step) .
			(new we_html_multiIconBox($parts))->marginLeft()->headline(g_l('modules_customer', '[import_step3]'))->get()
		;
	}

	private function getHTMLImportStep4(): string{
		$filename = we_base_request::_(we_base_request::FILE, 'filename', '');
		$type = we_base_request::string('type');
		$dataset = we_base_request::_(we_base_request::RAW, 'dataset', '');
		$csv_delimiter = we_base_request::_(we_base_request::RAW_CHECKED, 'csv_delimiter', self::CSV_DELIMITER);
		$csv_enclose = we_base_request::_(we_base_request::RAW_CHECKED, 'csv_enclose', self::CSV_ENCLOSE);
		$csv_lineend = we_base_request::string('csv_lineend', self::CSV_LINEEND);
		$the_charset = we_base_request::string('the_charset', self::THE_CHARSET);
		$csv_fieldnames = we_base_request::bool('csv_fieldnames');
		$same = we_base_request::string('same', we_exim_ExIm::COLLISION_RENAME);

		$field_mappings = we_base_request::_(we_base_request::RAW, 'field_mappings', '');

		switch($type){
			case we_exim_ExIm::TYPE_CSV:
				$arrgs = [
					'delimiter' => $csv_delimiter,
					'enclose' => $csv_enclose,
					'lineend' => $csv_lineend,
					'fieldnames' => $csv_fieldnames,
					'charset' => $the_charset,
				];
				break;
			default:
				$arrgs = ['dataset' => $dataset];
		}

		$nodes = we_customer_EI::getDataset($type, $filename, $arrgs);
		$records = we_customer_EI::getCustomersFieldset();

		$tableheader = ($type == we_exim_ExIm::TYPE_XML ?
			[g_l('modules_customer', '[we_flds]'), g_l('modules_customer', '[rcd_flds]'), g_l('import', '[attributes]')] :
			[g_l('modules_customer', '[we_flds]'), g_l('modules_customer', '[rcd_flds]')]
			);

		$rows = [];

		foreach($records as $record){
			$we_fields = new we_html_select([
				'name' => 'field_mappings[' . $record . ']',
				'class' => 'defaultfont',
				'onclick' => '',
				'style' => '']
			);

			$we_fields->addOption('', g_l('modules_customer', '[any]'));

			foreach(array_keys($nodes) as $node){
				'@phan-var string $node';
				$we_fields->addOption(htmlspecialchars(str_replace(' ', '', $node)), htmlspecialchars($node));
				if(isset($field_mappings[$record])){
					if($node == $field_mappings[$record]){
						$we_fields->selectOption($node);
					}
				} else {
					if($node == $record){
						$we_fields->selectOption($node);
					}
				}
			}
			$rows[] = [$record, $we_fields->getHTML()];
		}

		$table = (new we_html_table(['class' => 'default'], 4, 1))
			->setCol(0, 0, we_html_forms::radiobuttonV(we_exim_ExIm::COLLISION_RENAME, $same, 'same', g_l('modules_customer', '[same_rename]'), true, 'defaultfont', ''))
			->setCol(1, 0, we_html_forms::radiobuttonV(we_exim_ExIm::COLLISION_OVERWRITE, $same, 'same', g_l('modules_customer', '[same_overwrite]'), true, 'defaultfont', ''))
			->setCol(2, 0, we_html_forms::radiobuttonV(we_exim_ExIm::COLLISION_SKIP, $same, 'same', g_l('modules_customer', '[same_skip]'), true, 'defaultfont', ''));

		$parts = (new we_html_multiIconBoxContent())
			->add(new we_html_multiIconBoxContentLine(g_l('modules_customer', '[same_names]'), $table->getHtml()))
			->add((new we_html_multiIconBoxContentLine(g_l('modules_customer', '[import_step4]'),
																									we_html_element::br() . we_html_tools::dialogBorder(510, $rows, $tableheader, 'defaultfont')))->space(we_html_multiIconBox::SPACE_MED2));


		return self::getHiddensImport($this->step) .
			(new we_html_multiIconBox($parts, 'xml'))->marginLeft()->headline(g_l('modules_customer', '[import_step4]'))->get()
		;
	}

	private function getHTMLImportStep5(): string{
		$tmpdir = we_base_request::_(we_base_request::FILE, 'tmpdir');
		$impno = we_base_request::int('impno', 0);

		$table = new we_html_table([], 3, 1);
		$table->setCol(0, 0, sprintf(g_l('modules_customer', '[import_finished_desc]'), $impno));

		if($tmpdir && is_file(TEMP_PATH . $tmpdir . '/' . $tmpdir . '.log') && is_readable(TEMP_PATH . $tmpdir . '/' . $tmpdir . '.log')){
			$log = we_base_file::load(TEMP_PATH . $tmpdir . '/' . $tmpdir . '.log', 'rb');
			if($log){
				$table
					->setCol(1, 0, we_html_tools::alertAttentionBox(g_l('modules_customer', '[show_log]'), we_html_tools::TYPE_ALERT, 550))
					->setCol(2, 0, we_html_element::textArea(['name' => 'log', 'rows' => '15', 'cols' => '15', 'style' => 'width: 550px; height: 200px;'], htmlspecialchars($log)));
				we_base_file::deleteLocalFile(TEMP_PATH . $tmpdir . '/' . $tmpdir . '.log');
			}
		}
		$parts = (new we_html_multiIconBoxContent())
			->add((new we_html_multiIconBoxContentLine('', $table->getHtml()))->space(we_html_multiIconBox::SPACE_SMALL));

		if(is_dir(TEMP_PATH . $tmpdir)){
			rmdir(TEMP_PATH . $tmpdir);
		}

		return (new we_html_multiIconBox($parts))->marginLeft()->headline(g_l('modules_customer', '[import_step5]'))->get()
		;
	}

	private static function getHTMLFooter(int $mode, int $step): string{
		return ($mode == self::ART_EXPORT ? self::getHTMLExportFooter($step) : self::getHTMLImportFooter($step));
	}

	private static function getHTMLExportFooter(int $step = 1): string{
		$content = new we_html_table(['class' => 'default', 'width' => '575', 'style' => 'text-align:right'], 1, 2);

		switch($step){
			case 0:
			case 1:
			case 4:
				$buttons = we_html_button::position_yes_no_cancel(
						(new we_html_button(we_html_button::BACK))->disable(true)->get() .
						(new we_html_button(we_html_button::NEXT))->value('step', (string) ($step + 1))->get(), we_html_button::create_button(we_html_button::CANCEL, "javascript:WE().util.dialogTop(window).doClose();")
				);
				break;
			case 5:
				$buttons = we_html_button::position_yes_no_cancel(
						(new we_html_button(we_html_button::BACK))->disable(true)->get() .
						(new we_html_button(we_html_button::NEXT))->disable(true)->get(), we_html_button::create_button(we_html_button::CANCEL, "javascript:WE().util.dialogTop(window).doClose();")
				);
				break;
			default:
				$buttons = we_html_button::position_yes_no_cancel(
						(new we_html_button(we_html_button::BACK))->value('step', (string) ($step - 1 ))->get() .
						(new we_html_button(we_html_button::NEXT))->value('step', (string) ($step + 1))->get(), we_html_button::create_button(we_html_button::CANCEL, "javascript:WE().util.dialogTop(window).doClose();")
				);
		}
		$content->setCol(0, 1, $buttons, ['style' => 'text-align:right']);

		return $content->getHtml();
	}

	private static function getHTMLImportFooter(int $step = 1): string{
		$content = new we_html_table(['class' => 'default', 'width' => '575', 'style' => "text-align:right"], 1, 2);
		switch($step){
			case 0:
			case 1:
				$buttons = we_html_button::position_yes_no_cancel(
						(new we_html_button(we_html_button::BACK))->disable(true)->get() .
						(new we_html_button(we_html_button::NEXT))->value('step', (string) ($step + 1))->get(), we_html_button::create_button(we_html_button::CANCEL, 'javascript:WE().util.dialogTop(window).doClose();')
				);
				break;
			case 2:
				$buttons = we_html_button::position_yes_no_cancel(
						(new we_html_button(we_html_button::BACK))->value('step', (string) ($step - 1 ))->get() .
						(new we_html_button(we_html_button::NEXT))->value('step', (string) ($step + 1))->get(), we_html_button::create_button(we_html_button::CANCEL, "javascript:we_cmd('uploader_cancel');")
				);
				break;
			case 5:
				$buttons = we_html_button::position_yes_no_cancel(
						(new we_html_button(we_html_button::BACK))->disable(true)->get() .
						(new we_html_button(we_html_button::NEXT))->disable(true)->get(), we_html_button::create_button(we_html_button::CANCEL, 'javascript:WE().util.dialogTop(window).doClose();')
				);
				break;
			case 6:
				$buttons = we_html_button::position_yes_no_cancel(
						we_html_button::create_button(we_html_button::CLOSE, "javascript:WE().util.dialogTop(window).doClose();")
				);
				break;
			case 99:
				$buttons = we_html_button::position_yes_no_cancel(
						(new we_html_button(we_html_button::BACK))->value('step', '2')->get() .
						(new we_html_button(we_html_button::NEXT))->disable(true)->get(), we_html_button::create_button(we_html_button::CANCEL, "javascript:WE().util.dialogTop(window).doClose();")
				);
				break;
			default:
				$buttons = we_html_button::position_yes_no_cancel(
						we_html_button::create_button(we_html_button::BACK, "javascript:we_cmd('reload_frame',{cmd:'import_back',step:" . $step . "});") .
						we_html_button::create_button(we_html_button::NEXT, "javascript:we_cmd('load_processCmd', {cmd:'import_next'});"), we_html_button::create_button(we_html_button::CANCEL, "javascript:WE().util.dialogTop(window).doClose();")
				);
		}
		$content->setCol(0, 1, $buttons, ['style' => 'text-align:right']);

		return $content->getHtml();
	}

	private function getExportCode(): string{
		$file_format = we_base_request::string('type', we_exim_ExIm::TYPE_XML);
		$file_name = we_base_request::_(we_base_request::FILE, 'filename', date('Y-m-d'));
		$export_to = we_base_request::string('export_to', self::EXPORT_SERVER);
		$path = ($export_to == self::EXPORT_SERVER ? we_base_request::_(we_base_request::FILE, 'path', '') : rtrim(TEMP_DIR, '/'));
		//$cdata = we_base_request::int('cdata', 0);
		$csv_delimiter = we_base_request::string('csv_delimiter', '');
		$csv_enclose = we_base_request::string('csv_enclose', '');
		$csv_lineend = we_base_request::string('csv_lineend', '');
		$csv_fieldnames = we_base_request::bool('csv_fieldnames');

		$customers = $this->getExportCustomers();
		if(!$customers){

			//FIXME: add code to switch to previous page
			t_e('noting to export', $customers);
		}

		$hiddens = we_html_element::hiddens([
				'pnt' => 'eiload',
				'art' => self::ART_EXPORT,
				'customers' => implode(',', $customers),
				'file_format' => $file_format,
				'filename' => $file_name,
				'export_to' => $export_to,
				'path' => $path,
				'all' => (string) count($customers),
				'cmd' => 'do_export',
				'step' => '4'
		]);


		switch($file_format){
			/* case we_exim_ExIm::TYPE_XML:
			  $hiddens .= we_html_element::hidden('cdata', $cdata);
			  break; */
			case we_exim_ExIm::TYPE_CSV:
				$hiddens .= ($csv_enclose === '"' ?
					"<input type='hidden' name='csv_enclose' value='" . $csv_enclose . "' />" :
					we_html_element::hidden('csv_enclose', $csv_enclose)
					) .
					we_html_element::hiddens([
						'csv_delimiter' => $csv_delimiter,
						'csv_lineend' => $csv_lineend,
						'csv_fieldnames' => (string) $csv_fieldnames
				]);
		}

		return we_html_tools::getHtmlTop(g_l('modules_customer', '[export_title]'), '', we_html_element::body(['onload' => 'document.we_form.submit()'], we_html_element::form([
						'name' => we_base_constants::WE_FORM,
						'method' => 'post',
						'action' => WEBEDITION_DIR . 'we_cmd.php?we_cmd[0]=we_showMod&mod=' . we_modules_info::CUSTOMER], $hiddens)
				)
		);
	}

	private function getDoExportCode(): string{
		$customers = we_base_request::intList('customers', []);
		$file_format = we_base_request::string('file_format');
		$export_to = we_base_request::string('export_to', self::EXPORT_SERVER);
		$path = ($export_to == self::EXPORT_SERVER ? we_base_request::_(we_base_request::FILE, 'path', '') : TEMP_DIR);
		$filename = we_base_request::_(we_base_request::FILE, 'filename');
		$ext = we_exim_ExIm::getExtensionByType($file_format);
		$all = we_base_request::int('all');
		//$cdata = we_base_request::int('cdata');

		$hiddens = we_html_element::hiddens([
				'file_format' => $file_format,
				'filename' => $filename,
				'export_to' => $export_to,
				'path' => $path,
				'art' => self::ART_EXPORT,
				'pnt' => 'eiload',
		]);
		$success = false;
		switch($file_format){
			case we_exim_ExIm::TYPE_ODS:
				$success = we_customer_EI::exportCustomersToODS($customers, WE_DOCUMENT_ROOT . $path . '/' . $filename . $ext);
				break;
			case we_exim_ExIm::TYPE_XML:
				//$hiddens .= we_html_element::hidden('cdata', $cdata);
				$success = we_customer_EI::exportCustomersToXML($customers, WE_DOCUMENT_ROOT . $path . '/' . $filename . $ext);
				break;
			case we_exim_ExIm::TYPE_CSV:
				$csv_delimiter = we_base_request::_(we_base_request::RAW_CHECKED, 'csv_delimiter', '');
				$csv_enclose = we_base_request::_(we_base_request::RAW_CHECKED, 'csv_enclose', '');
				$csv_lineend = we_base_request::string('csv_lineend', '');
				$csv_fieldnames = we_base_request::bool('csv_fieldnames', true);

				$hiddens .= we_html_element::hiddens([
						'csv_enclose' => $csv_enclose,
						'csv_delimiter' => $csv_delimiter,
						'csv_lineend' => $csv_lineend]);
				$success = we_customer_EI::exportCustomersToCSV($customers, WE_DOCUMENT_ROOT . $path . '/' . $filename . $ext, $csv_enclose, $csv_lineend, $csv_delimiter, $csv_fieldnames);
		}
		$hiddens .= we_html_element::hiddens(($customers ? [
				'cmd' => 'do_export',
				'all' => $all,
				'customers' => implode(',', $customers)
				] : [
				'cmd' => 'end_export'
				]
				)
		);

		return we_html_tools::getHtmlTop(g_l('modules_customer', '[export_title]'),
																			 we_html_element::jsScript(WE_JS_MODULES_DIR . 'customer/EIWizard.js') . $this->jsCmd->getCmds(),
																							($success ? we_html_element::body(['onload' => 'document.we_form.submit()'],
																												 we_html_element::form([
						'name' => we_base_constants::WE_FORM,
						'method' => 'post',
						'action' => WEBEDITION_DIR . 'we_cmd.php?we_cmd[0]=we_showMod&mod=' . we_modules_info::CUSTOMER], $hiddens)
				) :
				we_html_element::body([], g_l('backup', '[error]'))
				)
		);
	}

	private function getEndExportCode(): string{
		$export_to = we_base_request::_(we_base_request::FILE, 'export_to', self::EXPORT_SERVER);
		$file_format = we_base_request::string('file_format');
		$filename = we_base_request::_(we_base_request::FILE, 'filename') . we_exim_ExIm::getExtensionByType($file_format);

		$path = ($export_to === self::EXPORT_SERVER ? we_base_request::_(we_base_request::FILE, 'path', '') : TEMP_DIR);

		switch($file_format){
			case we_exim_ExIm::TYPE_XML:
				$file_name = WE_DOCUMENT_ROOT . $path . '/' . $filename;
				we_customer_EI::save2File($file_name, we_backup_util::weXmlExImFooter);
		}

		return we_html_tools::getHtmlTop(g_l('modules_customer', '[export_title]'), '', we_html_element::body(['onload' => 'document.we_form.submit()'], we_html_element::form([
						'name' => we_base_constants::WE_FORM,
						'method' => 'post',
						'action' => WEBEDITION_DIR . 'we_cmd.php?we_cmd[0]=we_showMod&mod=' . we_modules_info::CUSTOMER], we_html_element::hiddens([
							'pnt' => 'eibody',
							'step' => '4',
							'art' => self::ART_EXPORT,
							'export_to' => $export_to,
							'filename' => $filename,
							'path' => $path])
					)
				)
		);
	}

	private function getImportBackCode(): string{
		$this->jsCmd->addCmd('load_processCmd', ['cmd' => 'import_back', 'art' => self::ART_IMPORT]);
		return we_html_tools::getHtmlTop(g_l('modules_customer', '[import_title]'), $this->jsCmd->getCmds(), we_html_element::body([], we_html_element::form([
						'name' => we_base_constants::WE_FORM,
						'method' => 'post',
						'action' => WEBEDITION_DIR . 'we_cmd.php?we_cmd[0]=we_showMod&mod=' . we_modules_info::CUSTOMER], '')
				)
		);
	}

	private function getImportCode(): string{
		$filename = we_base_request::_(we_base_request::FILE, 'filename', '');
		$type = we_base_request::_(we_base_request::RAW, 'type', '');
		$same = we_base_request::string('same', we_exim_ExIm::COLLISION_RENAME);
		$field_mappings = we_base_request::_(we_base_request::RAW, 'field_mappings', []);

		switch($type){
			case we_exim_ExIm::TYPE_CSV:
				$csv_delimiter = we_base_request::_(we_base_request::RAW_CHECKED, 'csv_delimiter', self::CSV_DELIMITER);
				$csv_enclose = we_base_request::_(we_base_request::RAW_CHECKED, 'csv_enclose', self::CSV_ENCLOSE);
				$csv_lineend = we_base_request::string('csv_lineend', self::CSV_LINEEND);
				$the_charset = we_base_request::string('the_charset', self::THE_CHARSET);
				$csv_fieldnames = we_base_request::bool('csv_fieldnames', true);
				we_customer_EI::importCSV(WE_DOCUMENT_ROOT . $filename, $same, $the_charset, $csv_delimiter, $csv_enclose, $csv_lineend, $csv_fieldnames, $field_mappings);
				break;
			case we_exim_ExIm::TYPE_XML:
				we_customer_EI::importXML(WE_DOCUMENT_ROOT . $filename, $field_mappings, $same);
				break;
		}

		return $this->getEndImportCode();
	}

	private function getEndImportCode(): string{
		return we_html_tools::getHtmlTop(g_l('modules_customer', '[import_title]'), $this->jsCmd->getCmds(), we_html_element::body([], we_html_element::form([
						'name' => we_base_constants::WE_FORM,
						'method' => 'post',
						'action' => WEBEDITION_DIR . 'we_cmd.php?we_cmd[0]=we_showMod&mod=' . we_modules_info::CUSTOMER], we_html_element::hiddens([
							'pnt' => 'eibody',
							'art' => self::ART_IMPORT,
							'step' => '5'
						])
					)
				)
		);
	}

	private function getHTMLCustomer(): string{
		switch(we_base_request::string('wcmd')){
			case 'add_customer':
				$customers = we_base_request::intList('customers', []);
				$customers = array_unique(array_merge($customers, we_base_request::intList('cus', [])));
				break;
			case 'del_customer':
				$customers = we_base_request::intList('customers', []);
				if(($id = we_base_request::int('cus'))){
					foreach($customers as $k => $v){
						if($v == $id){
							unset($customers[$k]);
						}
					}
				}
				break;
			case 'del_all_customers':
				$customers = [];
				break;
			default:
				$customers = we_base_request::intList('customers', []);
		}
		$customers = array_filter($customers);
		$hiddens = we_html_element::hiddens([
				'wcmd' => '',
				'cus' => implode(',', we_base_request::intList('cus', [])),
				'customers' => implode(',', $customers)]);

		$delallbut = we_html_button::create_button(we_html_button::DELETE_ALL, "javascript:we_cmd('del_all_customers')", '', '', ($customers ? false : true));
		$addbut = we_html_button::create_button(we_html_button::ADD, "javascript:we_cmd('we_customer_selector',{jsCmd:'add_customer'})");
		$custs = new we_chooser_multiDir(($customers ?: []), 'del_customer', $delallbut . $addbut, [], '"' . we_base_ContentTypes::CUSTOMER . '"', CUSTOMER_TABLE);

		$custs->setEditable(we_base_permission::hasPerm([we_base_perm::CUSTOMER_EDIT]));
//		$this->jsCmd->addCmd('set_topVar', ['name' => 'customers', 'value' => '', 'fromInput' => 1]);

		return $hiddens . $custs->get($this->jsCmd);
	}

	/**
	 *
	 * @param string $name
	 * @param string $value
	 * @param array<string,string> $values
	 * @param string $title
	 * @return string
	 */
	private function getHTMLChooser(string $name, string $value, array $values, string $title): string{
		$select = new we_html_select(['name' => $name . '_select', 'onchange' => "we_cmd('chooser_onchange', this, '" . $name . "')",
			'style' => 'width:200px;']);
		$select->addOption('', '');
		foreach($values as $k => $v){
			$select->addOption(htmlspecialchars($k), htmlspecialchars($v));
		}
		$table = new we_html_table(['class' => 'default', 'width' => '250'], 1, 3);

		$table
			->setCol(0, 0, we_html_tools::textInput($name, $value))
			->setCol(0, 1, $select->getHtml(), ['style' => 'padding-left:10px;']);

		return we_html_tools::formElementTable($title, $table->getHtml());
	}

	private function getHTMLCustomerFilter(): string{
		$count = we_base_request::int('filter_count', 0);

		switch(we_base_request::string('fcmd')){
			case 'add_filter':
				$count++;
				break;
			case 'del_filter':
				if($count){
					$count--;
				} else {
					$count = 0;
				}
				break;
			default:
		}

		$this->jsCmd->addCmd('set_formField', ['name' => 'filter_count', 'value' => $count]);
		$custfields = [];
		foreach(array_column($this->db->getAllq('SHOW FIELDS FROM ' . CUSTOMER_TABLE), 'Field') as $fv){
			switch($fv){
				case 'ParentID':
				case 'IsFolder':
				case 'Path':
				case 'Text':
					break;
				default:
					$custfields[$fv] = $fv;
			}
		}

		$table = new we_html_table(['class' => 'default'], 1, 3);
		$colspan = '3';

		$c = 0;
		$fields_names = ['fieldname', 'operator', 'fieldvalue', 'logic'];

		for($i = 0; $i < $count; $i++){
			$new = ['fieldname' => '', 'operator' => '', 'fieldvalue' => '', 'logic' => ''];
			foreach($fields_names as $field){
				if(($val = we_base_request::string('filter_' . $field . '_' . $i))){
					$new[$field] = $val;
				}
			}
			$table
				->setCol($c++, 0, ($i == 0 ?
						we_html_element::hidden('filter_logic_0', '') :
						we_html_tools::select('filter_logic_' . $i, we_customer_view::SEARCH_LOGIC, 1, $new['logic'], false, [], true, 70))
					, ['colspan' => $colspan]
				)
				->setCol($c, 0, we_html_tools::select('filter_fieldname_' . $i, $custfields, 1, $new['fieldname'], false, [], true, 200))
				->setCol($c, 1, we_html_tools::select('filter_operator_' . $i, we_customer_view::SEARCH_OPERATORS, 1, $new['operator'], false, [], true, 70))
				->setCol($c, 2, we_html_tools::textInput('filter_fieldvalue_' . $i, $new['fieldvalue']));
			$c++;
		}

		$plus = we_html_button::create_button(we_html_button::PLUS, "javascript:we_cmd('change_filter', 'add_filter')");
		$trash = we_html_button::create_button(we_html_button::TRASH, "javascript:we_cmd('change_filter', 'del_filter')");

		$c++;
		$table->setCol($c, 0, $plus . $trash, ['colspan' => $colspan, 'style' => 'padding-top:5px;']);

		return
			//(array('name'=>"filter_count","value"=>$count)).
			we_html_element::hidden('fcmd', '') .
			$table->getHtml();
	}

	private function getOperator(int $num): string{
		switch($num){
			default:
			case 0:
				return '=';
			case 1:
				return '<>';
			case 2:
				return '<';
			case 3:
				return '<=';
			case 4:
				return '>';
			case 5:
				return '>=';
			case 6:
				return 'LIKE';
		}
	}

	private function getExportCustomers(): array{
		switch(we_base_request::string('selection')){
			case self::SELECTION_MANUAL:
				return we_base_request::intList('customers', []);
			default:
				$filter_count = we_base_request::int('filter_count', 0);
				$filters = [
					'fieldname' => [],
					'operator' => [],
					'fieldvalue' => [],
					'logic' => []
				];
				'@phan-var array<string,string[]> $filters';
				for($i = 0; $i < $filter_count; $i++){
					foreach($filters as $field => &$value){
						$value[] = we_base_request::string('filter_' . $field . '_' . $i);
					}
				}
				$filterarr = [];
				foreach($filters['fieldname'] as $k => $v){
					$filterValue = (is_numeric($filters['fieldvalue'][$k]) ? $filters['fieldvalue'][$k] : $this->db->escape($filters['fieldvalue'][$k]));
					$filterarr[] = ($k ? (' ' . $filters['logic'][$k] . ' ') : '') . $v . ' ' . $this->getOperator((int) $filters['operator'][$k]) . " '" . ((intval($filters['operator'][$k]) === 6 ? '%' . $filterValue . '%' : $filterValue)) . "'";
				}

				return $this->db->getAllq('SELECT ID FROM ' . CUSTOMER_TABLE . ($filterarr ? ' WHERE (' . implode(' ', $filterarr) . ')' : ''), true);
		}
	}

	protected function formFileChooser(string $IDName = 'ParentID', string $IDValue = '/', string $cmd = '', string $filter = 'all_Types'): string{
		return we_html_tools::formElementTable('', we_html_tools::textInput($IDName, $IDValue, 0, ['readonly' => 'readonly'], 'text', '400px'), '', (we_base_permission::hasPerm([
					we_base_perm::WORK_EXTERNAL_FILES_CAN_SELECT]) ? we_html_button::create_button(we_html_button::SELECT, "javascript:we_cmd('browse_server','" . $IDName . "','" . $filter . "',document.we_form.elements['" . $IDName . "'].value,'" . $cmd . "');") : ''));
	}

}
