<?php
declare(strict_types=1);

/**
 * webEdition CMS
 *
 * $Rev: 2fcfd49ac317 $
 * $Author: Marc Kraemer <marc.kraemer@aldente-it.de> $
 * $Date: 2020-07-06 14:53 +0200 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
class we_dialog_link extends we_dialog_base{
	/** @var string */
	protected $type = we_base_link::TYPE_INT;

	/** @var int */
	protected $intID = 0;

	/** @var int */
	protected $objID = 0;

	/** @var string */
	protected $intHref = '';

	/** @var string */
	protected $extHref = we_base_link::EMPTY_EXT;

	/** @var string */
	protected $objHref = '';

	/** @var string */
	protected $emailHref = '';

	/** @var string */
	protected $cType = we_base_link::CONTENT_TEXT;

	/** @var string */
	protected $cText = '';

	/** @var int */
	protected $cId = 0;

	/** @var string */
	protected $cHref = '';

	/** @var string */
	protected $cAlt = '';

	/** @var string */
	protected $cTitle = '';

	/** @var int */
	protected $width = 0;

	/** @var int */
	protected $height = 0;

	/** @var int */
	protected $border = 0;

	/** @var int */
	protected $hspace = 0;

	/** @var int */
	protected $vspace = 0;

	/** @var int */
	protected $align = 0;

	/** @var string */
	protected $anchor = '';

	/** @var string */
	protected $params = '';

	/** @var string */
	protected $target = '';

	/** @var string[] */
	protected $cssClasses = [];

	/** @var string */
	protected $subject = '';

	/** @var string */
	protected $cc = '';

	/** @var string */
	protected $bcc = '';

	/** @var string */
	protected $lang = '';

	/** @var string */
	protected $hreflang = '';

	/** @var string */
	protected $title = '';

	/** @var string */
	protected $accesskey = '';

	/** @var int */
	protected $tabindex = 0;

	/** @var string */
	protected $rel = '';

	/** @var string */
	protected $attribs = '';

	/** @var bool */
	protected $isDialogInitialized = false;

	/** @var string */
	private $transaction = '';

	/** @var string */
	protected $fieldname = '';
	protected $identifier = '';

	/** @var string */
	protected $we_field = '';

	/** @var array<string,string> */
	protected $dialogToWedoc = [
		'intID' => 'id',
		'objID' => 'obj_id',
		'intHref' => 'href_int',
		'extHref' => 'href',
		'objHref' => 'href_obj',
		'emailHref' => 'emaillink',
		'cType' => 'ctype',
		'cText' => 'text',
		'cId' => 'img_id',
		'cHref' => 'img_src',
		'cAlt' => 'alt',
		'cTitle' => 'img_title',
	];

	public function __construct(bool $noInternals = true){
		parent::__construct($noInternals);
		$this->argTypes = array_merge($this->argTypes, [
			'type' => we_base_request::STRING,
			'intID' => we_base_request::INT,
			'objID' => we_base_request::INT,
			'intHref' => we_base_request::URL,
			'extHref' => we_base_request::STRING, // STRING instead of URL to allow 'tel:+12345678'
			'objHref' => we_base_request::URL,
			'emailHref' => we_base_request::STRING,
			'cType' => we_base_request::STRING,
			'cText' => we_base_request::STRING,
			'cId' => we_base_request::INT,
			'cHref' => we_base_request::URL,
			'cAlt' => we_base_request::STRING,
			'cTitle' => we_base_request::STRING,
			'width' => we_base_request::INT,
			'height' => we_base_request::INT,
			'border' => we_base_request::INT,
			'hspace' => we_base_request::INT,
			'vspace' => we_base_request::INT,
			'align' => we_base_request::STRING,
			'anchor' => we_base_request::STRING,
			'params' => we_base_request::STRING,
			'target' => we_base_request::STRING,
			'cssClasses' => we_base_request::STRING_LIST,
			'subject' => we_base_request::STRING,
			'cc' => we_base_request::STRING,
			'bcc' => we_base_request::STRING,
			'lang' => we_base_request::STRING,
			'hreflang' => we_base_request::STRING,
			'title' => we_base_request::STRING,
			'accesskey' => we_base_request::STRING,
			'tabindex' => we_base_request::INT,
			'rel' => we_base_request::STRING,
			'attribs' => we_base_request::STRING,
		]);

		$this->initByHttp();
	}

	protected function initByHttp(): void{
		$this->isDialogInitialized = we_base_request::bool('isDialogInitialized', false);
		$this->transaction = we_base_request::string('we_cmd', '', 'transaction');
		$this->cmd = we_base_request::string('we_cmd', '', 0);
		$this->fieldname = we_base_request::string('we_cmd', '', 'fieldname');
		$this->identifier = we_base_request::string('we_cmd', '', 'identifier');
		$this->we_field = we_base_request::string('we_field', '');
		$this->cText = g_l('global', '[new_link]');

		if(!$this->isDialogInitialized){
			$this->initLinkFromSource();
		}
		parent::initByHttp();
	}

	/**
	 *
	 * @return array{type?:string,intID?:int,objID?:int,intHref?:string,extHref?:string,objHref?:string,emailHref?:string,emaillink?:string,cType?:string,cText?:string,cId?:int,cHref?:string,cAlt?:string,cTitle?:string,width?:int,height?:int,border?:int,hspace?:int,vspace?:int,align?:int|string,anchor?:string,params?:string,target?:string,cssClasses?:array,subject?:string,cc?:string,bcc?:string,lang?:string,hreflang?:string,title?:string,accesskey?:string,tabindex?:int,rel?:string,attribs?:string}
	 */
	protected function makeLink(): array{
		$link = [];
		foreach(array_keys($this->argTypes) as $key){
			if(in_array($key, ['objID, intID'], true)){ // FIXME: why do we skip these?
				continue;
			}
			$mappedKey = $this->dialogToWedoc[$key] ?? $key;
			$link[$mappedKey] = $this->{$key};
		}

		switch($link['type']){
			case we_base_link::TYPE_OBJ:
				if($link['params'] && substr($link['params'], 0, 1) !== '&'){
					$link['params'] = (substr($link['params'], 0, 1) === '?' ? '&' . substr($link['params'], 1) : '&' . $link['params']);
				}
				break;
			case we_base_link::TYPE_MAIL:
				$link['params'] = $link['anchor'] = $link['target'] = '';
				break;
			default:
				if($link['params'] && substr($link['params'], 0, 1) !== '?'){
					$link['params'] = (substr($link['params'], 0, 1) === '&' ? '?' . substr($link['params'], 1) : '?' . $link['params']);
				}
		}
		$link['anchor'] = (empty($link['anchor']) || $link['anchor'][0] === '#' ? '' : '#' ) . $link['anchor']; //@phan-suppress-current-line PhanRedundantCondition

		return $link;
	}

	protected function cmdFunction(array $args): string{
		$link = $this->makeLink();

		switch($link['type']){
			case we_base_link::TYPE_MAIL:
				$link['href'] = we_base_link::TYPE_MAIL_PREFIX . $link['emaillink'];
				$link['emaillink'] = '';
		}
		unset($link['emaillink']);

		if(!empty($link)){
			switch($this->cmd){
				case 'edit_link_at_class':
					$_SESSION['weS']['WE_LINK'] = array_filter($link);
					$this->jsCmd->addCmd('we_cmd', ['classfield_link_writeback', $this->transaction, $this->identifier, $this->fieldname]);
					//FIXME: we_field XSS
					break;
				case 'edit_link_at_object':
					$_SESSION['weS']['WE_LINK'] = array_filter($link);
					$this->jsCmd->addCmd('we_cmd', ['objectfield_link_writeback', $this->transaction, $this->identifier]);
					break;
				case 'edit_link':
					$_SESSION['weS']['WE_LINK'] = array_filter($link);
					$this->jsCmd->addCmd('we_cmd', ['tag_welink_writeback', $this->fieldname]);
			}
		}

		return we_html_tools::getHtmlTop('', $this->getJsCmd() . $this->jsCmd->getCmds(), we_html_element::body());
	}

	protected function initLinkFromSource(): void{
		if(!$this->transaction){
			return;
		}

		$we_doc = we_contents_root::getSessionDocument($this->transaction);
		switch($this->cmd){
			case 'edit_link_at_class':
			case 'edit_link_at_object':
				$link = $we_doc->getElementA($this->fieldname);
				break;
			default:
				$link = we_unserialize($we_doc->getElementS($this->fieldname));
		}

		if($link){
			$mapping = array_flip($this->dialogToWedoc);

			foreach($link as $key => $value){
				if(($prop = isset($this->{$key}) ? $key : (!empty($mapping[$key]) && isset($this->{$mapping[$key]}) ? $mapping[$key] : ''))){
					//$this->{$prop} = $value;
					$this->{$prop} = we_base_request::filterVar($value, $this->argTypes[$prop]); //@phan-suppress-current-line PhanDeprecatedFunction
				}
			}

			if($this->extHref && strpos($this->extHref, we_base_link::TYPE_MAIL_PREFIX) === 0){
				$this->type = we_base_link::TYPE_MAIL;
				$this->emailHref = substr($this->extHref, strlen(we_base_link::TYPE_MAIL_PREFIX));
				$this->extHref = we_base_link::EMPTY_EXT;
			}
		}
	}

	protected function getDialogPartsHTML(): we_html_multiIconBoxContent{
		$this->intHref = $this->intID ? id2path($this->intID) : '';
		$this->cHref = $this->cId ? id2path($this->cId) : '';
		$this->objHref = we_modules_info::isActive(we_modules_info::OBJECT) && $this->objID ? id2path($this->objID, OBJECT_FILES_TABLE) : '';

		$typeSelect = we_html_tools::select('we_dialog_args[type]', array_filter([
				we_base_link::TYPE_EXT => g_l('linklistEdit', '[external_link]'),
				we_base_link::TYPE_INT => g_l('linklistEdit', '[internal_link]'),
				we_base_link::TYPE_MAIL => g_l('wysiwyg', '[emaillink]'),
				we_base_link::TYPE_OBJ => (we_modules_info::isActive(we_modules_info::OBJECT) ? g_l('linklistEdit', '[objectFile]') : ''),
				]), 1, $this->type, false, ['onchange' => "changeTypeSelect(this.value);",
				'class' => 'big',
				'style' => 'margin-bottom:5px;'
		]);

		$extBtn = we_base_permission::hasPerm([we_base_perm::WORK_EXTERNAL_FILES_CAN_SELECT]) ? we_html_button::create_button(we_html_button::SELECT, "javascript:we_cmd('browse_server', 'we_dialog_args[extHref]', 'all_Types', document.we_form.elements['we_dialog_args[extHref]'].value, '')") : '';
		$extLink = we_html_tools::formElementTable('', we_html_tools::textInput('we_dialog_args[extHref]', $this->extHref, 0, ['placeholder' => 'http://www.example.com']), $extBtn);
		$mailLink = we_html_tools::textInput('we_dialog_args[emailHref]', $this->emailHref, 0, ['placeholder' => "user@example.com"]);

		$weSuggest = we_gui_suggest::getInstance();
		$weSuggest->setAcId('Doc')
			->setContentType(array_merge(we_base_ContentTypes::LINK_ABLE, [we_base_ContentTypes::FOLDER_DOCUMENT]))
			->setInput('we_dialog_args[intHref]', $this->intHref)
			->setMaxResults(20)
			->setResult('we_dialog_args[intID]', (string) $this->intID)
			->setSelector(we_gui_suggest::DocSelector)
			->setWidth(200)
			->setSelectButton(we_html_button::create_button(we_html_button::SELECT, "javascript:we_cmd('we_selector_document',{id:document.we_form.elements['we_dialog_args[intID]'].value,table:WE().consts.tables.FILE_TABLE,targetID:'we_dialog_args[intID]',targetName:'we_dialog_args[intHref]',rootDirID:0,open_doc:" . (we_base_permission::hasPerm([
						we_base_perm::WORK_WORKSPACE_CAN_SELECT_OTHER_FILES]) ? 0 : 1) . "});"))
			->setOpenButton(we_html_button::create_button(we_html_button::OPEN_TO_EDIT, "javascript:WE().layout.openToEdit(WE().consts.tables.FILE_TABLE,document.we_form.elements['we_dialog_args[intID]'].value);"));
		$intLink = $weSuggest->getHTML();
		$objLink = '';
		if(we_modules_info::isActive(we_modules_info::OBJECT)){
			$weSuggest->setAcId('Obj')
				->setContentType([we_base_ContentTypes::FOLDER_OBJECTFILE, we_base_ContentTypes::OBJECT_FILE])
				->setInput('we_dialog_args[objHref]', $this->objHref)
				->setMaxResults(20)
				->setResult('we_dialog_args[objID]', (string) $this->objID)
				->setSelector(we_gui_suggest::DocSelector)
				->setTable(OBJECT_FILES_TABLE)
				->setWidth(200)
				->setSelectButton(we_html_button::create_button(we_html_button::SELECT, "javascript:we_cmd('we_selector_document',{id:document.we_form.elements['we_dialog_args[objID]'].value,table:WE().consts.tables.OBJECT_FILES_TABLE,targetID:'we_dialog_args[objID]',targetName:'we_dialog_args[objHref]',filter:WE().consts.contentTypes.OBJECT_FILE,open_doc:" . (we_base_permission::hasPerm([
							we_base_perm::OBJECTS_CAN_SELECT_OTHER_USERS]) ? 0 : 1) . "});"))
				->setOpenButton(we_html_button::create_button(we_html_button::OPEN_TO_EDIT, "javascript:WE().layout.openToEdit(WE().consts.tables.OBJECT_FILES_TABLE,document.we_form.elements['we_dialog_args[objID]'].value,WE().consts.contentTypes.OBJECT_FILE);"));
			$objLink = $weSuggest->getHTML();
		}

		$classesAllButEmail = implode(' ', [we_base_link::TYPE_INT . '_row', we_base_link::TYPE_EXT . '_row', we_base_link::TYPE_OBJ . '_row']);

		$langHtml = we_html_element::div(['class' => 'inputSelect small'], we_html_tools::getLangField('we_dialog_args[lang]', $this->lang, g_l('linklistEdit', '[link_language]'))) .
			we_html_element::div(['class' => 'inputSelect small'], we_html_tools::getLangField('we_dialog_args[hreflang]', $this->hreflang, g_l('linklistEdit', '[href_language]')));
		$keyboardHtml = we_html_element::div(['class' => 'col2input'],
																			 we_html_tools::formElementTable(g_l('linklistEdit', '[accesskey]'), we_html_tools::textInput('we_dialog_args[accesskey]', $this->accesskey)) .
				we_html_tools::formElementTable(g_l('linklistEdit', '[tabindex]'), we_html_tools::textInput('we_dialog_args[tabindex]', (string) $this->tabindex ?: ''))
		);

		if($this->isTinyDialog){
			$cssHtml = $this->getClassSelect();
			$contentSelect = $textHtml = $imgextHtml = $imgPropsHtml = $imgintHtml = '';
		} else {
			$cssHtml = '';
			$contentSelect = we_html_tools::select('we_dialog_args[cType]', [we_base_link::CONTENT_TEXT => htmlspecialchars(g_l('linklistEdit', '[text]')),
					we_base_link::CONTENT_EXT => htmlspecialchars(g_l('linklistEdit', '[external_image]')),
					we_base_link::CONTENT_INT => htmlspecialchars(g_l('linklistEdit', '[internal_image]')),
					], 1, $this->cType, false, ['onchange' => "changeCTypeSelect(this.value);",
					'class' => 'big',
					'style' => 'margin-bottom:5px;'
			]);
			$textHtml = we_html_tools::textInput('we_dialog_args[cText]', $this->cText);
			$imgextHtml = we_html_tools::formElementTable('', we_html_tools::textInput('we_dialog_args[cHref]', $this->cHref), we_base_permission::hasPerm([
						we_base_perm::WORK_EXTERNAL_FILES_CAN_SELECT]) ? we_html_button::create_button(we_html_button::SELECT, "javascript:we_cmd('browse_server', 'we_dialog_args[cHref]', '" . we_base_ContentTypes::IMAGE . "', document.we_form.elements['we_dialog_args[cHref]'].value, '')") : '');
			$weSuggest->setAcId('Image')
				->setContentType([we_base_ContentTypes::FOLDER_DOCUMENT, we_base_ContentTypes::IMAGE])
				->setInput('we_dialog_args[cHref]', $this->cHref)
				->setMaxResults(20)
				->setResult('we_dialog_args[cId]', (string) $this->cId)
				->setSelector(we_gui_suggest::DocSelector)
				->setSelectButton(we_html_button::create_button(we_html_button::SELECT, "javascript:we_cmd('we_selector_image',{id:document.we_form.elements['we_dialog_args[cId]'].value,table:WE().consts.tables.FILE_TABLE,targetID:'we_dialog_args[cId]',targetName:'we_dialog_args[cHref]',open_doc:" . (we_base_permission::hasPerm([
							we_base_perm::WORK_WORKSPACE_CAN_SELECT_OTHER_FILES]) ? 0 : 1) . "});"));
			$imgintHtml = $weSuggest->getHTML();
			$imgPropsHtml = we_html_element::div(['class' => 'imgProps'],
																				we_html_tools::formElementTable(g_l('global', '[width]'), we_html_tools::textInput('we_dialog_args[width]', (string) $this->width ?: '', 0, [
							], 'number')) .
					we_html_tools::formElementTable(g_l('global', '[height]'), we_html_tools::textInput('we_dialog_args[height]', (string) $this->height ?: '', 0, [], 'number')) .
					we_html_tools::formElementTable(g_l('global', '[border]'), we_html_tools::textInput('we_dialog_args[border]', (string) $this->border ?: '', 0, [], 'number'))
				) .
				we_html_element::div(['class' => 'imgProps'],
												 we_html_tools::formElementTable(g_l('global', '[hspace]'), we_html_tools::textInput('we_dialog_args[hspace]', (string) $this->hspace ?: '', 0, [], 'number')) .
					we_html_tools::formElementTable(g_l('global', '[vspace]'), we_html_tools::textInput('we_dialog_args[vspace]', (string) $this->vspace ?: '', 0, [], 'number')) .
					we_html_tools::formElementTable(g_l('global', '[align]'), we_html_tools::select('we_dialog_args[align]', [
							'' => 'Default',
							'top' => 'Top',
							'middle' => 'Middle',
							'bottom' => 'Bottom',
							'left' => 'Left',
							'right' => 'Right',
							'texttop' => 'Text Top',
							'absmiddle' => 'Abs Middle',
							'baseline' => 'Baseline',
							'absbottom' => 'Abs Bottom',
							], 1, $this->align))
				) .
				we_html_tools::formElementTable(g_l('linklistEdit', '[title]'), we_html_tools::textInput('we_dialog_args[cAlt]', (string) $this->cAlt ?: '')) .
				we_html_tools::formElementTable(g_l('weClass', '[alt]'), we_html_tools::textInput('we_dialog_args[cTitle]', (string) $this->cTitle ?: ''));
		}

		$parts = (new we_html_multiIconBoxContent())
			->add((new we_html_multiIconBoxContentLine('URL', we_html_element::div([], $typeSelect) .
				we_html_element::div(['class' => 'changetype ' . we_base_link::TYPE_EXT . '_row' . ($this->type !== we_base_link::TYPE_EXT ? ' hidden' : '')], $extLink) .
				we_html_element::div(['class' => 'changetype ' . we_base_link::TYPE_INT . '_row' . ($this->type !== we_base_link::TYPE_INT ? ' hidden' : '')], $intLink) .
				we_html_element::div(['class' => 'changetype ' . we_base_link::TYPE_MAIL . '_row' . ($this->type !== we_base_link::TYPE_MAIL ? ' hidden' : '')], $mailLink) .
				(!we_modules_info::isActive(we_modules_info::OBJECT) ? '' :
				we_html_element::div(['class' => 'changetype ' . we_base_link::TYPE_OBJ . '_row' . ($this->type !== we_base_link::TYPE_OBJ ? ' hidden' : '')], $objLink)
				)))->space(we_html_multiIconBox::SPACE_MED2)
			->noline()
		);
		if(!$this->isTinyDialog){
			$parts->add((new we_html_multiIconBoxContentLine(g_l('global', '[content]'), we_html_element::div([], $contentSelect) .
					we_html_element::div(['class' => 'changecontent ' . we_base_link::CONTENT_TEXT . '_row' . ($this->cType !== we_base_link::CONTENT_TEXT ? ' hidden' : '')], $textHtml) .
					we_html_element::div(['class' => 'changecontent ' . we_base_link::CONTENT_EXT . '_row' . ($this->cType !== we_base_link::CONTENT_EXT ? ' hidden' : '')], $imgextHtml) .
					we_html_element::div(['class' => 'changecontent ' . we_base_link::CONTENT_INT . '_row' . ($this->cType !== we_base_link::CONTENT_INT ? ' hidden' : '')], $imgintHtml) .
					we_html_element::div(['class' => 'changecontent ' . implode(' ', [we_base_link::CONTENT_INT . '_row', we_base_link::CONTENT_EXT . '_row']) . (!$this->cType || $this->cType === we_base_link::CONTENT_TEXT ? ' hidden' : '')], $imgPropsHtml)
				))->space(we_html_multiIconBox::SPACE_MED2)
			);
		}
		$parts->add((new we_html_multiIconBoxContentLine(g_l('linklistEdit', '[link_anchor]'), we_html_element::div([],
																																																							we_html_tools::textInput('we_dialog_args[anchor]', $this->anchor, 0, ['pattern' => '^#?[a-zA-Z]+[a-zA-Z0-9_:.-=]*$',
							'onkeyup' => "checkMakeEmptyHrefExt()",
							'onblur' => "checkMakeEmptyHrefExt(); checkAnchor(this);"]) .
						we_html_element::div(['id' => 'anchorlistcontainer', 'style' => 'display:inline-block;']))
				))
				->space(we_html_multiIconBox::SPACE_MED2)
				->class('changetype ' . $classesAllButEmail . ($this->type === we_base_link::TYPE_MAIL ? ' hidden' : ''))
				->noline())
			->add((new we_html_multiIconBoxContentLine(g_l('linklistEdit', '[link_params]'), we_html_tools::textInput('we_dialog_args[params]', $this->params)))
				->space(we_html_multiIconBox::SPACE_MED2)
				->class('changetype ' . $classesAllButEmail . ($this->type === we_base_link::TYPE_MAIL ? ' hidden' : ''))
				->noline())
			->add((new we_html_multiIconBoxContentLine(g_l('linklistEdit', '[link_target]'), we_html_element::div(['class' => 'inputSelect small'],
																																																				 we_html_tools::formElementTable(g_l('linklistEdit', '[link_target]'), we_html_tools::targetBox('we_dialog_args[target]', 0, '', $this->target))) .
					we_html_element::div(['class' => 'inputSelect small'], we_html_tools::getRelSelect('we_dialog_args[rel]', $this->rel, 'rel', '', 0)
					)))
				->space(we_html_multiIconBox::SPACE_MED2)
				->class('changetype ' . $classesAllButEmail . ($this->type === we_base_link::TYPE_MAIL ? ' hidden' : ''))
				->noline($this->isTinyDialog)
		);

		if($this->isTinyDialog){
			$parts->add((new we_html_multiIconBoxContentLine(g_l('wysiwyg', '[css_style]'), $cssHtml))
					->space(we_html_multiIconBox::SPACE_MED2)
					->class('changetype ' . $classesAllButEmail . ($this->type === we_base_link::TYPE_MAIL ? ' hidden' : ''))
			);
		}
		$parts->add((new we_html_multiIconBoxContentLine(g_l('modules_messaging', '[subject]'), we_html_tools::textInput('we_dialog_args[subject]', $this->subject)))
				->space(we_html_multiIconBox::SPACE_MED2)
				->class('changetype ' . we_base_link::TYPE_MAIL . '_row' . ($this->type !== we_base_link::TYPE_MAIL ? ' hidden' : ''))
				->noline())
			->add((new we_html_multiIconBoxContentLine('CC', we_html_tools::textInput('we_dialog_args[cc]', $this->cc)))
				->space(we_html_multiIconBox::SPACE_MED2)
				->class('changetype ' . we_base_link::TYPE_MAIL . '_row' . ($this->type !== we_base_link::TYPE_MAIL ? ' hidden' : ''))
				->noline())
			->add((new we_html_multiIconBoxContentLine('BCC', we_html_tools::textInput('we_dialog_args[bcc]', $this->bcc)))
				->space(we_html_multiIconBox::SPACE_MED2)
				->class('changetype ' . we_base_link::TYPE_MAIL . '_row' . ($this->type !== we_base_link::TYPE_MAIL ? ' hidden' : ''))
		);
		if(we_base_permission::hasPerm([we_base_perm::WORK_ACCESSIBLE_PARAMETERS_CAN_SEE])){
			$parts->add((new we_html_multiIconBoxContentLine(g_l('linklistEdit', '[language]'), $langHtml))
					->space(we_html_multiIconBox::SPACE_MED2)
					->noline())
				->add((new we_html_multiIconBoxContentLine(g_l('linklistEdit', '[title]'), we_html_tools::textInput('we_dialog_args[title]', $this->title)))
					->space(we_html_multiIconBox::SPACE_MED2)
					->noline())
				->add((new we_html_multiIconBoxContentLine(g_l('linklistEdit', '[keyboard]'), $keyboardHtml))
					->space(we_html_multiIconBox::SPACE_MED2)
					->noline())
				->add((new we_html_multiIconBoxContentLine(g_l('linklistEdit', '[link_attr]'), we_html_tools::textInput('we_dialog_args[attribs]', $this->attribs)))
					->space(we_html_multiIconBox::SPACE_MED2)
			);
		}

		return $parts;
	}

	protected function getJs(): string{
		return we_html_element::jsScript(JS_DIR . 'dialogs/we_dialog_link.js', '', [
				'id' => 'loadVarLink', 'data-link' => setDynamicVar($this->getJsDynamic())]);
	}

	/**
	 *
	 * @return array{editname:string,classNames:string,is_frontend:bool}
	 */
	protected function getJsDynamic(): array{
		return [
			'editname' => '',
			'classNames' => ($this->cssClasses ? implode(',', $this->cssClasses) : 'getFromTiny'),
			'is_frontend' => $this->noInternals
		];
	}

	protected function getJsCmd(): string{
		return we_html_element::jsScript(JS_DIR . 'dialogs/we_dialog_cmdFrame.js');
	}

	protected function getHiddenArgs(): string{
		return we_html_element::hiddens([
				'we_cmd[0]' => $this->cmd,
				'fieldname' => $this->fieldname,
				'transaction' => $this->transaction,
				'we_field' => $this->we_field,
				'isDialogInitialised' => true,
		]);
	}

	protected function getDialogButtons(): string{
		return we_html_button::position_yes_no_cancel(we_html_button::create_button(we_html_button::OK, "javascript:document.forms.we_form.submit();", '', "", false, false), '', $this->getBtnCancel());
	}

}
