<?php
declare(strict_types=1);

/**
 * webEdition CMS
 *
 * $Rev: 0627c1498cde $
 * $Author: Marc Kraemer <marc.kraemer@aldente-it.de> $
 * $Date: 2020-07-02 12:07 +0200 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
abstract class we_editor_header{

	private static function getTabs(we_contents_root $we_doc, string $we_transaction): we_gui_tabs{
		$we_tabs = new we_gui_tabs();
		switch($_SESSION['weS']['we_mode']){
			default:
			case we_base_constants::MODE_NORMAL:
				// user has no access to file - only preview mode.
				$access = $we_doc->userHasAccess();
				if($access != we_contents_root::USER_HASACCESS && $access != we_contents_root::USER_NO_SAVE){
					if(in_array(we_base_constants::WE_EDITPAGE_PREVIEW, $we_doc->EditPageNrs, true)){
						$we_tabs->addTab('', we_gui_tabs::WE_ICON_PREVIEW, (($we_doc->EditPageNr == we_base_constants::WE_EDITPAGE_PREVIEW)), "['switch_edit_page'," . we_base_constants::WE_EDITPAGE_PREVIEW . ",'" . $we_transaction . "'];", [
							'id' => 'tab_' . we_base_constants::WE_EDITPAGE_PREVIEW, 'title' => g_l('weClass', '[preview]')]);
					}
					return $we_tabs;
				} //	show tabs according to permissions
				if(in_array(we_base_constants::WE_EDITPAGE_PROPERTIES, $we_doc->EditPageNrs, true) && we_base_permission::hasPerm([we_base_perm::WORK_PROPERTIES_CAN_SEE])){
					$we_tabs->addTab('', we_gui_tabs::WE_ICON_PROPERTIES, (($we_doc->EditPageNr == we_base_constants::WE_EDITPAGE_PROPERTIES)), "['switch_edit_page'," . we_base_constants::WE_EDITPAGE_PROPERTIES . ",'" . $we_transaction . "']", [
						'id' => 'tab_' . we_base_constants::WE_EDITPAGE_PROPERTIES, 'title' => g_l('weClass', '[tab_properties]')]);
				}
				if(in_array(we_base_constants::WE_EDITPAGE_CONTENT, $we_doc->EditPageNrs, true)){
					$we_tabs->addTab('', (we_base_ContentTypes::isBinaryFile($we_doc->ContentType) ? 'fa-upload' : 'fa-edit'), (($we_doc->EditPageNr == we_base_constants::WE_EDITPAGE_CONTENT)), "['switch_edit_page'," . we_base_constants::WE_EDITPAGE_CONTENT . ",'" . $we_transaction . "']", [
						'id' => 'tab_' . we_base_constants::WE_EDITPAGE_CONTENT, 'title' => (we_base_ContentTypes::isBinaryFile($we_doc->ContentType) ?
							g_l('weClass', '[upload]') :
							g_l('weClass', '[edit]')
						)
					]);
				}

				if((in_array(we_base_constants::WE_EDITPAGE_VARIANTS, $we_doc->EditPageNrs, true) && we_base_permission::hasPerm([we_base_perm::VARIANTS_CAN_EDIT]) && ($we_doc->canHaveVariants(true) ))){
					$we_tabs->addTab('', 'fa-sitemap', (($we_doc->EditPageNr == we_base_constants::WE_EDITPAGE_VARIANTS)), "['switch_edit_page'," . we_base_constants::WE_EDITPAGE_VARIANTS . ",'" . $we_transaction . "']", [
						'id' => 'tab_' . we_base_constants::WE_EDITPAGE_VARIANTS, 'title' => g_l('weClass', '[variants]')]);
				}

				if(in_array(we_base_constants::WE_EDITPAGE_IMAGEEDIT, $we_doc->EditPageNrs, true)){
					$we_tabs->addTab('', we_gui_tabs::WE_ICON_EDIT, (($we_doc->EditPageNr == we_base_constants::WE_EDITPAGE_IMAGEEDIT)), "['switch_edit_page'," . we_base_constants::WE_EDITPAGE_IMAGEEDIT . ",'" . $we_transaction . "']", [
						'id' => 'tab_' . we_base_constants::WE_EDITPAGE_IMAGEEDIT, 'title' => g_l('weClass', '[edit_image]')]);
				}

				if(in_array(we_base_constants::WE_EDITPAGE_THUMBNAILS, $we_doc->EditPageNrs, true)){
					$we_tabs->addTab('', 'fa-image', (($we_doc->EditPageNr == we_base_constants::WE_EDITPAGE_THUMBNAILS)), "['switch_edit_page'," . we_base_constants::WE_EDITPAGE_THUMBNAILS . ",'" . $we_transaction . "']", [
						'id' => 'tab_' . we_base_constants::WE_EDITPAGE_THUMBNAILS, 'title' => g_l('weClass', '[thumbnails]')]);
				}

				if(in_array(we_base_constants::WE_EDITPAGE_WORKSPACE, $we_doc->EditPageNrs, true)){
					$we_tabs->addTab('', 'fa-desktop', (($we_doc->EditPageNr == we_base_constants::WE_EDITPAGE_WORKSPACE)), "['switch_edit_page'," . we_base_constants::WE_EDITPAGE_WORKSPACE . ",'" . $we_transaction . "']", [
						'id' => 'tab_' . we_base_constants::WE_EDITPAGE_WORKSPACE, 'title' => g_l('weClass', '[workspace]')]);
				}

				if(in_array(we_base_constants::WE_EDITPAGE_INFO, $we_doc->EditPageNrs, true) && we_base_permission::hasPerm([we_base_perm::WORK_INFOTAB_CAN_SEE])){
					$we_tabs->addTab('', 'fa-info-circle', (($we_doc->EditPageNr == we_base_constants::WE_EDITPAGE_INFO)), "['switch_edit_page'," . we_base_constants::WE_EDITPAGE_INFO . ",'" . $we_transaction . "']", [
						'id' => 'tab_' . we_base_constants::WE_EDITPAGE_INFO, 'title' => g_l('weClass', '[information]')]);
				}

				if(in_array(we_base_constants::WE_EDITPAGE_PREVIEW, $we_doc->EditPageNrs, true)){
					$we_tabs->addTab('', 'fa-eye' . ($we_doc->ContentType === we_base_ContentTypes::TEMPLATE ? ',fa-edit' : ''), (($we_doc->EditPageNr == we_base_constants::WE_EDITPAGE_PREVIEW)), "['switch_edit_page'," . we_base_constants::WE_EDITPAGE_PREVIEW . ",'" . $we_transaction . "']", [
						'id' => 'tab_' . we_base_constants::WE_EDITPAGE_PREVIEW, 'title' => g_l('weClass', ($we_doc->ContentType === we_base_ContentTypes::TEMPLATE ? '[previeweditmode]' : '[preview]'))]);
				}

				if(in_array(we_base_constants::WE_EDITPAGE_PREVIEW_TEMPLATE, $we_doc->EditPageNrs, true)){
					//preview of real page in templates
					$we_tabs->addTab('', 'fa-eye', (($we_doc->EditPageNr == we_base_constants::WE_EDITPAGE_PREVIEW_TEMPLATE)), "['switch_edit_page'," . we_base_constants::WE_EDITPAGE_PREVIEW_TEMPLATE . ",'" . $we_transaction . "']", [
						'id' => 'tab_' . we_base_constants::WE_EDITPAGE_PREVIEW_TEMPLATE, 'title' => g_l('weClass', '[preview]')]);
				}
				if(in_array(we_base_constants::WE_EDITPAGE_TEMPLATE_UNUSEDELEMENTS, $we_doc->EditPageNrs, true)){
					//show unused elements on pages not available in template
					$we_tabs->addTab('', 'fa-balance-scale', (($we_doc->EditPageNr == we_base_constants::WE_EDITPAGE_TEMPLATE_UNUSEDELEMENTS)), "['switch_edit_page'," . we_base_constants::WE_EDITPAGE_TEMPLATE_UNUSEDELEMENTS . ",'" . $we_transaction . "']", [
						'id' => 'tab_' . we_base_constants::WE_EDITPAGE_TEMPLATE_UNUSEDELEMENTS, 'title' => g_l('weClass', '[unusedElementsTab]')]);
				}
				if(in_array(we_base_constants::WE_EDITPAGE_FIELDS, $we_doc->EditPageNrs, true)){
					$we_tabs->addTab('', we_gui_tabs::WE_ICON_CONTENT, (($we_doc->EditPageNr == we_base_constants::WE_EDITPAGE_FIELDS)), "['switch_edit_page'," . we_base_constants::WE_EDITPAGE_FIELDS . ",'" . $we_transaction . "']", [
						'id' => 'tab_' . we_base_constants::WE_EDITPAGE_FIELDS, 'title' => g_l('weClass', '[docList]')]);
				}
				if(in_array(we_base_constants::WE_EDITPAGE_DOCLIST, $we_doc->EditPageNrs, true)){
					$we_tabs->addTab('', 'fa-search', (($we_doc->EditPageNr == we_base_constants::WE_EDITPAGE_DOCLIST)), "['switch_edit_page'," . we_base_constants::WE_EDITPAGE_DOCLIST . ",'" . $we_transaction . "']", [
						'id' => 'tab_' . we_base_constants::WE_EDITPAGE_DOCLIST, 'title' => g_l('weClass', '[docList]')]);
				}

				if(we_base_permission::hasPerm([we_base_perm::SCHEDULER_CAN_SEE]) && we_modules_info::isActive(we_modules_info::SCHEDULER) && in_array(we_base_constants::WE_EDITPAGE_SCHEDULER, $we_doc->EditPageNrs, true) && !in_array($we_doc->ContentType, we_base_ContentTypes::FOLDER_TYPES)){
					$we_tabs->addTab('', 'fa-clock-o', (($we_doc->EditPageNr == we_base_constants::WE_EDITPAGE_SCHEDULER)), "['switch_edit_page'," . we_base_constants::WE_EDITPAGE_SCHEDULER . ",'" . $we_transaction . "']", [
						'id' => 'tab_' . we_base_constants::WE_EDITPAGE_SCHEDULER, 'title' => g_l('weClass', '[scheduler]')]);
				}
				if((in_array(we_base_constants::WE_EDITPAGE_VALIDATION, $we_doc->EditPageNrs, true) && (in_array($we_doc->ContentType, [
						we_base_ContentTypes::WEDOCUMENT,
						we_base_ContentTypes::CSS,
						we_base_ContentTypes::HTML], true) )) && we_base_permission::hasPerm([we_base_perm::WORK_VALIDATION_CAN_SEE])){
					$we_tabs->addTab('', 'fa-bug', (($we_doc->EditPageNr == we_base_constants::WE_EDITPAGE_VALIDATION)), "['switch_edit_page'," . we_base_constants::WE_EDITPAGE_VALIDATION . ",'" . $we_transaction . "']", [
						'id' => 'tab_' . we_base_constants::WE_EDITPAGE_VALIDATION, 'title' => g_l('weClass', '[validation]')]);
				}

				if(in_array(we_base_constants::WE_EDITPAGE_WEBUSER, $we_doc->EditPageNrs, true) && (we_base_permission::hasPerm([we_base_perm::CUSTOMER_CAN_EDIT_CUSTOMERFILTER,
						we_base_perm::CUSTOMER_CAN_CHANGE_DOCS]))){
					$we_tabs->addTab('', we_gui_tabs::WE_ICON_CUSTOMER_FILTER, (($we_doc->EditPageNr == we_base_constants::WE_EDITPAGE_WEBUSER)), "['switch_edit_page'," . we_base_constants::WE_EDITPAGE_WEBUSER . ",'" . $we_transaction . "']", [
						'id' => 'tab_' . we_base_constants::WE_EDITPAGE_WEBUSER, 'title' => g_l('weClass', '[webUser]')]);
				}

				if(we_base_permission::hasPerm([we_base_perm::VERSIONS_CAN_SEE]) && in_array(we_base_constants::WE_EDITPAGE_VERSIONS, $we_doc->EditPageNrs, true)){
					$we_tabs->addTab('', 'fa-history', (($we_doc->EditPageNr == we_base_constants::WE_EDITPAGE_VERSIONS)), "['switch_edit_page'," . we_base_constants::WE_EDITPAGE_VERSIONS . ",'" . $we_transaction . "']", [
						'id' => 'tab_' . we_base_constants::WE_EDITPAGE_VERSIONS, 'title' => g_l('weClass', '[version]')]);
				}

				return $we_tabs;
			case we_base_constants::MODE_SEE://	No tabs in Super-Easy-Edit_mode
				return $we_tabs;
		}
	}

	private static function getExtraHead(we_contents_root $we_doc): string{
		switch($we_doc->ContentType){
			case we_base_ContentTypes::WEDOCUMENT:
				assert($we_doc instanceof we_document_webEdition);
				return ($we_doc->TemplateID && we_base_permission::hasPerm([we_base_perm::WORK_TEMPLATES_CAN_SEE]) ?
					' - <a class="bold" style="color:#006699" href="javascript:WE().layout.weEditorFrameController.openDocument(WE().consts.tables.TEMPLATES_TABLE,' . $we_doc->TemplateID . ',\'' . we_base_ContentTypes::TEMPLATE . '\');">' . g_l('weClass', '[openTemplate]') . '</a>' :
					'');
			case we_base_ContentTypes::TEMPLATE:
				assert($we_doc instanceof we_contents_template);
				return ($we_doc->MasterTemplateID ?
					' - <a class="bold" style="color:#006699" href="javascript:WE().layout.weEditorFrameController.openDocument(WE().consts.tables.TEMPLATES_TABLE,' . $we_doc->MasterTemplateID . ',\'' . we_base_ContentTypes::TEMPLATE . '\');">' . g_l('weClass', '[openMasterTemplate]') . '</a>' :
					'');
			case we_base_ContentTypes::OBJECT_FILE:
				assert($we_doc instanceof we_contents_objectFile);
				return ($we_doc->TableID && we_base_permission::hasPerm([we_base_perm::OBJECTS_CAN_SEE]) ?
					' - <a class="bold" style="color:#006699" href="javascript:WE().layout.weEditorFrameController.openDocument(WE().consts.tables.OBJECT_TABLE,' . $we_doc->TableID . ',\'' . we_base_ContentTypes::OBJECT . '\');">' . g_l('weClass', '[openObject]') . '</a>' :
					'');
			default:
				return '';
		}
	}

	public static function show(): void{
// init document
		$we_transaction = we_base_request::_(we_base_request::TRANSACTION, 'we_cmd', we_base_request::_(we_base_request::TRANSACTION, 'we_transaction'), 1);
		try{
			$we_doc = we_contents_root::getSessionDocument($we_transaction);
		} catch (we_exception_base $e){
			//fix some error on delete
			return;
		}
		$we_tabs = self::getTabs($we_doc, $we_transaction);
		$extraHead = self::getExtraHead($we_doc);

		['link' => $link, 'class' => $publClass] = self::getDocumentClass($we_doc);

		echo we_html_tools::getHtmlTop('', we_html_element::cssLink(CSS_DIR . 'we_tab.css') .
			we_html_element::jsScript(JS_DIR . 'we_editor_header.js', '', ['id' => 'edHeader', 'data-doc' => setDynamicVar([
					'page' => $we_doc->EditPageNr,
					'contentType' => $we_doc->ContentType,
					'path' => $we_doc->Path,
					'text' => $we_doc->getText(),
					'id' => intval($we_doc->ID),
					'pulishedClass' => $publClass,
			])]), we_html_element::body(array_merge($we_doc->getEditorBodyAttributes(we_contents_root::EDITOR_HEADER), ['id' => 'eHeaderBody']), we_html_element::div(['id' => 'main'], we_html_element::div([
						'id' => 'headrow'], ($we_doc->ContentType ? we_html_element::b(str_replace(' ', '&nbsp;', g_l('contentTypes', '[' . $we_doc->ContentType . ']'))) : '') . ': ' .
						$link .
						'<span id="h_path" class="bold cutText"></span>' . ($link ? '</a>' : '') .
						' (ID: <span id="h_id"></span>)' .
						$extraHead
					) .
					$we_tabs->getHTML())
			)
		);
	}

	/**
	 *
	 * @param we_contents_root $we_doc
	 * @return array{link:string,class:string}
	 */
	public static function getDocumentClass(we_contents_root $we_doc): array{
		switch($we_doc->Table){
			case TEMPLATES_TABLE:
			case VFILE_TABLE:
				$link = '';
				$publState = we_contents_root::STATE_PUBLISHED;
				break;
			case FILE_TABLE:
				$publState = $we_doc->getDocumentState();
				$link = ($we_doc->ID ? '<a href="' . WEBEDITION_DIR . 'we_cmd.php?we_cmd[0]=openBrowser&id=' . $we_doc->ID . '" target="browser">' : '');
				break;
			default:
				$link = '';
				$publState = $we_doc->getDocumentState();
		}
		switch($publState){
			default:
			case we_contents_root::STATE_PUBLISHED:
				$publClass = 'published';
				break;
			case we_contents_root::STATE_MODIFIED:
				$publClass = 'changed';
				break;
			case we_contents_root::STATE_UNPUBLISHED:
				$publClass = 'notpublished';
				break;
		}
		return [
			'link' => $link,
			'class' => $publClass
		];
	}

}
