<?php
declare(strict_types=1);

/**
 * webEdition CMS
 *
 * $Rev: 2a5ff5e6b121 $
 * $Author: Lukas Imhof $
 * $Date: 2020-01-02 18:58 +0100 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
class we_exim_ExportXML extends we_exim_Export{
	protected $permittedContentTypes = [
		we_base_ContentTypes::WEDOCUMENT,
		we_base_ContentTypes::OBJECT_FILE
	];

	function __construct(){
		parent::__construct();
		$this->exportType = we_exim_ExIm::TYPE_XML;
	}

	protected function fileCreate(): bool{
		return (parent::fileCreate() ?
			we_base_file::save($this->exportProperties['file'], we_exim_ExIm::getHeader(we_exim_ExIm::TYPE_XML)) :
			false);
	}

	protected function writeExportItem(we_base_mainmodel $doc, array $attribute = []): void{
		assert(is_resource($this->fh));
		if($doc instanceof we_backup_binary){
			return;
		}
		if($doc instanceof we_contents_root){
			switch($doc->Table){
				case OBJECT_FILES_TABLE:
					assert($doc instanceof we_contents_objectFile);
					$classname = we_exim_Export::correctTagname(fS('SELECT Text FROM ' . OBJECT_TABLE . ' WHERE ID=' . intval($doc->TableID)), 'object');
					fwrite($this->fh, "\t<" . $classname . ">\n");
					$this->object2nonWE($doc, (bool) $this->options['xml_cdata']);
					fwrite($this->fh, "\t</" . $classname . ">\n");
					break;
				case FILE_TABLE:
					if($doc instanceof we_document_textContent){
						$doctypeText = we_exim_Export::correctTagname(fS('SELECT Text FROM ' . DOC_TYPES_TABLE . ' dt WHERE dt.ID=' . intval($doc->DocType)), 'document');
					} else {
						$doctypeText = 'document';
					}
					fwrite($this->fh, "\t<" . $doctypeText . ">\n");
					$this->document2nonWE($doc, (bool) $this->options['xml_cdata']);
					fwrite($this->fh, "\t</" . $doctypeText . ">\n");
					break;
			}
		}
	}

	protected function formatOutput(string $content, string $tagname = '', string $format = we_exim_ExIm::TYPE_XML, int $tabs = 2, bool $cdata = false, bool $fix_content = false): string{
		switch($format){
			case we_exim_ExIm::TYPE_XML:
				// Generate XML output if content is given
				return '<' . $tagname . ($content ?
					'>' . ($fix_content ? ($cdata ? ('<![CDATA[' . $content . ']]>') : htmlspecialchars($content, ENT_QUOTES)) : $content) . '</' . $tagname . ">\n" :
					"/>\n");
			case 'cdata':
				// Generate CDATA XML output if content is given
				return ($content ? '<![CDATA[' . $content . ']]>' : '');
			default:
				return $content;
		}
	}

	protected function remove_from_check_array(array $check_array, string $tagname = ''): array{
		for($i = 0; $i < count($check_array); $i++){
			if(isset($check_array[$i]) && $check_array[$i] == $tagname){
				unset($check_array[$i]);
			}
		}

		return $check_array;
	}

	protected function fileComplete(): void{
		we_base_file::save($this->exportProperties['file'], we_backup_util::weXmlExImFooter, 'ab');
	}

}
