<?php
declare(strict_types=1);

/**
 * webEdition CMS
 *
 * $Rev: 33b8422b6be7 $
 * $Author: Marc Kraemer <marc.kraemer@aldente-it.de> $
 * $Date: 2020-10-08 17:46 +0200 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */

/**
 * class
 * @desc    class for tag <we:listview type='search'>
 *          the difference to the normal listview is, that you can only
 *          display the fields from the index table (tblIndex) which are
 *          Title, Description we_text, we_path
 *
 */
class we_listview_search extends we_listview_base{
	/** @var int[] doctype  */
	var $docType = [];

	/** @var int ID of a class. Search only in Objects of this class */
	var $class = 0;

	/** @var int ID of a document which to use for displaying thr detail page */
	var $triggerID = 0;

	/** @var bool  set to true when a search should be case sensitive  */
	var $casesensitive = false;

	/** @var string[] of Language */
	protected $languages = [];

	/** @var bool */
	var $objectseourls = false;

	/** @var bool */
	var $hidedirindex = false;

	/**
	 *
	 * @desc    constructor of class
	 *
	 * @param   string $name         name of listview
	 * @param   int $rows          number of rows to display per page
	 * @param   int $offset         start offset of first page
	 * @param   string $order          field name(s) to order by
	 * @param   bool $desc           set to true, if order should be descendend
	 * @param   array $docType       doctype
	 * @param   int $class         ID of a class. Search only in Objects of this class
	 * @param   string[] $cats          comma separated categories
	 * @param   bool $catOr         set to true if it should be an 'OR condition'
	 * @param   bool $casesensitive		set to true when a search should be case sensitive
	 * @param   int[] $workspaceID   commaseperated list of id's of workspace
	 * @param   int $cols   		  to display a table this is the number of cols
	 * @param string[] $languages
	 *
	 */
	function __construct(string $name, int $rows, int $offset, string $order, bool $desc, array $docType, int $class, array $cats, bool $catOr, bool $casesensitive, array $workspaceID, int $triggerID, int $cols, string $customerFilterType, array $languages, bool $hidedirindex, bool $objectseourls){
		parent::__construct($name, $rows, $offset, $order, $desc, $cats, $catOr, $workspaceID, $cols, '', '', '', '', [], $customerFilterType);

		$this->triggerID = $triggerID;
		$this->objectseourls = $objectseourls;
		$this->hidedirindex = $hidedirindex;
		$this->languages = $languages;
		$this->docType = $docType;
		$this->class = $class;
		$this->casesensitive = $casesensitive;

		$where_lang = ($this->languages ? ' AND i.Language IN ("' . implode('","', $this->languages) . '") ' : '');

		// correct order
		$orderArr = [];

		$orderArr1 = array_map('trim', explode(',', $this->order));
		$random = (in_array(self::RANDOM_FIELD, $orderArr1));
		$orderArr1 = $random ? [] : $orderArr1;

		foreach($orderArr1 as $o){
			if(trim($o)){
				$foo = preg_split('/ +/', $o);
				$oname = $foo[0];
				$otype = $foo[1] ?? '';
				$orderArr[] = ['oname' => $oname, 'otype' => $otype];
			}
		}
		$this->order = '';
		foreach($orderArr as $o){
			switch(strtolower($o['oname'])){
				case self::DATECREATION_FIELD:
					$this->order .= 'COALESCE(f.Creation_Date' . (we_modules_info::isActive(we_modules_info::OBJECT) ? ',`of`.Creation_Date' : '') . ')' . ((trim(strtolower($o['otype'])) === 'desc' || $desc) ? ' DESC' : '') . ',';
					break;
				case self::DATEMOD_FIELD:
					$this->order .= 'COALESCE(f.Mod_Date' . (we_modules_info::isActive(we_modules_info::OBJECT) ? ',`of`.Mod_Date' : '') . ')' . ((trim(strtolower($o['otype'])) === 'desc' || $desc) ? ' DESC' : '') . ',';
					break;
				case self::FILENAME_FIELD:
					$this->order .= 'COALESCE(f.Filename' . (we_modules_info::isActive(we_modules_info::OBJECT) ? ',`of`.Filename' : '') . ')' . ((trim(strtolower($o['otype'])) === 'desc' || $desc) ? ' DESC' : '') . ',';
					break;
				case self::PATH_FIELD:
				case 'path':
					$this->order .= 'COALESCE(f.Path' . (we_modules_info::isActive(we_modules_info::OBJECT) ? ',`of`.Path' : '') . ')' . ((trim(strtolower($o['otype'])) === 'desc' || $desc) ? ' DESC' : '') . ',';
					break;
				case 'workspace':
					$this->order .= 'wsp.Path' . ((trim(strtolower($o['otype'])) === 'desc' || $desc) ? ' DESC' : '') . ',';
					break;
				case self::ID_FIELD:
				case 'oid':
				case 'did':
				case 'id':
					$this->order .= 'ID' . ((trim(strtolower($o['otype'])) === 'desc' || $desc) ? ' DESC' : '') . ',';
					break;
				case 'title':
				case 'text':
				case 'description':
					$this->order .= $o['oname'] . ((trim(strtolower($o['otype'])) === 'desc' || $desc) ? ' DESC' : '') . ',';
					break;
			}
		}
		$this->order = rtrim($this->order, ',');

		/* if($this->order && $this->desc && (!preg_match('|.+ desc$|i', $this->order))){
		  $this->order .= ' DESC';
		  } */


		$this->search = $this->DB_WE->escape($this->search);

		$cat_tail = ($this->cats ? we_category_category::getCatSQLTail($this->cats, 'i', $this->catOr) : '');

		if($this->docType && $this->class){
			$dtcl_query = ' AND (i.Doctype IN(' . implode(',', $this->docType) . ') OR i.ClassID=' . $this->class . ') ';
		} else if($this->docType){
			$dtcl_query = ' AND i.Doctype IN(' . implode(',', $this->docType) . ') ';
		} else if($this->class){
			$dtcl_query = ' AND i.ClassID=' . $this->class . ' ';
		} else {
			$dtcl_query = '';
		}

		//FIXME: use fulltext index: MATCH(Text) AGAINST([+-]words* IN BINARY MODE)

		$spalte = ($this->casesensitive ? 'BINARY ' : '') . 'i.Text';
		$bOR = $bAND = [];
		foreach(preg_split('/ +/', $this->search) as $v1){
			if(preg_match('|^[-\+]|', $v1)){
				$bAND[] = (preg_match('|^-|', $v1) ? 'NOT ' : '') .
					$spalte . ' LIKE "%' . preg_replace('|^[-\+]|', '', $v1) . '%"';
			} else {
				$bOR[] = $spalte . ' LIKE "%' . $v1 . '%"';
			}
		}

		if($bOR){
			$bAND[] = '(' . implode(' OR ', $bOR) . ')';
		}

		$bedingung_sql = '(' . implode(' AND ', $bAND) . ')';
		$ranking = '(ROUND(MATCH(i.`Text`,i.`Title`,i.`Description`,i.`Keywords`) AGAINST("' . str_replace(['+', '-'], '', $this->search) . '"),3))';
		$ws_where = '';
		if($this->workspaceID){
			$allWsp = we_base_util::pushChildsFromArr($this->workspaceID, FILE_TABLE, true);
			$cond = ['i.WorkspaceID IN (' . implode(',', $allWsp) . ')'];
			$ws_where = ' AND (' . implode(' OR ', $cond) . ')';
		}
		$weDocumentCustomerFilter_tail = (we_modules_info::isActive(we_modules_info::CUSTOMER) ?
			we_customer_documentFilter::getConditionForListviewQuery($this->customerFilterType, $this) :
			'');
//FIXME: remove wsp, if possible
		$where = ' WHERE ' . $bedingung_sql . ' ' . $dtcl_query . ($cat_tail ? ' AND ' . $cat_tail : '') . $ws_where . ' ' . $where_lang . ' ' . $weDocumentCustomerFilter_tail;
//FIXME use window function COUNT(*) OVER() if mariadb 10.2/mysql 8
		$this->anz_all = fI('SELECT COUNT(DISTINCT i.ID,i.ClassID) FROM ' . INDEX_TABLE . ' i LEFT JOIN ' . FILE_TABLE . ' wsp ON wsp.ID=i.WorkspaceID ' . $where);

		$this->DB_WE->query('SELECT
	i.Category,
	i.ID,
	i.ID AS DID,
	i.ID AS OID,
	i.ClassID,
	i.ClassID AS `' . we_listview_base::CLASS_FIELD . '`,
	i.Text,
	COALESCE(wsp.Path,"/") AS Workspace,
	i.WorkspaceID,
	i.Title,
	i.Description,
	COALESCE(f.Path' . (we_modules_info::isActive(we_modules_info::OBJECT) ? ',`of`.Path' : '') . ') AS Path,
	i.Language,
	' . ($random ? 'RAND() ' : $ranking) . ' AS ranking
FROM ' .
			INDEX_TABLE . ' i LEFT JOIN ' .
			FILE_TABLE . ' wsp ON wsp.ID=i.WorkspaceID LEFT JOIN ' .
			FILE_TABLE . ' f ON (i.ID=f.ID AND i.ClassID=0) ' .
			(we_modules_info::isActive(we_modules_info::OBJECT) ? 'LEFT JOIN ' .
				OBJECT_FILES_TABLE . ' `of` ON (i.ID=`of`.ID AND i.ClassID>0) ' : '') .
			$where . '
GROUP BY
	i.ClassID,
	i.ID
ORDER BY
	ranking DESC ' . ($this->order ? (',' . $this->order) : '') .
			(($this->maxItemsPerPage > 0) ? (' LIMIT ' . intval($this->start) . ',' . intval($this->maxItemsPerPage)) : ''));
		$this->anz = $this->DB_WE->num_rows();
	}

	public function next_record(): bool{
		if($this->DB_WE->next_record()){
			$fileData = array_change_key_case(getHash('SELECT * FROM ' .
					($this->DB_WE->f('ClassID') ? OBJECT_FILES_TABLE : FILE_TABLE ) .
					' WHERE ID=' . intval($this->DB_WE->f('ID')) . ' LIMIT 1'));

			$this->Record = array_merge(array_change_key_case($this->DB_WE->getRecord()), self::preFixArrayKeys($fileData));

			if($this->Record['classid'] && $this->objectseourls && show_SeoLinks()){
				$objecttriggerid = ($this->triggerID ?: ($fileData ? $fileData['triggerid'] : 0));

				$path_parts = ($objecttriggerid ?
					pathinfo(id2path($objecttriggerid)) :
					pathinfo(we_http_util::getScriptName())
					);

				$pidstr = ($this->Record['workspaceid'] ? '?pid=' . intval($this->Record['workspaceid']) : '');

				if($this->hidedirindex && seoIndexHide($path_parts['basename'])){
					$this->Record[self::PATH_FIELD] = ($path_parts['dirname'] != '/' ? $path_parts['dirname'] : '') .
						(!empty($fileData['url']) ?
						'/' . $fileData['url'] . $pidstr :
						'/?' . WEOBJ_PARAM . '=' . $this->Record['id'] . str_replace('?', '&amp;', $pidstr));
				} else {
					$this->Record[self::PATH_FIELD] = (!empty($fileData['url']) ?
						($path_parts['dirname'] != '/' ? $path_parts['dirname'] : '') . '/' . $path_parts['filename'] . '/' . $fileData['url'] . $pidstr :
						we_http_util::getScriptName() . '?' . WEOBJ_PARAM . '=' . $this->Record['id'] . str_replace('?', '&amp;', $pidstr));
				}
				$this->Record[self::TRIGGER_FIELD] = $objecttriggerid;
			}
			$this->count++;
			return true;
		}

		$this->stop_next_row = $this->shouldPrintEndTR();
		if($this->cols && ($this->count <= $this->maxItemsPerPage) && !$this->stop_next_row){
			$this->Record = [];
			$this->count++;
			return true;
		}

		return false;
	}

	function f(string $key, bool $stripWE = false): string{
		return parent::f($key, $stripWE);
	}

	public function getCustomerRestrictionQuery(int $classID): array{
		return ['', 'modelTable IN ("tblFile","tblObjectFiles") AND '];
	}

	public function getTableID(): int{
		return intval($this->f(we_listview_base::CLASS_FIELD));
	}

	public function useSeoUrls(): bool{
		return $this->objectseourls;
	}

}
