<?php
declare(strict_types=1);

/**
 * webEdition CMS
 *
 * $Rev: aa01842ef312 $
 * $Author: Marc Kraemer <marc.kraemer@aldente-it.de> $
 * $Date: 2020-04-15 10:54 +0200 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */

/**
 * class    we_listview_object
 * @desc    class for tag <we:listview type="object">
 *
 */
class we_listview_sql extends we_listview_base{

	/**
	 * @desc    constructor of class
	 *
	 * @param   string $name          name of listview
	 * @param   int $rows          number of rows to display per page
	 * @param   int $offset        start offset of first page
	 * @param   string $order         field name(s) to order by
	 * @param   bool $desc          boolean - set to true, if order should be descendend
	 *
	 */
	public function __construct(string $name, int $rows, int $offset, string $order, bool $desc, string $select, string $from, string $where, string $predefinedSQL){
		parent::__construct($name, $rows, $offset, $order, $desc);

		if($this->order && $this->desc && (!preg_match('|.+ desc$|i', $this->order))){
			$this->order .= ' DESC';
		}

		$predefinedSQL = $predefinedSQL ?: 'SELECT ' . $select . ' FROM ' . $from . ' WHERE ' . $where;

		$this->DB_WE->query($predefinedSQL);
//FIXME use window function COUNT(*) OVER() if mariadb 10.2/mysql 8
		$this->anz_all = $this->DB_WE->num_rows();

		$this->DB_WE->query($predefinedSQL .
			($this->order ? ' ORDER BY ' . $this->order : '') .
			(($this->maxItemsPerPage) ? (' LIMIT ' . $this->start . ',' . $this->maxItemsPerPage) : ''));
		$this->anz = $this->DB_WE->num_rows();
	}

	public function next_record(): bool{
		if($this->DB_WE->next_record()){
			$this->count++;
			$this->Record = $this->DB_WE->getRecord() + [
				we_listview_base::ID_FIELD => $this->count
			];

			return true;
		}
		$this->stop_next_row = $this->shouldPrintEndTR();
		if($this->cols && ($this->count <= $this->maxItemsPerPage) && !$this->stop_next_row){
			$this->Record = [];
			$this->count++;
			return true;
		}
		return false;
	}

	function f(string $key, bool $stripWE = false): string{
		return ($this->Record[$key] ?? '');
	}

}
