<?php
declare(strict_types=1);

/**
 * webEdition CMS
 *
 * $Rev: 6cc64d37551c $
 * $Author: Marc Kraemer <marc.kraemer@aldente-it.de> $
 * $Date: 2020-07-25 11:32 +0200 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
class we_search_base{
	/** @var we_database_base */
	protected $db;

	/** @var int */
	var $rows = -1;

	/** @var int */
	var $start = 0;

	/** @var string */
	protected $order = '';

	/** @var bool */
	var $desc = false;

	/** @var string */
	protected $table = '';

	/** @var int */
	var $defaultanzahl = 10;

	/** @var string */
	protected $where;

	/** @var string */
	protected $get;

	/** @var string */
	protected $Order;

	/** @var int */
	var $anzahl = 10;

	/** @var int */
	var $maxItems = 0;

	/** @var int */
	var $searchstart = 0;

	/** @var array<string,mixed> */
	protected $Record = [];

	const OP_IS = 'IS';
	const OP_CONTAIN = 'CONTAIN';
	const OP_START = 'START';
	const OP_END = 'END';
	const OP_LO = 'LO';
	const OP_LEQ = 'LEQ';
	const OP_HEQ = 'HEQ';
	const OP_HI = 'HI';
	const OP_IN = 'IN';

	function __construct(){
		$this->db = new DB_WE();
	}

	function init(array $sessDat = []): void{
		t_e('we_search_base::init() called although it seemed obsolete');
	}

	// select expects an array with 	Searchid['field'] => "searchstring"
	// 									Searchid['search'] => "searchstring"
	//									Searchid['concat'] => "AND"
	//																("OR","XOR")
	//									Searchid['type'] => "START"
	//																("IS","END","CONTAIN","<","<=",">",..)

	/**
	 *
	 * @param array<int,string|array> $searchname
	 * @param string[] $searchfields
	 * @param string[] $searchlocation
	 * @param string $tablename
	 * @param int $rows
	 * @param int $start
	 * @param string $order
	 * @param bool $desc
	 * @param bool $fieldsQualified
	 * @return string
	 */
	function searchfor(array $searchname, array $searchfields, array $searchlocation, string $tablename, string $alias = '', int $rows = -1, int $start = 0, string $order = '', bool $desc = false, bool $fieldsQualified = false): string{
		$tablename = $fieldsQualified ? '' : ($tablename ? $tablename . '.' : ($alias ? $alias . '.' : ''));
		$allSql = [];

		foreach($searchfields as $i => $searchfieldCsv){
			'@phan-var string $searchfieldCsv';
			$sql = [];
			foreach(explode(',', $searchfieldCsv) as $searchfield){
				$regs = explode('_', $searchfield, 2);
				switch((count($regs) === 2 ? str_replace($tablename, '', $regs[0]) : '')){
					case 'checkbox':
						$sql[] = $this->sqlwhere($tablename . $searchfield, '=' . intval($searchname[$i]) . ' ');
						break;
					case 'date':
						if($searchname[$i]){
							$date = new DateTime((string) $searchname[$i]);
							$op = '';
							switch($searchlocation[$i]){
								case self::OP_IS:
									$op = '=';
								case self::OP_LO:
									$op = $op ?: '<';
								case self::OP_LEQ:
									$op = $op ?: '<=';
								case self::OP_HEQ:
									$op = $op ?: '>=';
								case self::OP_HI:
									$op = $op ?: '>';
									$sql[] = $this->sqlwhere($tablename . $searchfield, ' ' . $op . ' "' . $date->format(we_database_base::DATE_FORMAT) . '" ');
									break;
								case self::OP_IN:
								default:
									$sql[] = $this->sqlwhere($tablename . $searchfield, ' BETWEEN "' . $date->format(we_database_base::DATE_FORMAT) . '" AND ("' . $date->format(we_database_base::DATE_FORMAT) . '"+INTERVAL 1 DAY)');
									break;
							}
						}
						break;
					default:
						$forName = (string) $searchname[$i];
						if($forName){
							switch($searchlocation[$i]){
								case self::OP_END:
									$searching = ' LIKE "%' . $this->db->escape($forName) . '" ';
									$sql[] = $this->sqlwhere($tablename . $searchfield, $searching);
									break;
								case self::OP_START:
									$searching = ' LIKE "' . $this->db->escape($forName) . '%" ';
									$sql[] = $this->sqlwhere($tablename . $searchfield, $searching);
									break;
								case self::OP_IN:
									$tmp = array_map('trim', explode(',', $forName));
									$searching = ' IN ("' . implode('","', $tmp) . '") ';
									$sql[] = $this->sqlwhere($tablename . $searchfield, $searching);
									break;
								case self::OP_IS:
									$sql[] = $this->sqlwhere($tablename . $searchfield, '="' . $this->db->escape($forName) . '" ');
									break;
								case self::OP_LO:
									$searching = ' < "' . $this->db->escape($forName) . '" ';
									$sql[] = $this->sqlwhere($tablename . $searchfield, $searching);
									break;
								case self::OP_LEQ:
									$searching = ' <= "' . $this->db->escape($forName) . '" ';
									$sql[] = $this->sqlwhere($tablename . $searchfield, $searching);
									break;
								case self::OP_HI:
									$searching = ' > "' . $this->db->escape($forName) . '" ';
									$sql[] = $this->sqlwhere($tablename . $searchfield, $searching);
									break;
								case self::OP_HEQ:
									$searching = ' >= "' . $this->db->escape($forName) . '" ';
									$sql[] = $this->sqlwhere($tablename . $searchfield, $searching);
									break;
								default:
									$searching = ' LIKE "%' . $this->db->escape($forName) . '%" ';
									$sql[] = $this->sqlwhere($tablename . $searchfield, $searching);
									break;
							}
						}
				}
			}
			$allSql[] = '(' . implode(' OR ', $sql) . ')';
		}

		return implode(' AND ', $allSql);
	}

	protected function sqlwhere(string $we_SearchField, string $searchlocation): string{
		if(strpos($we_SearchField, ',') !== false){
			$foo = array_filter(explode(',', $we_SearchField));
			$q = [];
			foreach($foo as $f){
				$tmp = str_replace('.', '.`', $f);
				if($tmp == $f){
					$tmp = '`' . $tmp;
				}
				$q[] = $tmp . '` ' . $searchlocation;
			}
			return ' ( ' . implode(' OR ', $q) . ' ) ';
		}
		$tmp = str_replace('.', '.`', $we_SearchField);
		if($tmp == $we_SearchField){
			$tmp = '`' . $tmp;
		}
		return $tmp . '` ' . $searchlocation . ' ';
	}

	function countitems(string $where = '', string $table = ''): int{
		$this->table = ($table ?: ($this->table ?: ''));

		if(!$this->table){
			return -1;
		}
		$this->where = ($where ?: ($this->where ?: '1'));
		return fI('SELECT COUNT(1) FROM ' . SEARCHRESULT_TABLE . ' AS s WHERE s.UID=' . we_users_user::getID());
	}

	function setlimit(int $anzahl = 0, int $searchstart = 0): string{
		$this->anzahl = ($anzahl ?: ($this->anzahl ?: $this->defaultanzahl));
		$this->searchstart = ($searchstart ?: ($this->searchstart ?: '0'));


		return ' ' . $this->searchstart . ',' . $this->anzahl . ' ';
	}

	function setstart(int $z): void{
		$this->searchstart = $z;
	}

	function setorder(string $z): void{
		$this->Order = $z;
	}

	function setwhere(string $z): void{
		$this->where = $z;
	}

	function settable(string $z): void{
		$this->table = $z;
	}

	function setget(string $z): void{
		$this->get = $z;
	}

	static function getLocationDate(string $name = 'locationField', string $select = '', int $size = 1): string{
		return we_html_tools::select($name, we_search_search::getLocation('date'), $size, $select);
	}

	static function getLocationMeta(string $name = 'locationField', string $select = '', int $size = 1): string{
		return we_html_tools::select($name, we_search_search::getLocation('meta'), $size, $select);
	}

	function getNextPrev(int $count): string{
		$out = '<table class="default">
<tr>
	<td>' .
			($this->searchstart ?
			we_html_button::create_button(we_html_button::BACK, 'javascript:back();') : //bt_back
			we_html_button::create_button(we_html_button::BACK, '', '', '', true)
			) . '
	</td>
	<td class="defaultfont"><b>' . (($count) ? $this->searchstart + 1 : 0) . '-' .
			(($count - $this->searchstart) < $this->anzahl ?
			$count :
			$this->searchstart + $this->anzahl) .
			' ' . g_l('global', '[from]') . ' ' . $count . '</b></td>
	<td>' .
			(($this->searchstart + $this->anzahl) < $count ?
			we_html_button::create_button(we_html_button::NEXT, "javascript:next();") : //bt_back

			we_html_button::create_button(we_html_button::NEXT, "", '', "", true)) .
			'</td>
	<td>';

		$pages = [];
		$maxPages = ceil($count / $this->anzahl);
		for($i = 0; $i < $maxPages; $i++){
			$pages[($i * $this->anzahl)] = ($i + 1);
		}

		$page = ceil($this->searchstart / $this->anzahl) * $this->anzahl;

		$select = we_html_tools::select('page', $pages, 1, $page, false, ['onchange' => "this.form.elements.SearchStart.value = this.value;we_cmd('reload_editpage');"]);
		if(!defined('SearchStart')){//we need this, since pager is shown above & under the results
			define("SearchStart", true);
			$out .= we_html_element::hidden('SearchStart', $this->searchstart);
		}

		return $out . $select . '</td></tr></table>';
	}

	function getWorkspaces(array $foo, int $len): string{
		return '';
	}

}
