<?php
declare(strict_types=1);

/**
 * webEdition CMS
 *
 * $Rev: 869defb635b1 $
 * $Author: Marc Kraemer <marc.kraemer@aldente-it.de> $
 * $Date: 2020-05-27 16:27 +0200 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */

/**
 * class representing the model data of the search
 */
class we_search_model extends we_search_modelBase{
	/** @var int default ParentId in which own searches are saved	 */
	public $ParentID = 8;

	/** @var int */
	public $mode = 0;

	/** @var int position to start the search	 */
	protected $searchstartDocSearch = 0;

	/** @var int */
	protected $searchstartTmplSearch = 0;

	/** @var int */
	protected $searchstartMediaSearch = 0;

	/** @var int */
	protected $searchstartAdvSearch = 0;

	/** @var string[] includes the text to search for */
	protected $searchDocSearch = [];

	/** @var string[] includes the text to search for */
	protected $searchTmplSearch = [];

	/** @var string[] includes the text to search for */
	protected $searchMediaSearch = [];

	/** @var string[] includes the text to search for */
	protected $searchAdvSearch = [];

	/** @var string[] includes the operators	 */
	protected $locationDocSearch = [];

	/** @var string[] includes the operators	 */
	protected $locationTmplSearch = [];

	/** @var string[] includes the operators	 */
	protected $locationMediaSearch = [];

	/** @var string[] includes the operators	 */
	protected $locationAdvSearch = [];

	/** @var bool flag that shows what you are searching for in the docsearch	 */
	protected $searchForTextDocSearch = true;

	/** @var bool flag  */
	protected $searchForTitleDocSearch = false;

	/** @var bool flag */
	protected $searchForContentDocSearch = false;

	/** @var bool flag that shows what you are searching for in the tmplsearch	 */
	protected $searchForTextTmplSearch = true;

	/** @var bool */
	protected $searchForContentTmplSearch = false;

	/** @var bool flag that shows what you are searching for in the mediaearch	 */
	protected $searchForTextMediaSearch = true;

	/** @var bool flag that shows what you are searching for in the mediaearch	 */
	protected $searchForTitleMediaSearch = false;

	/** @var bool flag that shows what you are searching for in the mediaearch	 */
	protected $searchForMetaMediaSearch = false;

	/** @var bool flag that shows what you are searching for in the mediaearch	 */
	protected $searchForImageMediaSearch = true;

	/** @var bool flag that shows what you are searching for in the mediaearch	 */
	protected $searchForVideoMediaSearch = false;

	/** @var bool flag that shows what you are searching for in the mediaearch	 */
	protected $searchForAudioMediaSearch = false;

	/** @var bool flag that shows what you are searching for in the mediaearch	 */
	protected $searchForOtherMediaSearch = false;

	/** @var array<string,bool> shows which tables you have to search in in the advsearch	 */
	protected $search_tables_advSearch = [];

	/** @var int folder-ids of the docsearch and the tmplsearch	 */
	protected $folderIDDoc = 0;

	/** @var int */
	protected $folderIDTmpl = 0;

	/** @var int */
	protected $folderIDMedia = 0;

	/** @var string flag that shows what view is set in each search	 */
	protected $setViewDocSearch = we_search_view::VIEW_LIST;

	/** @var string */
	protected $setViewTmplSearch = we_search_view::VIEW_LIST;

	/** @var string */
	protected $setViewMediaSearch = we_search_view::VIEW_LIST;

	/** @var string */
	protected $setViewAdvSearch = we_search_view::VIEW_LIST;

	/** @var int gives the number of entries in each search for one page	 */
	protected $anzahlDocSearch = 10;

	/** @var int */
	protected $anzahlTmplSearch = 10;

	/** @var int */
	protected $anzahlMediaSearch = 10;

	/** @var int */
	protected $anzahlAdvSearch = 10;

	/**
	 * @var int gives the number of entries in each search for one page
	 */
	protected $anzahlMedialinksDocSearch = 0;

	/** @var int */
	protected $anzahlMedialinksTmplSearch = 0;

	/** @var int */
	protected $anzahlMedialinksMediaSearch = 10;

	/** @var int */
	protected $anzahlMedialinksAdvSearch = 0;

	/** @var string gives the order	 */
	protected $OrderDocSearch = 'Text';

	/** @var string gives the order	 */
	protected $OrderTmplSearch = 'Text';

	/** @var string gives the order	 */
	protected $OrderMediaSearch = 'Text';

	/** @var string gives the order	 */
	protected $OrderAdvSearch = 'Text';

	/** @var string[] includes the searchfiels which you are searching in	 */
	protected $searchFieldsDocSearch = [];

	/** @var string[] includes the searchfiels which you are searching in	 */
	protected $searchFieldsTmplSearch = [];

	/** @var string[] includes the searchfiels which you are searching in	 */
	protected $searchFieldsMediaSearch = [];

	/** @var string[] includes the searchfiels which you are searching in	 */
	protected $searchFieldsAdvSearch = [];

	/** @var string[] includes the searchfiels which you are searching in	 */
	protected $searchTablesDocSearch = [];

	/** @var string[] includes the searchfiels which you are searching in	 */
	protected $searchTablesTmplSearch = [];

	/** @var string[] includes the searchfiels which you are searching in	 */
	protected $searchTablesMediaSearch = [];

	/** @var string[] */
	protected $searchTablesAdvSearch = [];

	/** @var array<string,bool> */
	protected $searchForFieldTmplSearch = [
		'text' => false,
		'title' => false,
		'content' => false,
		'meta' => false,
	];

	/** @var array<string,bool> */
	protected $searchForFieldDocSearch = [
		'text' => false,
		'title' => false,
		'content' => false,
		'meta' => false,
	];

	/** @var array<string,bool> */
	protected $searchForFieldMediaSearch = [
		'text' => false,
		'title' => false,
		'content' => false,
		'meta' => false,
	];

	/** @var array<string,bool> */
	protected $searchForContentTypeMediaSearch = [
		'image' => false,
		'audio' => false,
		'video' => false,
		'other' => false,
	];

	/**
	 * Default Constructor
	 * Can load or create new searchtool object depends of parameter
	 */
	public function __construct(int $id = 0){
		parent::__construct(SEARCH_TABLE);
		$this->persistent_slots = array_merge($this->persistent_slots, [
			'ID' => we_base_request::INT,
			'ParentID' => we_base_request::INT,
			'IsFolder' => we_base_request::BOOL,
			'Path' => we_base_request::STRING,
			'Text' => we_base_request::STRING,
			'predefined' => we_base_request::BOOL,
			'activTab' => we_base_request::INT,
			'anzahlDocSearch' => we_base_request::INT,
			'anzahlTmplSearch' => we_base_request::INT,
			'anzahlAdvSearch' => we_base_request::INT,
			'anzahlMediaSearch' => we_base_request::INT,
			'anzahlMedialinksMediaSearch' => we_base_request::INT,
			'folderIDDoc' => we_base_request::INT,
			'folderIDTmpl' => we_base_request::INT,
			'folderIDMedia' => we_base_request::INT,
			'locationAdvSearch' => we_base_request::SERIALIZED_JSON,
			'locationMediaSearch' => we_base_request::SERIALIZED_JSON,
			'OrderDocSearch' => we_base_request::STRING,
			'OrderTmplSearch' => we_base_request::STRING,
			'OrderAdvSearch' => we_base_request::STRING,
			'OrderMediaSearch' => we_base_request::STRING,
			'searchDocSearch' => we_base_request::SERIALIZED_JSON,
			'searchTmplSearch' => we_base_request::SERIALIZED_JSON,
			'searchMediaSearch' => we_base_request::SERIALIZED_JSON,
			'searchAdvSearch' => we_base_request::SERIALIZED_JSON,
			'searchFieldsAdvSearch' => we_base_request::SERIALIZED_JSON,
			'searchFieldsMediaSearch' => we_base_request::SERIALIZED_JSON,
			'searchForTextDocSearch' => we_base_request::BOOL,
			'searchForTitleDocSearch' => we_base_request::BOOL,
			'searchForContentDocSearch' => we_base_request::BOOL,
			'searchForTextTmplSearch' => we_base_request::BOOL,
			'searchForContentTmplSearch' => we_base_request::BOOL,
			'searchForTextMediaSearch' => we_base_request::BOOL,
			'searchForTitleMediaSearch' => we_base_request::BOOL,
			'searchForMetaMediaSearch' => we_base_request::BOOL,
			'searchForImageMediaSearch' => we_base_request::BOOL,
			'searchForAudioMediaSearch' => we_base_request::BOOL,
			'searchForVideoMediaSearch' => we_base_request::BOOL,
			'searchForOtherMediaSearch' => we_base_request::BOOL,
			'search_tables_advSearch' => we_base_request::SERIALIZED_JSON,
			'setViewDocSearch' => we_base_request::STRING,
			'setViewTmplSearch' => we_base_request::STRING,
			'setViewAdvSearch' => we_base_request::STRING,
			'setViewMediaSearch' => we_base_request::STRING,
		]);
		$this->ID = $id;
		if($id){
			$this->load($id);
		}
	}

	public function initByHttp(string $whichSearch = '', bool $isWeCmd = true): void{
		$modelVars = array_merge([// some vars are not persistent in db but must be written to session anyway: must COMPLETE!!
			'searchstartDocSearch' => we_base_request::INT,
			'searchstartTmplSearch' => we_base_request::INT,
			'searchstartMediaSearch' => we_base_request::INT,
			'searchstartAdvSearch' => we_base_request::INT,
			], $this->persistent_slots);
		// prepare searchFields, location and search(text) to read as array
		if($isWeCmd){
			// FIXME: temporary solution
			foreach($_REQUEST['we_cmd'] as $k => $v){
				if(stristr((string) $k, 'search' . $whichSearch . '[')){
					$_REQUEST['we_cmd'][(string) $k] = $v ?: we_base_request::NOT_VALID_S;
				}
			}

			$request = we_base_request::stringList('we_cmd');
			foreach($request as $k => $v){
				if(is_numeric($k)){
					continue;
				}
				if(stristr($k, 'searchFields' . $whichSearch . '[') && !stristr($k, 'hidden_')){
					$_REQUEST['we_cmd']['searchFields' . $whichSearch][] = $v;
				}
				if(stristr($k, 'location' . $whichSearch . '[')){
					$_REQUEST['we_cmd']['location' . $whichSearch][] = $v;
				}
				if(stristr($k, 'search' . $whichSearch . '[')){
					$_REQUEST['we_cmd']['search' . $whichSearch][] = $v;
				}
				if($v == 1 || $v == 0){
					switch($k){
						case 'search_tables_advSearch[' . FILE_TABLE:
							$_REQUEST['we_cmd']['search_tables_advSearch'][FILE_TABLE] = $v;
							break;
						case 'search_tables_advSearch[' . TEMPLATES_TABLE :
							$_REQUEST['we_cmd']['search_tables_advSearch'][TEMPLATES_TABLE] = $v;
							break;
						case 'search_tables_advSearch[' . VERSIONS_TABLE :
							$_REQUEST['we_cmd']['search_tables_advSearch'][VERSIONS_TABLE] = $v;
							break;
						case 'search_tables_advSearch[' . OBJECT_FILES_TABLE :
							$_REQUEST['we_cmd']['search_tables_advSearch'][OBJECT_FILES_TABLE] = $v;
							break;
						case 'search_tables_advSearch[' . OBJECT_TABLE :
							$_REQUEST['we_cmd']['search_tables_advSearch'][OBJECT_TABLE] = $v;
							break;
					}
				}
			}
			foreach($modelVars as $val => $type){
				//in request these var's are transported as request arrays
				$type = ($type === we_base_request::SERIALIZED_JSON ? we_base_request::STRING : $type);
				$this->{$val} = we_base_request::_($type, 'we_cmd', $this->{$val}, $val);
			}
		} else {
			// FIXME: temporary solution
			foreach([we_search_view::SEARCH_DOCS, we_search_view::SEARCH_TMPL, we_search_view::SEARCH_ADV, we_search_view::SEARCH_MEDIA] as $lastsearch){
				if(!empty($_REQUEST['search' . $lastsearch])){
					foreach($_REQUEST['search' . $lastsearch] as $k => $v){
						$_REQUEST['search' . $lastsearch][$k] = $v ?: we_base_request::NOT_VALID_S;
					}
				}
			}

			foreach($modelVars as $val => $type){
				//in request these var's are transported as request arrays
				$type = ($type === we_base_request::SERIALIZED_JSON ? we_base_request::STRING : $type);
				$this->{$val} = we_base_request::_($type, $val, $this->{$val});
			}
		}
		$this->prepareModelForSearch($whichSearch);
	}

	private function getSearchFromTab(): string{
		switch($this->activTab){
			default:
			case 1:
				return we_search_view::SEARCH_DOCS;
			case 2:
				return we_search_view::SEARCH_TMPL;
			case 3:
				return we_search_view::SEARCH_ADV;
			case 5:
				return we_search_view::SEARCH_MEDIA;
		}
	}

	public function prepareModelForSearch(string $whichSearch = ''): void{
		$whichSearch = $whichSearch ?: $this->getSearchFromTab();

		switch($whichSearch){
			case we_search_view::SEARCH_DOCS:
				// process some SEARCH_DOCS specialties
				if(($tmpSearch = $this->searchDocSearch)){
					$this->searchFieldsDocSearch = $this->locationDocSearch = $this->searchDocSearch = [];
					foreach(['Text' => $this->searchForTextDocSearch, 'Title' => $this->searchForTitleDocSearch, 'Content' => $this->searchForContentDocSearch] as $field => $val){
						if($val){
							$this->searchFieldsDocSearch[] = $field;
							$this->locationDocSearch[] = we_search_base::OP_CONTAIN;
							$this->searchDocSearch[] = implode('', $tmpSearch);
						}
					}
				}
				$this->searchForFieldDocSearch = [
					'text' => $this->searchForTextDocSearch,
					'title' => $this->searchForTitleDocSearch,
					'content' => $this->searchForContentDocSearch,
					'meta' => false,
				];

				// write current set
				$this->currentSearchTables = $this->searchTablesDocSearch = [FILE_TABLE];
				$this->currentSearchFields = $this->searchFieldsDocSearch;
				$this->currentSearchForField = $this->searchForFieldDocSearch;
				$this->currentLocation = $this->locationDocSearch;
				$this->currentSearch = $this->searchDocSearch;
				$this->currentFolderID = $this->folderIDDoc;
				$this->currentOrder = $this->OrderDocSearch;
				$this->currentSetView = $this->setViewDocSearch;
				$this->currentSearchstart = $this->searchstartDocSearch;
				$this->currentAnzahl = $this->anzahlDocSearch;
				break;
			case we_search_view::SEARCH_TMPL:
				if(($tmpSearch = $this->searchTmplSearch)){
					$this->searchFieldsTmplSearch = $this->locationTmplSearch = $this->searchTmplSearch = [];
					foreach(['Text' => $this->searchForTextTmplSearch, 'Content' => $this->searchForContentTmplSearch] as $field => $val){
						if($val){
							$this->searchFieldsTmplSearch[] = $field;
							$this->locationTmplSearch[] = we_search_base::OP_CONTAIN;
							$this->searchTmplSearch[] = implode('', $tmpSearch);
						}
					}
				}
				$this->searchForFieldTmplSearch = [
					'text' => $this->searchForTextTmplSearch,
					'title' => false,
					'content' => $this->searchForContentTmplSearch,
					'meta' => false,
				];

				// write current set
				$this->currentSearchTables = $this->searchTablesTmplSearch = [TEMPLATES_TABLE];
				$this->currentSearchFields = $this->searchFieldsTmplSearch;
				$this->currentSearchForField = $this->searchForFieldTmplSearch;
				$this->currentLocation = $this->locationTmplSearch;
				$this->currentSearch = $this->searchTmplSearch;
				$this->currentFolderID = $this->folderIDTmpl;
				$this->currentOrder = $this->OrderTmplSearch;
				$this->currentSetView = $this->setViewTmplSearch;
				$this->currentSearchstart = $this->searchstartTmplSearch;
				$this->currentAnzahl = $this->anzahlTmplSearch;
				break;
			case we_search_view::SEARCH_MEDIA:
				// process some SEARCH_MEDIA specialties
				$this->searchForFieldMediaSearch = ['text' => $this->searchForTextMediaSearch,
					'title' => $this->searchForTitleMediaSearch,
					'content' => false,
					'meta' => $this->searchForMetaMediaSearch,
				];
				$this->searchForContentTypeMediaSearch = [
					'image' => $this->searchForImageMediaSearch,
					'audio' => $this->searchForAudioMediaSearch,
					'video' => $this->searchForVideoMediaSearch,
					'other' => $this->searchForOtherMediaSearch,
				];

				// write current set
				$this->currentSearchTables = $this->searchTablesMediaSearch = [FILE_TABLE];
				$this->currentSearchFields = $this->searchFieldsMediaSearch;
				$this->currentLocation = $this->locationMediaSearch;
				$this->currentSearch = $this->searchMediaSearch;
				$this->currentFolderID = $this->folderIDMedia;
				$this->currentOrder = $this->OrderMediaSearch;
				$this->currentSetView = $this->setViewMediaSearch;
				$this->currentSearchstart = $this->searchstartMediaSearch;
				$this->currentAnzahl = $this->anzahlMediaSearch;
				$this->currentAnzahlMedialinks = $this->anzahlMedialinksMediaSearch;
				$this->currentSearchForField = $this->searchForFieldMediaSearch;
				$this->currentSearchForContentType = $this->searchForContentTypeMediaSearch;
				break;
			case we_search_view::SEARCH_ADV:
				// process some SEARCH_ADV specialties
				$tmp = [// default db entry
					FILE_TABLE => true,
					TEMPLATES_TABLE => false,
					OBJECT_FILES_TABLE => true,
					OBJECT_TABLE => false,
					VERSIONS_TABLE => false,
				];

				$this->searchTablesAdvSearch = [];
				foreach($this->search_tables_advSearch as $k => $v){
					switch($k){
						case FILE_TABLE:
						case stripTblPrefix(FILE_TABLE): // in older predefined searches we have hardcoded tables without prefix)
							$tmp[FILE_TABLE] = $v;
							if($v && we_base_permission::hasPerm([we_base_perm::WORK_DOCUMENTS_CAN_SEE])){
								$this->searchTablesAdvSearch[] = FILE_TABLE; // this is used in search: so check perms!
							}
							break;
						case TEMPLATES_TABLE :
						case stripTblPrefix(TEMPLATES_TABLE):
							$tmp[TEMPLATES_TABLE] = $v;
							if($v && we_base_permission::hasPerm([we_base_perm::WORK_TEMPLATES_CAN_SEE]) && we_editor_functions::isNormalMode()){
								$this->searchTablesAdvSearch[] = TEMPLATES_TABLE;
							}
							break;
						case OBJECT_FILES_TABLE :
						case stripTblPrefix(OBJECT_FILES_TABLE) :
							$tmp[OBJECT_FILES_TABLE] = $v;
							if($v && we_base_permission::hasPerm([we_base_perm::OBJECTFILES_CAN_SEE])){
								$this->searchTablesAdvSearch[] = OBJECT_FILES_TABLE;
							}
							break;
						case OBJECT_TABLE :
						case stripTblPrefix(OBJECT_TABLE):
							$tmp[OBJECT_TABLE] = $v;
							if($v && we_base_permission::hasPerm([we_base_perm::OBJECTS_CAN_SEE]) && we_editor_functions::isNormalMode()){
								$this->searchTablesAdvSearch[] = OBJECT_TABLE;
							}
							break;
						case VERSIONS_TABLE:
						case stripTblPrefix(VERSIONS_TABLE):
							$tmp[VERSIONS_TABLE] = $v;
							if($v && we_base_permission::hasPerm([we_base_perm::VERSIONS_CAN_SEE])){
								$this->searchTablesAdvSearch[] = VERSIONS_TABLE;
							}
							break;
					}
				}
				$this->search_tables_advSearch = $tmp;

				// write current set
				$this->currentSearchTables = $this->searchTablesAdvSearch;
				$this->currentSearchFields = $this->searchFieldsAdvSearch;
				$this->currentLocation = $this->locationAdvSearch;
				$this->currentSearch = $this->searchAdvSearch;
				$this->currentOrder = $this->OrderAdvSearch;
				$this->currentSetView = $this->setViewAdvSearch;
				$this->currentSearchstart = $this->searchstartAdvSearch;
				$this->currentAnzahl = $this->anzahlAdvSearch;
		}

		$this->currentSearch = array_map(function($v){
			return $v === we_base_request::NOT_VALID_S ? '' : $v;
		}, $this->currentSearch);
	}

	public function setPredefinedSearch($tab = 3, string $keyword = '', int $tables = 0): void{
		// set activTab
		$this->activTab = $tab;

		switch($tables){
			default:
			case we_search_search::TABLE_FILE:
				$whichSearch = we_search_view::SEARCH_DOCS;
				break;
			case we_search_search::TABLE_TEMPLATES:
				$whichSearch = we_search_view::SEARCH_TMPL;
				break;
			case we_search_search::TABLE_OBJECTFILES:
				$whichSearch = we_search_view::SEARCH_ADV;
				break;
			case we_search_search::TABLE_OBJECT:
				$whichSearch = we_search_view::SEARCH_CLASS;
				break;
			case we_search_search::TABLE_VERSION:
				$whichSearch = we_search_view::SEARCH_VERSION;
		}


		// set SEARCH_ADV tables
		$this->search_tables_advSearch = [
			FILE_TABLE => false,
			TEMPLATES_TABLE => false,
			OBJECT_FILES_TABLE => false,
			OBJECT_TABLE => false,
			VERSIONS_TABLE => false,
		];

		// set searchfields
		switch($whichSearch){ // FIXME: make fn tabToWhichsearch()
			case we_search_view::SEARCH_DOCS://Doc
				$this->search_tables_advSearch[FILE_TABLE] = true;
				$this->searchForTextDocSearch = $this->searchForTitleDocSearch = $this->searchForContentDocSearch = true; // FIXME: make this default

				if($keyword){
					$this->searchDocSearch = [$keyword];
					$this->searchFieldsDocSearch = ['Filename', 'Title', 'Content'];
					$this->locationDocSearch = [we_search_base::OP_CONTAIN, we_search_base::OP_CONTAIN, we_search_base::OP_CONTAIN];

					$this->searchAdvSearch = [$keyword, $keyword, $keyword];
					$this->searchFieldsAdvSearch = ['Filename', 'Title', 'Content'];
					$this->locationAdvSearch = [we_search_base::OP_CONTAIN, we_search_base::OP_CONTAIN, we_search_base::OP_CONTAIN];

					$this->searchMediaSearch[0] = $keyword;
					$this->locationMediaSearch[0] = we_search_base::OP_CONTAIN;
					$this->searchFieldsMediaSearch[0] = 'keyword';
				}

				// FIXME: make this default
				$this->searchForTextMediaSearch = $this->searchForTitleMediaSearch = $this->searchForMetaMediaSearch = true;
				$this->searchForImageMediaSearch = $this->searchForAudioMediaSearch = $this->searchForVideoMediaSearch = $this->searchForOtherMediaSearch = true;
				break;
			case we_search_view::SEARCH_TMPL://Templ
				$this->search_tables_advSearch[TEMPLATES_TABLE] = true;
				$this->searchForTextTmplSearch = $this->searchForContentTmplSearch = true;

				if($keyword){
					$this->searchTmplSearch = [$keyword];
					$this->searchAdvSearch = [$keyword, $keyword];
					$this->searchFieldsTmplSearch = $this->searchFieldsAdvSearch = ['Filename', 'Content'];
					$this->locationTmplSearch = $this->locationAdvSearch = [we_search_base::OP_CONTAIN, we_search_base::OP_CONTAIN];
				}
				break;
			case we_search_view::SEARCH_ADV://Adv
			case we_search_view::SEARCH_CLASS:
				if($whichSearch === we_search_view::SEARCH_CLASS){
					$this->search_tables_advSearch[OBJECT_TABLE] = true;
				} else {
					$this->search_tables_advSearch[OBJECT_FILES_TABLE] = true;
				}
				if($keyword){
					$this->searchAdvSearch = [$keyword];
					$this->searchFieldsAdvSearch = ['Filename'];
					$this->locationAdvSearch = [we_search_base::OP_CONTAIN];
				}
				break;
			case we_search_view::SEARCH_VERSION:
				$this->search_tables_advSearch[VERSIONS_TABLE] = true;
				break;
			default:
				$this->search_tables_advSearch[FILE_TABLE] = true;
				$this->search_tables_advSearch[OBJECT_FILES_TABLE] = true;
		}
	}

}
