<?php
declare(strict_types=1);

/**
 * webEdition CMS
 *
 * $Rev: 6cc64d37551c $
 * $Author: Marc Kraemer <marc.kraemer@aldente-it.de> $
 * $Date: 2020-07-25 11:32 +0200 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
class we_search_resultDocuments extends we_search_resultBase{

	public function __construct(we_search_search $searchclass){
		parent::__construct($searchclass);
		$this->type = we_search_view::SEARCH_DOCS;
	}

	public static function getListColumns(): array{
		return [
			['sort' => '', 'title' => ''],
			['sort' => '', 'title' => ''],
			['sort' => 'Text', 'title' => g_l('searchtool', '[dateiname]')],
			['sort' => 'SiteTitle', 'title' => g_l('searchtool', '[seitentitel]')],
			['sort' => 'Creation_Date', 'title' => g_l('searchtool', '[created]')],
			['sort' => 'Mod_Date', 'title' => g_l('searchtool', '[modified]')],
			['sort' => 'docID', 'title' => g_l('searchtool', '[idDiv]')],
		];
	}

	protected function processElementList(): void{
		$fontColor = '';
		switch($this->ContentType){
			case we_base_ContentTypes::WEDOCUMENT:
			case we_base_ContentTypes::HTML:
				$fontColor = ($this->IsPublished ? ($this->IsModified ? 'changed' : '') : 'notpublished');
				$htmlCheckbox = (we_base_permission::hasPerm([we_base_perm::WORK_CAN_PUBLISH]) ?
					we_html_forms::checkbox($this->docID . '_' . addTblPrefix($this->docTable), false, 'publish_docs_' . $this->type, '', false, 'middlefont', '') :
					'');
				break;
			default:
				$htmlCheckbox = '';
		}
		$htmlIcon = '<span class="iconListview">' . $this->getHtmlIconThmubnail(64, 140, we_search_view::VIEW_LIST)['imageView'] . '</span>';
		$htmlFilename = we_html_element::a([
				'class' => $fontColor,
				'title' => $this->Path . ' (ID: ' . $this->docID . ')',
				'href' => 'javascript:' . $this->getJsOpenToEdit()
				], $this->Text);
		$htmlTitlePath = $this->type === we_search_view::SEARCH_TMPL ? str_replace('/' . $this->Text, '', $this->Path) : $this->SiteTitle;
		$htmlCreationDate = $this->Creation_Date ? (new DateTime($this->Creation_Date))->format(g_l('searchtool', '[date_format]')) : '-';
		$htmlModDate = $this->Creation_Date ? (new DateTime($this->Creation_Date))->format(g_l('searchtool', '[date_format]')) : '-';

		$class = 'middlefont bold textCol';
		$this->elementList = (new we_html_tr([
				new we_html_td($htmlCheckbox, ['class' => $class . ' checkbox']),
				new we_html_td($htmlIcon, ['class' => $class . ' icon']),
				new we_html_td($htmlFilename, ['class' => $class . ' filename']),
				new we_html_td($htmlTitlePath, ['class' => $class . ' titlePath']),
				new we_html_td($htmlCreationDate, ['class' => $class . ' creationdate']),
				new we_html_td($htmlModDate, ['class' => $class . ' moddate']),
				new we_html_td((string) $this->docID, ['class' => $class . ' id']),
				]))->getHtml();
	}

}
