<?php
declare(strict_types=1);

/**
 * webEdition CMS
 *
 * $Rev: 869defb635b1 $
 * $Author: Marc Kraemer <marc.kraemer@aldente-it.de> $
 * $Date: 2020-05-27 16:27 +0200 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
class we_search_resultVersion extends we_search_resultAdvanced{

	public function __construct(we_search_search $searchclass){
		parent::__construct($searchclass);
		$this->type = we_search_view::SEARCH_ADV;
	}

	protected function processElementList(): void{
		$resetDisabled = false;
		if(!we_base_permission::hasPerm([we_base_perm::VERSIONS_RESET]) ||
			($this->VersionActive && f('SELECT 1 FROM ' . escape_sql_query(addTblPrefix($this->docTable)) . ' WHERE ID=' . intval($this->docID)))){
			$resetDisabled = true;
		}

		//if class doesn't exists it's not possible to reset object-version!
		if($this->ContentType === we_base_ContentTypes::OBJECT_FILE &&
			!f('SELECT 1 FROM ' . OBJECT_TABLE . ' WHERE ID=' . ((int) $this->TableID))){
			$resetDisabled = true;
			$classNotExistsText = g_l('versions', '[objClassNotExists]');
		} else {
			$classNotExistsText = '';
		}

		$btnPreview = we_html_button::create_button(we_html_button::PREVIEW, "javascript:weSearch.previewVersion('" . addTblPrefix($this->docTable) . "'," . $this->docID . "," . $this->Text . ");");
		$htmlText = we_html_element::div(['style' => 'display:inline-block;padding:8px 0 0 12px;'], g_l('versions', '[version]') . ' ' . $this->Text) . we_html_element::div([
				'style' => 'display:inline-block;float:right;'], $btnPreview);

		$hint = $classNotExistsText ? we_html_element::div(['class' => 'notpublished', 'style' => 'padding: 4px 0 4px 8px;'], $classNotExistsText) : '';
		$checkoxReset = we_html_forms::checkbox((string) $this->VersionID, false, 'resetVersion[' . $this->VersionID . '_' . VERSIONS_TABLE . ']', g_l('versions', '[reset]'), false, 'defaultfont', '', $resetDisabled);
		$checkboxPublish = (in_array($this->ContentType, [we_base_ContentTypes::WEDOCUMENT, we_base_ContentTypes::HTML, we_base_ContentTypes::OBJECT_FILE]) ?
			we_html_forms::checkbox((string) $this->VersionID, false, 'publishVersion_' . $this->VersionID, g_l('versions', '[publishIfReset]'), false, 'middlefont', '', $resetDisabled) :
			''
			);
		$htmlTitlePath = $hint . we_html_element::div(['style' => 'margin-bottom:5px;margin-left:5px;float:left;', 'class' => 'bold'], $checkoxReset) .
			we_html_element::div(['style' => 'margin-bottom:5px;margin-left:5px;float:left;'], $checkboxPublish);

		$htmlCreationDate = $this->Creation_Date ? (new DateTime($this->Creation_Date))->format(g_l('searchtool', '[date_format]')) : '-';

		$class = 'middlefont bold textCol';
		$this->elementList = (new we_html_tr([
				new we_html_td('', ['class' => $class . ' checkbox']),
				new we_html_td('', ['class' => $class . ' icon']),
				new we_html_td($htmlText, ['class' => $class]),
				new we_html_td($htmlTitlePath, ['class' => 'middlefont textCol titlePath']),
				new we_html_td($htmlCreationDate, ['class' => $class . ' creationdate']),
				new we_html_td($this->VersionActive ? g_l('versions', '[active_version]')/* 'aktive Version' */ : '', ['class' => 'middlefont textCol moddate']),
				new we_html_td('', ['class' => $class . ' id']),
				]))->getHtml();
	}

}
