<?php
declare(strict_types=1);

/**
 * webEdition CMS
 *
 * $Rev: 1eb1f702a3b6 $
 * $Author: Marc Kraemer <marc.kraemer@aldente-it.de> $
 * $Date: 2020-08-25 15:34 +0200 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
class we_shop_Basket implements Countable{
	//FIXME: this is set back to public due to some shop restrictions, see #6530, #6954
	/**
	 * 	this array contains all shopping items
	 * 	a shopping item is an associated array containining
	 * 	'id'       => integer
	 * 	'type'     => w | o
	 * 	'variant'  => string
	 * 	'quantity' => integer //fr�her - jetzt umstellung auf float, feature #4875
	 * 	'serial'   => string
	 *  'customFields' => array
	 *
	 * @var array<string,array{id:int,type:string,variant:string,quantity:float,serial:array,customFields:array}>
	 */
	public $ShoppingItems = [];

	/**
	 * user can define custom fields saved with the order.
	 *
	 * @var array
	 */
	public $CartFields = [];

	/** @var int */
	public $orderID = 0;

	/** @var int */
	private $creationTime = 0;

	function __construct(array $array){
		$this->setCartProperties($array);
	}

	function initCartFields(): void{
		if(empty($_REQUEST[we_shop_shop::CART_CUSTOM_FIELD]) || !is_array($_REQUEST[we_shop_shop::CART_CUSTOM_FIELD])){
			return;
		}
		foreach($_REQUEST[we_shop_shop::CART_CUSTOM_FIELD] as $key => $value){
			$this->CartFields[$key] = $value;
		}
	}

	public function getCreationTime(): int{
		return $this->creationTime;
	}

	/**
	 * returns array of shoppingItems
	 *
	 * @return array<string,array{id:int,type:string,variant:string,quantity:float,serial:array,customFields:array}>
	 */
	function getShoppingItems(): array{
		return $this->ShoppingItems;
	}

	/**
	 * returns array of all shopping cartfields
	 *
	 * @return array
	 */
	function getCartFields(): array{
		return $this->CartFields;
	}

	function hasCartField(string $name): bool{
		return isset($this->CartFields[$name]);
	}

	function getCartField(string $name): string{
		return $this->CartFields[$name] ?? '';
	}

	/**
	 * returns the items in the shopping cart and all custom cart fields
	 * former getProperties
	 *
	 * @return array<string,array>
	 */
	function getCartProperties(): array{
		return [
			'shoppingItems' => $this->getShoppingItems(),
			'cartFields' => $this->getCartFields()
		];
	}

	/**
	 * initialies the shoppingCart
	 * former name setProperties
	 *
	 * @param array<string,mixed> $array
	 */
	private function setCartProperties(array $array): void{
		if(isset($array['shoppingItems']) && isset($array['cartFields'])){
			$this->ShoppingItems = $array['shoppingItems'];
			$this->CartFields = $array['cartFields'];
			$this->creationTime = $array['creationTime'] ?? time();
			return;
		}
		$this->ShoppingItems = [];
		$this->CartFields = [];
		$this->creationTime = time();
	}

	/**
	 * add an item to the array
	 *
	 * @param integer $id
	 * @param float $quantity
	 * @param string $type
	 * @param string $variant
	 */
	function addItem(int $id, float $quantity = 1, string $type = we_shop_shop::DOCUMENT, string $variant = '', array $customFields = []): void{
		// check if this item is already in the shoppingCart
		if(($key = $this->getShoppingItemIndex($id, $type, $variant))){ // item already exists
			$this->ShoppingItems[$key]['quantity'] += $quantity;
			if($this->ShoppingItems[$key]['quantity'] <= 0){
				$this->delItem($id, $type, $variant);
			}
			return;
		}
		if($quantity <= 0){// only add new item with positive number
			return;
		}
// add the item
		$key = $type . $id . $variant;

		$this->ShoppingItems[$key] = [
			'id' => $id,
			'type' => $type,
			'variant' => $variant,
			'quantity' => $quantity,
			'serial' => self::getserial($id, $type, $variant, $customFields),
			'customFields' => $customFields
		];
	}

	/**
	 * returns size of shoppingCart
	 *
	 * @return integer
	 */
	function getBasketCount(): int{
		return count($this->ShoppingItems);
	}

	public function count(): int{
		return count($this->ShoppingItems);
	}

	/**
	 * returns shoppingItems
	 *
	 * @return array
	 */
	function getAllData(): array{
		return $this->getCartProperties();
	}

	/**
	 * returns shoppingItem - serial
	 *
	 * @param integer $id
	 * @param string $type
	 * @param string $variant
	 * @return array
	 */
	public static function getserial(int $id, string $type, string $variant = '', array $customFields = []): array{
		$DB_WE = DB_WE::$inst;
		$Record = [];

		switch($type){
			case we_shop_shop::DOCUMENT:
				// unfortunately this is not made with initDocById,
				// but its much faster -> so we use it
				$Record = $DB_WE->getAllFirstq('SELECT c.Name,IF(c.BDID>0,c.BDID,c.Dat) AS Dat FROM ' . CONTENT_TABLE . ' c WHERE c.DID=' . $id . ' AND c.DocumentTable="' . stripTblPrefix(FILE_TABLE) . '"', false);
				'@phan-var array<string,int|string> $Record';
				if($variant){
					we_base_variants::useVariantForShop($Record, $variant);
				}

				if(($hash = array_change_key_case(getHash('SELECT * FROM ' . FILE_TABLE . ' WHERE ID=' . $id, $DB_WE, MYSQLI_ASSOC)))){
					$Record = array_merge($Record, we_listview_base::preFixArrayKeys($hash));
				}

				$Record[we_listview_base::PATH_FIELD] .= ($variant ? '?' . we_base_variants::REQUEST . '=' . $variant : '');
				$Record[we_listview_base::VARIANT_FIELD] = $variant;
				$Record[we_listview_base::ID_FIELD] = $id;

				// at last add custom fields to record and to path
				if(!empty($customFields)){
					$Record[we_listview_base::PATH_FIELD] .= ($variant ? '&amp;' : '?');

					foreach($customFields as $name => $value){
						$Record[$name] = $value;
						$Record[we_listview_base::PATH_FIELD] .= we_shop_shop::ARTICLE_CUSTOM_FIELD . '[' . $name . ']=' . $value . '&amp;';
					}
				}
				break;
			case we_shop_shop::OBJECT:
				$classID = fI('SELECT TableID FROM ' . OBJECT_FILES_TABLE . ' WHERE IsFolder=0 AND ID=' . $id);
				if(!$classID){
					t_e('fatal shop error', $id, $_REQUEST);
					return [];
				}

				$olv = new we_listview_object('', 1, 0, '', false, $classID, [], false, 'ID=' . $id, 0, 0, false, '', '', '', '', [], [], '', 0, [], [], [], TAGLINKS_DIRECTORYINDEX_HIDE, TAGLINKS_OBJECTSEOURLS);
				$olv->next_record();

				$Record = $olv->getRecord();
				unset($Record[we_listview_base::LASTPATH_FIELD]);
				if($variant){
					// init model to detect variants
					// :TODO: change this to match above version
					$obj = new we_contents_objectFile($id);

					$Record = we_base_variants::useVariantForShopObject($Record, $variant, $obj);

					// add variant to path ...
					$Record[we_listview_base::PATH_FIELD] .= '?' . we_base_variants::REQUEST . '=' . $variant;
				}
				$Record[we_listview_base::VARIANT_FIELD] = $variant;
				$Record['we_obj'] = $id;

				// at last add custom fields to record and to path
				if(!empty($customFields)){
					foreach($customFields as $name => $value){
						$Record[$name] = $value;
						$Record[we_listview_base::PATH_FIELD] .= '&amp;' . we_shop_shop::ARTICLE_CUSTOM_FIELD . '[' . $name . ']=' . $value;
					}
				}

				break;
		}

		// at last add custom fields and vat to shopping card
		$Record[we_shop_shop::ARTICLE_CUSTOM_FIELD] = $customFields;

		return $Record;
	}

	/**
	 * returns amount of shopping items by key
	 *
	 * @param string $key
	 * @return float
	 */
	function getItemQuantity(string $key): float{
		return (float) $this->ShoppingItems[$key]['quantity'];
	}

	/**
	 * remove item from shop
	 *
	 * @param integer $id
	 * @param string $type
	 * @param string $variant
	 */
	function delItem(int $id, string $type, string $variant = ''): void{
		if(($key = $this->getShoppingItemIndex($id, $type, $variant))){
			unset($this->ShoppingItems[$key]);
		}
	}

	/**
	 * resets the shoppingCart
	 *
	 */
	function emptyBasket(): void{
		$this->ShoppingItems = [];
		$this->CartFields = [];
	}

	/**
	 * changes abilities of item in the shoppingCart
	 *
	 * @param integer $id
	 * @param float $quantity
	 * @param string $type
	 * @param string $variant
	 */
	function setItem(int $id, float $quantity = 1, string $type = we_shop_shop::DOCUMENT, string $variant = '', array $customFields = []): void{
		if(($key = $this->getShoppingItemIndex($id, $type, $variant))){ // item already in cart
			if($quantity > 0){
				$this->ShoppingItems[$key]['quantity'] = $quantity;
				return;
			}
			$this->delItem($id, $type, $variant);
			return;
		}
// new item
		$this->addItem($id, $quantity, $type, $variant, $customFields);
	}

	/**
	 * set cart item by the assoc array
	 *
	 * @param string $cart_id
	 * @param float $cart_amount
	 */
	function setCartItem(string $cart_id, float $cart_amount): void{
		if(!isset($this->ShoppingItems[$cart_id])){
			return;
		}
		$item = $this->ShoppingItems[$cart_id];
		$this->setItem($item['id'], $cart_amount, $item['type'], $item['variant'], $item['customFields']);
	}

	/**
	 * returns key for shoppingItem or false
	 *
	 * @param integer $id
	 * @param string $type
	 * @param string $variant
	 * @return string
	 */
	function getShoppingItemIndex(int $id, string $type = we_shop_shop::DOCUMENT, string $variant = ''): string{
		$key = $type . $id . $variant;
		if(isset($this->ShoppingItems[$key])){
			return $key;
		}
		return '';
	}

	function getOrderID(): int{
		return $this->orderID;
	}

	function setOrderID(int $id): void{
		$this->orderID = $id;
	}

}
