<?php
declare(strict_types=1);

/**
 * webEdition CMS
 *
 * $Rev: 041bf6b9946d $
 * $Author: Marc Kraemer <marc.kraemer@aldente-it.de> $
 * $Date: 2020-08-14 19:07 +0200 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
class we_shop_category extends we_category_category{
	/** @var bool */
	public $DestPrinciple = false; //make getter and set private

	/** @var int */
	public $CatFallbackState = 0;

	/** @var bool */
	public $IsInactive = false;
	//FIXME: move some of these static vars to session

	/** @var bool */
	private static $isCategoryMode;

	/** @var int */
	private static $shopCatDir = -1;

	/** @var int[] */
	private static $shopCatIDs = [];
	private static $shopVatsByCategoryCountry = [];

	/** @var array<int,int> */
	private static $shopCatMapping = [];

	/** @var ?bool */
	private static $mustCheckIsInactive = null;

	/** @var int[] */
	private static $destPrinciples = [];

	/** @var int[] */
	private static $activeShopCats = [];

	const IS_CAT_FALLBACK_TO_STANDARD = 1;
	const IS_CAT_FALLBACK_TO_ACTIVE = 2;
	const IS_CAT_FALLBACK_TO_WEDOCCAT = 3;
	const IS_CAT_FALLBACK_TO_WEDOCCAT_AND_ACTIVE = 5;

	function __construct(int $id = 0){
		parent::__construct($id);
		if($id){
			$this->DestPrinciple = $this->getDestPrinciple();
			$this->IsInactive = $this->getIsInactive();
		}
	}

	public function we_save(bool $resave = false, bool $skipHook = false): bool{
		parent::we_save();
		return $this->saveDestPrincipleToDB();
	}

	/**
	 * get field DestinationPrinciple of this shop category
	 *
	 * @return int
	 */
	private function getDestPrinciple(): bool{
		if(!$this->ID){
			return false;
		}
		$ids = self::getDestPrincipleFromDB();

		return in_array(intval($this->ID), $ids);
	}

	/**
	 * get field IsInactive of this shop category
	 *
	 * @return int
	 */
	//FIXME: Combine getters, setters and save functions for IsInactive and DestPrinciple
	private function getIsInactive(): bool{
		if(!$this->ID){
			return false;
		}
		$ids = self::getIsInactiveFromDB();

		return in_array(intval($this->ID), $ids);
	}

	/**
	 * save field DestinationPrinciple of this shop category
	 *
	 * @return boolean
	 */
	private function saveDestPrincipleToDB(): bool{
		$ids = self::getDestPrincipleFromDB();
		if($this->DestPrinciple){
			if(in_array($this->ID, $ids)){
				return true;
			}
			$ids[] = $this->ID;
			sort($ids);
		} else {
			if(($k = array_search($this->ID, $ids, false)) === false){
				return true;
			}
			unset($ids[$k]);
		}

		return self::saveSettingDestPrinciple($ids);
	}

	/**
	 * get csv or array containing ids of all shop categories with DestinationPrinciple = true
	 *
	 * @return int[]
	 */
	public static function getDestPrincipleFromDB(): array{
		if(!empty(self::$destPrinciples)){
			return self::$destPrinciples;
		}

		self::$destPrinciples = we_base_request::filterIntList(fS('SELECT pref_value FROM ' . SETTINGS_TABLE . ' WHERE tool="shop" AND pref_name="shop_cats_destPrinciple"', '-1'));

		return self::$destPrinciples;
	}

	/**
	 * get csv or array containing ids of all shop categories with IsInactive = true
	 *
	 * @return int[]
	 */
	public static function getIsInactiveFromDB(): array{
		if(!empty(self::$activeShopCats)){
			return self::$activeShopCats;
		}

		self::$activeShopCats = we_base_request::filterIntList(fS('SELECT pref_value FROM ' . SETTINGS_TABLE . ' WHERE tool="shop" AND pref_name="shop_cats_isInactive"'));

		return self::$activeShopCats;
	}

	/**
	 * saves shop category field DestinationPrinciple of all shop categoeries
	 *
	 * @param int[] $ids
	 * @return boolean
	 */
	public static function saveSettingDestPrinciple(array $ids): bool{
		return DB_WE::$inst->query('REPLACE INTO ' . SETTINGS_TABLE . ' SET tool="shop", pref_name="shop_cats_destPrinciple", pref_value="' . implode(',', array_filter($ids)) . '"');
	}

	/**
	 * saves shop category field IsInactive of all shop categoeries
	 *
	 * @param int[] $ids
	 * @return boolean
	 */
	public static function saveSettingIsInactive(array $ids): bool{
		return DB_WE::$inst->query('REPLACE INTO ' . SETTINGS_TABLE . ' SET tool="shop", pref_name="shop_cats_isInactive", pref_value="' . implode(',', array_filter($ids)) . '"');
	}

	/**
	 * return shopCategoryDir defined in tblSettings as path
	 *
	 * @return int
	 */
	public static function getShopCatDir(): string{
		$id = self::getShopCatDirID();
		return $id ? id2path($id, CATEGORY_TABLE) : '';
	}

	/**
	 * return shopCategoryDir defined in tblSettings as id
	 *
	 * @return int
	 */
	public static function getShopCatDirID(): int{
		if(self::$shopCatDir < 0){
			self::$shopCatDir = fI('SELECT pref_value FROM ' . SETTINGS_TABLE . ' WHERE tool="shop" AND pref_name="shop_cats_dir"');
		}
		self::$shopCatDir = 1;
		return self::$shopCatDir ?: 0;
	}

	/**
	 * return true when category mode is activated in shop prefs
	 *
	 * @return bool
	 */
	public static function isCategoryMode(): bool{
		if(self::$isCategoryMode !== null){
			return (bool) self::$isCategoryMode;
		}

		return (self::$isCategoryMode = fB('SELECT pref_value FROM ' . SETTINGS_TABLE . ' WHERE tool="shop" AND pref_name="category_mode"'));
	}

	/**
	 * saves shopCategoryDir to tblSettings
	 *
	 * @param int $id
	 * @return boolean
	 */
	public static function saveShopCatsDir(int $id): bool{
		return DB_WE::$inst->query('REPLACE INTO ' . SETTINGS_TABLE . ' SET tool="shop", pref_name="shop_cats_dir", pref_value=' . $id);
	}

	/**
	 * returns shop category by id, false if there is no shop category of this id
	 *
	 * @param int $id
	 * @return we_shop_category|false
	 */
	public static function getShopCatById(int $id = 0, array $wedocCategory = [], bool $useFallback = false): self{
		if(!$id && !$useFallback){
			throw new we_exception_shop('no valid id');
		}

		$tmpValidID = self::checkGetValidID($id, $wedocCategory, $useFallback);
		if(!($validID = $tmpValidID['id'])){
			throw new we_exception_shop('no valid id');
		}

		$cat = new self();
		$cat->load($validID);
		$cat->CatFallbackState = $tmpValidID['state'];

		$path = self::getShopCatDir();
		if(!$cat->isnew && strpos($cat->Path, $path) === 0){
			$cat->DestPrinciple = $cat->getDestPrinciple();

			return $cat;
		}

		throw new we_exception_shop('no valid cat found');
	}

	/**
	 * returns all shop category ids, optionally with or without shop categories dir
	 *
	 * @param bool $incCatsDir
	 *
	 * @return int[]
	 */
	public static function getAllShopCatIDs(bool $incCatsDir = true): array{
		$shopCatDir = self::getShopCatDirID();
		if(!self::$shopCatIDs){
			self::$shopCatIDs = array_keys(self::getShopCatsFromDir($shopCatDir, false, false));
		}

		return array_unique($incCatsDir ? array_merge([$shopCatDir], self::$shopCatIDs) : self::$shopCatIDs);
	}

	/**
	 * writes mapping for inactive shop categories to static var $shopCatMapping
	 *
	 * @param int[] $inactives
	 * @return void
	 */
	private static function writeShopCatMapping(array $inactives): void{
		self::$shopCatMapping[self::getShopCatDirID()] = self::getShopCatDirID();
		foreach(($cats = self::getShopCatsFromDir(0, false, false, '', 'Path')) as $cat){
			self::$shopCatMapping[$cat->ID] = in_array($cat->ID, $inactives) ? self::$shopCatMapping[$cats[$cat->ID]->ParentID] : $cat->ID;
		}

		//debug
		/*
		  $arr = [];
		  foreach(self::$shopCatMapping as $id => $val){
		  $arr[$id] = !in_array($val, $inactives) ? "ok" : "bad";
		  }
		  unset($arr[self::getShopCatDir()]);
		  t_e("check is shopCatMapping ok", $arr, self::$shopCatMapping);
		 *
		 */
	}

	/**
	 * returns true if $id is some shop catregory's id
	 * Optionally it returns shop categories inside of some driectory defined by $dir.
	 *
	 * @param int $id
	 * @return bool
	 */
	public static function isShopCategoryID(int $id): bool{
		$ids = count(self::$shopCatIDs) ? self::$shopCatIDs : self::getAllShopCatIDs(false);
		$ids[] = self::getShopCatDirID();

		return in_array($id, $ids);
	}

	/**
	 * returns a valid shop category id computed by some param id considering all aloud falbacks
	 * Optionally it returns fallback state
	 *
	 * @param int $id
	 * @param int[] $wedocCategory
	 * @param bool $useFallback
	 * @return array{id:int,state:int}
	 */
	public static function checkGetValidID(int $id = 0, array $wedocCategory = [], bool $useFallback = true): array{
		//$origID = $id;
		$state = 0;
		if(!$id || !self::isShopCategoryID($id)){
			if(!$useFallback){
				return ['id' => 0, 'state' => 0];
			}

			//look for shop category linked to we_doc
			$id = 0;
			foreach($wedocCategory as $c){
				if(self::isShopCategoryID($c)){
					$id = $c;
					$state = self::IS_CAT_FALLBACK_TO_WEDOCCAT;
					break;
				}
			}

			//if still no valid id: get shopCatDir
			if(!$id){
				$id = self::getShopCatDirID();
				$state = self::IS_CAT_FALLBACK_TO_STANDARD;
			}
		}
		//$interimsID = $id;

		if($id !== self::getShopCatDirID()){
			$inactives = [];
			if(self::$mustCheckIsInactive === null){
				$inactives = self::getIsInactiveFromDB();
				$numCats = count(self::getAllShopCatIDs(false));
				self::$mustCheckIsInactive = (count($inactives) && $numCats !== count($inactives));
			}

			if(self::$mustCheckIsInactive === true){
				$inactives = $inactives ?: self::getIsInactiveFromDB();
				$tmpId = $id;
				if(!self::$shopCatMapping){
					self::writeShopCatMapping($inactives);
				}
				$id = isset(self::$shopCatMapping) ? self::$shopCatMapping[$id] : $id;
				$state += $tmpId === $id ? 0 : self::IS_CAT_FALLBACK_TO_ACTIVE;
			}
		}
		//t_e("getCheck", $origID, $interimsID, $id);

		return ['id' => $id, 'state' => $state];
	}

	public static function getInternalFieldname(string $field = 'ID'): string{
		$fieldMap = [
			'id' => 'ID',
			'category' => 'Category',
			'path' => 'Path',
			'title' => 'Title',
			'description' => 'Description',
			'is_destinationprinciple' => 'DestPrinciple',
			'is_from doc_object' => 'is_from doc_object',
			'is_fallback_to_standard' => 'is_fallback_to_standard',
			'is_fallback_to_active' => 'is_fallback_to_active'
		];

		return in_array($field, $fieldMap) ? $field : ($fieldMap[$field] ?? 'ID');
	}

	/**
	 * returns field $catfield of all shop categories.
	 *
	 * @param int $dir
	 * @param bool $activeOnly
	 * @param bool $includeDir
	 * @param string $rootdir
	 * @param string $order
	 * @return array<int,we_shop_category>
	 */
	static function getShopCatsFromDir(int $dir = 0, bool $activeOnly = true, bool $includeDir = true, string $rootdir = '', string $order = ''): array{
		if(!we_modules_info::isActive(we_modules_info::CATEGORY) || !($path = (id2path(($dir ?: self::getShopCatDirID()), CATEGORY_TABLE)))){
			return [];
		}

		$destPrincipleIds = self::getDestPrincipleFromDB();
		$isInactiveIds = self::getIsInactiveFromDB();
		$categories = self::we_getCategories([], $rootdir, $path, $includeDir, ($order ?: 'ID'));

		if($activeOnly){
			$isInactiveIds = $isInactiveIds ?: self::getIsInactiveFromDB();
			$numCats = count($categories) - ($includeDir ? 1 : 0);

			if(!empty($isInactiveIds) && count($isInactiveIds) !== $numCats){
				foreach($isInactiveIds as $k){
					unset($categories[$k]);
				}
			}
		}
		if(!$categories){
			return [];
		}

		foreach($categories as $id => $cat){
			assert($cat instanceof we_shop_category);
			$cat->DestPrinciple = in_array((int) $id, $destPrincipleIds);
			$cat->IsInactive = in_array((int) $id, $isInactiveIds);
		}

		return $categories;

		//return !$assoc ? array_values($ret) : $ret;
	}

	/**
	 * returns field $catfield of shop categories defined by their ids.
	 *
	 * @param int $id
	 * @param int[] $wedocCategory
	 * @param string $field
	 * @param bool $showpath
	 * @param string $rootdir
	 * @param bool $useFallback
	 * @throws we_exception_shop
	 * @return mixed of string
	 */
	static function getShopCatFieldByID(int $id, array $wedocCategory = [], string $field = '', bool $showpath = false, string $rootdir = '', bool $useFallback = true)/* :mixed */{
		$cat = self::getShopCatById($id, $wedocCategory, $useFallback);

		switch($field = self::getInternalFieldname($field)){
			case 'Path':
				return !$showpath ? $cat->Path : (substr($cat->Path, strlen($rootdir)));
			case 'is_from doc_object':
				return ($cat->CatFallbackState === self::IS_CAT_FALLBACK_TO_WEDOCCAT || $cat->CatFallbackState === self::IS_CAT_FALLBACK_TO_WEDOCCAT_AND_ACTIVE) ? 1 : 0;
			case 'is_fallback_to_standard':
				return $cat->CatFallbackState === self::IS_CAT_FALLBACK_TO_STANDARD ? 1 : 0;
			case 'is_fallback_to_active' :
				return ($cat->CatFallbackState === self::IS_CAT_FALLBACK_TO_ACTIVE || $cat->CatFallbackState === self::IS_CAT_FALLBACK_TO_WEDOCCAT_AND_ACTIVE) ? 1 : 0;
			default:
				return $cat->$field;
		}
	}

	/**
	 * returns we_shop_vat for determined by destination country
	 *
	 * @param string $country
	 * @return we_shop_vat
	 */
	private function getShopVatByCountry(string $country): we_shop_vat{
		if(!$country){
			throw new we_exception_base('No country given');
		}

		//FIXME: make saving stuff in static vars more concise, maybe move to session
		if(isset(self::$shopVatsByCategoryCountry[$this->ID][$country])){
			return self::$shopVatsByCategoryCountry[$this->ID][$country];
		}

		if(!isset(self::$shopVatsByCategoryCountry[$this->ID])){
			self::$shopVatsByCategoryCountry[$this->ID] = [];
		}

		$vatID = fI('SELECT id FROM ' . WE_SHOP_VAT_TABLE . ' WHERE territory="' . $this->db->escape($country) . '" AND FIND_IN_SET(' . intval($this->ID) . ', categories)', -1);
		try{
			if($vatID){
				return (self::$shopVatsByCategoryCountry[$this->ID][$country] = we_shop_vat::getVatById($vatID));
			}
		} catch (we_exception_shop $e){

		}
		try{
			$vat = we_shop_vats::getStandardShopVat();
			self::$shopVatsByCategoryCountry[$this->ID][$country] = $vat;
			return $vat;
		} catch (we_exception_shop $e){

		}

		$shopPrefs = explode('|', fS('SELECT pref_value FROM ' . SETTINGS_TABLE . ' WHERE tool="shop" AND pref_name="shop_pref"', '-1'));
		if(($pref = $shopPrefs[1])){
			self::$shopVatsByCategoryCountry[$this->ID][$country] = false;
			return new we_shop_vat(0, 'shop_pref', intval($pref), true, '', '', '');
		}
		throw new we_exception_shop('no vat found');
	}

	/**
	 * returns we_shop_vat for determined by shop category id destination country
	 *
	 * @param int $id
	 * @param int[] $wedocCategory
	 * @param string $country
	 * @return we_shop_vat
	 * @throws we_exception_shop
	 */
	public static function getShopVatByIdAndCountry(int $id = 0, array $wedocCategory = [], string $country = ''): we_shop_vat{
		$validID = self::checkGetValidID($id, $wedocCategory)['id'];
		$country = $country && in_array(intval($validID), self::getDestPrincipleFromDB()) ? $country : self::getDefaultCountry(); // only get vat of current (customer) country, when shop category is DestPrinciple!

		if(isset(self::$shopVatsByCategoryCountry[$validID][$country])){
			$ret = self::$shopVatsByCategoryCountry[$validID][$country];
			assert($ret instanceof we_shop_vat);
			return $ret;
		}

		$cat = self::getShopCatById($validID, [], false); //we have validID so we do not have to validate again!
		return $cat->getShopVatByCountry($country);
	}

	/**
	 * returns content of a customer's country field. Country fallback is used when param useFallback is true.
	 * Optionally teh function returns an array with country, customer and flag isFallback.
	 *
	 * @param bool $useFallback
	 * @param array $customer
	 * @param int $customerId
	 * @return array
	 */
	public static function getFullCountryFromCustomer(bool $useFallback = false, array $customer = [], int $customerId = 0): array{
		$customer = ($customer ?:
			($_SESSION['webuser'] ??
			($customerId ?
			(getHash('SELECT * FROM ' . CUSTOMER_TABLE . ' WHERE ID=' . $customerId) ?:
			[]) :
			[]
			)
			)
			);

		if($customer){
			$customer = array_merge($customer, we_customer_customer::getEncryptedFields());
			$stateField = we_shop_vatRule::getStateField();
			if(isset($customer[$stateField]) && ($c = $customer[$stateField])){
				return ['country' => $c, 'isFallback' => false, "customer" => $customer];
			}
		}

		if($useFallback && ($c = self::getDefaultCountry())){
			return ['country' => $c, "isFallback" => true, "customer" => false];
		}

		return ['country' => '', "isFallback" => false, "customer" => $customer ? true : false];
	}

	public static function getCountryFromCustomer(bool $useFallback = false, array $customer = [], int $customerId = 0): string{
		$ret = self::getFullCountryFromCustomer($useFallback, $customer, $customerId);
		return $ret['country'];
	}

	/**
	 * returns country defined in shop prefs as shop location
	 *
	 * @return string
	 */
	public static function getDefaultCountry(): string{
		return fS('SELECT pref_value FROM ' . SETTINGS_TABLE . ' WHERE tool="shop" AND pref_name="shop_location"', '-1');
	}

}
