<?php
declare(strict_types=1);

/**
 * webEdition CMS
 *
 * $Rev: 6cc64d37551c $
 * $Author: Marc Kraemer <marc.kraemer@aldente-it.de> $
 * $Date: 2020-07-25 11:32 +0200 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
abstract class we_sidebar_frames{

	public static function getHTML(string $what): string{
		switch($what){
			case 'content':
				return self::getHTMLContent();
			default:
				return self::getHTMLFrameset();
		}
	}

	private static function getHTMLFrameset(): string{
		return '
<div id="weSidebarBody">
	<div id="weSidebarHeader">
		<div id="Headline">' . g_l('sidebar', '[headline]') . '</div>
		<div id="CloseButton">
			<span class="close" onclick="WE().layout.sidebar.close();">
				<i class="fa fa-close fa-lg "></i>
			</span>

		</div>
	</div>
	<div id="weSidebarContentDiv">
		<iframe id="weSidebarContent" src="' . WEBEDITION_DIR . 'we_cmd.php?we_cmd[0]=loadSidebarDocument" name="weSidebarContent"></iframe>
	</div>
</div>';
	}

	/**
	 *
	 * @param array{link:string,text:string,headline:string}[] $textArray
	 * @return string
	 */
	private static function showSidebarText(array $textArray): string{
		$ret = '';
		foreach($textArray as $text){
			$link = "%s";
			if(!empty($text['link'])){

				if(stripos($text['link'], 'javascript:') === 0){
					$text['link'] = str_replace("\"", "'", $text['link']); #6625
					$text['link'] = str_replace("`", "'", $text['link']); #6625
					$link = '<a href="' . $text['link'] . '">%s</a>';
				} else {
					$link = '<a href="' . $text['link'] . '" target="_blank">%s</a>';
				}
			}

			$headline = (empty($text['headline']) ? '' : sprintf($link, $text['headline']));
			$ret .= '
<tr>
	<td style="vertical-align:top;padding-top:5px;" colspan="2"><strong>' . $headline . '</strong>' . we_html_element::br(2) . $text['text'] . '</td>
</tr>';
		}
		return $ret;
	}

	private static function getDefaultContent(): string{
		return we_html_tools::getHtmlTop('sideBar', '', we_html_element::body(['class' => 'weSidebarBody'], '<table class=" defaultfont">' .
					self::showSidebarText(we_base_country::g_l_a('sidebar', '[default]')) . //@phan-suppress-current-line PhanTypeMismatchArgument
					(we_base_permission::hasPerm([we_base_perm::ADMINISTRATOR]) ?
					self::showSidebarText(we_base_country::g_l_a('sidebar', '[admin]')) : //@phan-suppress-current-line PhanTypeMismatchArgument
					'') .
					'</table>'
		));
	}

	public static function getHTMLContent(): string{
		$id = we_base_request::int('we_cmd', 0, 1);
		$file = we_base_request::_(we_base_request::URL, 'we_cmd', '', 2);
		$params = we_base_request::string('we_cmd', '', 3);
		define('WE_SIDEBAR', true);

		if($id){
			$file = id2path($id, FILE_TABLE, false, true);
		} else {
			if(stripos($file, "http://") === 0 || stripos($file, "https://") === 0){
				//not implemented
				//header("Location: " . $file);
				exit();
			}
		}
		if(!file_exists(WE_DOCUMENT_ROOT . $file) || !is_file(WE_DOCUMENT_ROOT . $file)){
			$file = id2path(intval(SIDEBAR_DEFAULT_DOCUMENT), FILE_TABLE, false, true);
			if(!$file || substr($file, -1) === '/' || $file === 'default'){
				$file = '';
			}
		}

		if($file){
			//manipulate GET/REQUEST for document
			$_GET = [];
			parse_str($params, $_GET);
			$_REQUEST = $_GET;
			ob_start();
			include(WE_DOCUMENT_ROOT . $file);
			$cnt = 0;
			$SrcCode = str_replace('<head>', '<head>' . we_html_element::jsScript(JS_DIR . 'global.js'), (string) (ob_get_clean() ?? ''), $cnt);
		} else {
			$cnt = 0;
			$SrcCode = self::getDefaultContent();
		}

		return we_gui_SEEM::parseDocument(($cnt ? $SrcCode : we_html_element::jsScript(JS_DIR . 'global.js') . $SrcCode));
	}

}
