<?php
declare(strict_types=1);

/* webEdition CMS
 *
 * $Rev: c564388e074d $
 * $Author: Marc Kraemer <marc.kraemer@aldente-it.de> $
 * $Date: 2020-07-17 13:01 +0200 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */

/**
 * shows a comment in the defined language
 */
abstract class we_tag_comment extends we_weTag_tag{

	/**
	 * @param array<string,mixed> $attribs parsed tag-attributes
	 * @param string $content content between openingtag and closing tag @unused-param
	 * @return string parsed tag for php
	 */
	public static function parse(array $attribs, string $content): string{
		$type = self::getParserAttribute('type', $attribs, 'none', we_base_request::STRING);
		//remove we: parts since this will confuse the tag parser and it will parse these tags
		$content = str_replace('we:', 'we', $content);
		switch($type){
			case 'xml':
			case 'html':
				return '<!-- ' . $content . ' -->';
			case 'js':
				return '/* ' . $content . ' */';
			case 'php':
				return '<?php /*' . str_replace('*/', '', $content) . '*/ ?>';
			case 'none':
			default:
				return '';
		}
	}

	/**
	 * @param array<string,mixed> $attribs parsed tag-attributes @unused-param
	 * @param string $content content between openingtag and closing tag @unused-param
	 * @param bool $internal if used by we internally @unused-param
	 * @throws we_exception_parser if any attribute is missing
	 */
	public static function tag(array $attribs = [], string $content = '', bool $internal = false): string{
		/**
		 * Dummy tag - only parser part needed
		 */
		return '';
	}

}
