<?php
declare(strict_types=1);

/* webEdition CMS
 *
 * $Rev: c564388e074d $
 * $Author: Marc Kraemer <marc.kraemer@aldente-it.de> $
 * $Date: 2020-07-17 13:01 +0200 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */

abstract class we_tag_navigationEntry extends we_weTag_tag{

	public static function parse(array $attribs, string $content): string{
		self::attributeMissing($attribs, ['type' => false], __CLASS__);

		return '<?php ' . we_weTag_parser::printTag('navigationEntry', $attribs, strtr($content, ['global $lv;' => '', '\\\\$' => '']), true) . ';?>';
	}

	/**
	 * @param array<string,mixed> $attribs parsed tag-attributes
	 * @param string $content content between openingtag and closing tag @unused-param
	 * @param bool $internal if used by we internally @unused-param
	 * @return string output to print
	 * @throws we_exception_parser if any attribute is missing
	 */
	public static function tag(array $attribs = [], string $content = '', bool $internal = false): void{
		self::attributeMissing($attribs, ['type' => false], __CLASS__);

		$navigationName = self::getAttribute('navigationname', $attribs, 'default', we_base_request::STRING);
		$type = self::getAttribute('type', $attribs, '', we_base_request::STRING);
		$level = self::getAttribute('level', $attribs, we_navigation_items::TEMPLATE_DEFAULT_LEVEL, we_base_request::STRING);
		$current = (isset($attribs['current']) ?
			(self::getAttribute('current', $attribs, false, we_base_request::BOOL) ? '1' : '0') :
			we_navigation_items::TEMPLATE_DEFAULT_CURRENT);

		$positions = self::getAttribute('position', $attribs, [we_navigation_items::TEMPLATE_DEFAULT_POSITION], we_base_request::STRING_LIST);
		$items = we_tag_navigation::getNaviItem($navigationName);
		if(!$items){
			echo self::parseError('we:navigationentry "' . $navigationName . '" not set', __CLASS__);
			return;
		}

		foreach($positions as $position){
			$items->setTemplate(str_replace('\$', '$', $content), $type, $level, $current, $position);
		}
	}

}
