<?php
declare(strict_types=1);

/**
 * webEdition CMS
 *
 * $Rev: bdbf1140d319 $
 * $Author: Marc Kraemer <marc.kraemer@aldente-it.de> $
 * $Date: 2020-07-07 16:59 +0200 $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
class we_tree_customer extends we_tree_base{
	private static $sortField = '';

	public function __construct(we_base_jsCmd $jsCmd){
		parent::__construct($jsCmd);
		$this->addSorted = false;
	}

	public static function setSort(string $sort): void{
		self::$sortField = $sort;
	}

	protected function customJSFile(): string{
		return we_html_element::jsScript(WE_JS_MODULES_DIR . 'customer/customer_tree.js', 'initTree();');
	}

	public static function getItems(int $pid, int $offset = 0, int $segment = 500, bool $sort = false): array{
		if(empty($pid)){
			$_SESSION['we']['custTree'] = [];
		}
		return (empty(self::$sortField) ?
			static::getItemsFromDB($pid, $offset, $segment) :
			static::getSortFromDB($pid, self::$sortField, $offset, $segment));
	}

	private static function getItemsFromDB(int $ParentID = 0, int $offset = 0, int $segment = 500, string $elem = 'ID,Forename,Surname', string $addWhere = ''): array{
		$db = new DB_WE();

		$prevoffset = max(0, $offset - $segment);
		$items = ($offset && $segment ? [
			[
				'id' => 'prev_' . $ParentID,
				'parentid' => $ParentID,
				'text' => 'display (' . $prevoffset . '-' . $offset . ')',
				'contenttype' => self::CT_UP,
				'table' => CUSTOMER_TABLE,
				'typ' => self::TYP_DOTS,
				'open' => 0,
				'published' => 1,
				'disabled' => 0,
				'tooltip' => '',
				'offset' => $prevoffset
			]
			] : []);


		$settings = new we_customer_settings();
		$settings->load();

		$ws = we_users_util::get_ws(CUSTOMER_TABLE);
		$where = ' WHERE 1 ' .
			($ws ? ' AND ' . implode('', $ws) : '') .
			' ' . $addWhere;

		foreach($db->getAllq('SELECT ' . $settings->treeTextFormatSQL . ' AS treeFormat,' . $elem . ',LoginDenied FROM ' . CUSTOMER_TABLE . ' ' . $where . ' ' . self::getSortOrder($settings) . ($segment ? ' LIMIT ' . $offset . ',' . $segment : '')) as $rec){
			$isFolder = !empty($rec['IsFolder']);
			$items[] = [
				'typ' => ($isFolder ? self::TYP_FOLDER : self::TYP_ITEM),
				'contenttype' => ($isFolder ? we_base_ContentTypes::CUSTOMERGROUP : we_base_ContentTypes::CUSTOMER),
				'disabled' => 0,
				'path' => '',
				'published' => $rec['LoginDenied'] ? 0 : 1,
				'tooltip' => intval($rec['ID']),
				'offset' => $offset,
				'id' => intval($rec['ID']),
				'parentid' => 0,
				'isfolder' => 0,
				'text' => htmlspecialchars($rec['treeFormat'])
			];
		}

		$total = fI('SELECT COUNT(1) FROM ' . CUSTOMER_TABLE . ' ' . $where);
		$nextoffset = $offset + $segment;
		if($segment && ($total > $nextoffset)){
			$items[] = [
				'id' => 'next_' . $ParentID,
				'parentid' => 0,
				'text' => 'display (' . $nextoffset . '-' . ($nextoffset + $segment) . ')',
				'contenttype' => self::CT_DOWN,
				'table' => CUSTOMER_TABLE,
				'typ' => self::TYP_DOTS,
				'open' => 0,
				'disabled' => 0,
				'tooltip' => '',
				'offset' => $nextoffset
			];
		}

		return $items;
	}

	private static function getSortFromDB(int $pid, string $sort, int $offset = 0, int $segment = 500): array{
		$db = new DB_WE();
		$items = $foo = $select = $grouparr = $orderarr = $havingarr = $sort_defs = $check = [];
		$noVal = g_l('modules_customer', '[no_value]');

		$settings = new we_customer_settings();
		$settings->load();
		$ws = we_users_util::get_ws(CUSTOMER_TABLE);

		$pos = ($pid * -1);
		$pidarr = $pid && isset($_SESSION['we']['custTree'][$pos]) ? explode('-|-', $_SESSION['we']['custTree'][$pos]) : [];

		if(isset($settings->SortView[$sort])){
			$sort_defs = $settings->SortView[$sort];
		}

		$total = fI('SELECT COUNT(1) FROM ' . CUSTOMER_TABLE);

		foreach($sort_defs as $c => $sortdef){
			if(!empty($sortdef['function'])){
				$select[] = sprintf(we_customer_settings::FUNCTION_TABLE[$sortdef['function']], $sortdef['field']) . ' AS ' . $sortdef['field'] . '_' . $sortdef['function'];

				$grouparr[] = $sortdef['field'] . '_' . $sortdef['function'];
				$orderarr[] = $sortdef['field'] . '_' . $sortdef['function'] . ' ' . $sortdef['order'];
				$orderarr[] = $sortdef['field'] . ' ' . $sortdef['order'];
				if(isset($pidarr[$c])){
					$havingarr[] = ($pidarr[$c] === $noVal ?
						'(' . $sortdef['field'] . '_' . $sortdef['function'] . "='' OR " . $sortdef['field'] . '_' . $sortdef['function'] . ' IS NULL)' :
						$sortdef['field'] . '_' . $sortdef['function'] . "='" . $pidarr[$c] . "'");
				}
			} else {
				$select[] = $sortdef['field'];
				$grouparr[] = $sortdef['field'];
				$orderarr[] = $sortdef['field'] . ' ' . $sortdef['order'];
				if(!empty($pidarr[$c])){
					$havingarr[] = ($pidarr[$c] === $noVal ?
						'(' . $sortdef['field'] . "='' OR " . $sortdef['field'] . ' IS NULL)' :
						$sortdef['field'] . "='" . $pidarr[$c] . "'");
				}
			}
		}

		$level = count($pidarr);
		$levelcount = count($grouparr);
		$select = array_filter($select);
		$grp = implode(',', array_slice($grouparr, 0, $level + 1));
		$old = 0;
		$first = true;

		$db->query('SELECT
	COUNT(1) AS Anz,
	' . $settings->treeTextFormatSQL . ' AS treeFormat,
	ID,
	LoginDenied,
	Forename,
	Surname' .
			($select ? ',' . implode(',', $select) : '' ) . '
FROM ' .
			CUSTOMER_TABLE .
			($ws ? ' WHERE ' . implode('', $ws) : '') .
			' GROUP BY ' . $grp . ($grouparr ? ($level ? ',ID' : '') : 'ID') . ($havingarr ? ' HAVING ' . implode(' AND ', $havingarr) : '') . ' ORDER BY ' . implode(',', $orderarr) . self::getSortOrder($settings, ($orderarr ? ',' : '')) . (($level == $levelcount && $segment) ? ' LIMIT ' . $offset . ',' . $segment : ''));


		while($db->next_record()){
			$old = 0;

			if(!empty($grouparr) && $level == 0){
				$gname = $db->f((string) $grouparr[0]) ?: $noVal;
				if(($key = array_search($gname, $_SESSION['we']['custTree'])) === false){
					$_SESSION['we']['custTree'][] = $gname;
					end($_SESSION['we']['custTree']);
					$key = key($_SESSION['we']['custTree']);
				}
				$gid = ($key + 1) * -1;

				$groupTotal = $db->f('Anz');
				$items[] = [
					'id' => $gid,
					'parentid' => $old,
					'path' => '',
					'text' => $gname . ' (' . $groupTotal . '/' . '<abbr title="' . g_l('modules_customer', '[all]') . ' ' . g_l('modules_customer', '[customer_data]') . '">' . $total . '</abbr>)',
					'contenttype' => we_base_ContentTypes::FOLDER_COMMON,
					'isfolder' => 1,
					'typ' => 'group',
					'disabled' => 0,
					'open' => 0
				];
				$check[$gid] = 1;
			} else {
				$foo = [];
				for($i = 0; $i < $levelcount; $i++){
					$foo[] = ((string) $db->f((string) $grouparr[$i]) ?: $noVal);
					$gname = implode('-|-', $foo);
					if(($key = array_search($gname, $_SESSION['we']['custTree'])) === false){
						$_SESSION['we']['custTree'][] = $gname;
						end($_SESSION['we']['custTree']);
						$key = key($_SESSION['we']['custTree']);
					}
					$gid = ($key + 1) * -1;
					if($i >= $level){
						if(!isset($check[$gid])){
							$items[] = [
								'id' => $gid,
								'parentid' => $old,
								'path' => '',
								'text' => ($db->f((string) $grouparr[$i]) ?: $noVal) /* . ' (' . $db->f('Anz') . ')' */,
								'contenttype' => we_base_ContentTypes::FOLDER_COMMON,
								'isfolder' => 1,
								'typ' => 'group',
								'disabled' => 0,
								'open' => 0
							];
							$check[$gid] = 1;
						}
					}
					$old = $gid;
				}
				$gname = implode('-|-', $foo);
				if(($key = array_search($gname, $_SESSION['we']['custTree'])) === false){
					$_SESSION['we']['custTree'][] = $gname;
					end($_SESSION['we']['custTree']);
					$key = key($_SESSION['we']['custTree']);
				}
				$gid = ($key + 1) * -1;

				if($level == $levelcount){
					$tt = $db->f('treeFormat');
					if($first){
						$prevoffset = max(0, $offset - $segment);
						if($offset && $segment){
							$items[] = [
								'id' => 'prev_' . $gname,
								'parentid' => $gid,
								'text' => 'display (' . $prevoffset . '-' . $offset . ')',
								'contenttype' => self::CT_UP,
								'table' => CUSTOMER_TABLE,
								'typ' => self::TYP_DOTS,
								'open' => 0,
								'published' => 1,
								'disabled' => 0,
								'tooltip' => '',
								'offset' => $prevoffset
							];
						}
						$first = false;
					}
					$items[] = [
						'id' => $db->f('ID'),
						'parentid' => $gid,
						'path' => '',
						'text' => htmlspecialchars($tt),
						'contenttype' => we_base_ContentTypes::CUSTOMER,
						'isfolder' => 0,
						'typ' => 'item',
						'disabled' => 0,
						'published' => $db->f('LoginDenied') ? 0 : 1,
						'tooltip' => $db->f('ID')
					];
				}
			}
		}

		if($level == $levelcount){
			$total = fI('SELECT COUNT(ID) ' . (empty($select) ? '' : ',' . implode(',', $select)) . ' FROM ' . CUSTOMER_TABLE . ' GROUP BY ' . $grp . (empty($grouparr) ? 'ID' : ($level ? ',ID' : '')) . (empty($havingarr) ? '' : ' HAVING ' . implode(' AND ', $havingarr)));

			$nextoffset = $offset + $segment;
			if($segment && ($total > $nextoffset)){
				$items[] = [
					'id' => 'next_' . $old,
					'parentid' => $old,
					'text' => 'display (' . $nextoffset . '-' . ($nextoffset + $segment) . ')',
					'contenttype' => self::CT_DOWN,
					'table' => CUSTOMER_TABLE,
					'typ' => self::TYP_DOTS,
					'open' => 0,
					'disabled' => 0,
					'tooltip' => '',
					'offset' => $nextoffset
				];
			}
		}

		return $items;
	}

	public static function getSortOrder(we_customer_settings $settings, string $concat = 'ORDER BY'): string{
		$ret = ($settings->getSettings('default_order') ?
			($settings->formatFields ?
			implode(' ' . $settings->getSettings('default_order') . ',', $settings->formatFields) . ' ' . $settings->getSettings('default_order') :
			'Username ' . $settings->getSettings('default_order')) :
			'Username');
		return ($ret ? $concat . ' ' . $ret : '');
	}

}
