/* global WE, top,Fileupload_controller_base,Fileupload_sender_base,Fileupload_view_base,Fileupload_imageEdit_base,Fileupload_utils_base,Fileupload_controller_bindoc,Fileupload_sender_bindoc,Fileupload_view_bindoc,Fileupload_imageEdit_bindoc,Fileupload_utils_bindoc, Fileupload_controller_import,Fileupload_sender_import,Fileupload_view_import,Fileupload_imageEdit_import,Fileupload_utils_import */

/**
 webEdition CMS
 *
 * $Author: Lukas Imhof $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
'use strict';

function Fileupload_uploader_abstract(win) {
	this.win = win;
	this.doc = win.document;

	this.controller = null;
	this.sender = null;
	this.view = null;
	this.imageEdit = null;
	this.utils = null;

	this.fieldName = '';
	this.genericFilename = '';
	this.fileuploadType = 'base';
	this.uiType = 'base';
	this.debug = false;
	this.uploaderState = 'empty';

	this.init = function (conf) {
		if (typeof conf !== 'undefined') {
			this.fieldName = conf.fieldName ? conf.fieldName : this.fieldName;
			this.uiType = conf.uiType ? conf.uiType : this.uiType;
			this.genericFilename = conf.genericFilename ? conf.genericFilename : this.genericFilename;

			this.controller.init(conf);
			this.sender.init(conf);
			this.view.init(conf);
			this.imageEdit.init(conf);
			this.utils.init(conf);

			this.init_sub(conf);

			if (!this.onload()) {
				this.win.addEventListener('load', this.onload.bind(this), true);
			}

			WE().util.setIconOfDocClass(this.doc, 'filedrag_set_icon');
		}
	};

	this.onload = function () {
		if (this.uiType !== 'editFull' && !this.doc.getElementById(this.fieldName)) {
			return false;
		}

		this.sender.onload();
		this.view.onload();
		this.controller.onload();
		this.onload_sub();

		if (this.controller.checkIsPresetFiles()) {
			this.controller.loadPresetFiles();
		} else {
			this.controller.processEvents(this.controller.ON.GUI_INIT);
		}


		return true;
	};

	this.onload_sub = function () {
		// to be overridden
	};

	this.startUpload = function (replaceCmdUploadFinished) {
		this.sender.replaceCmdUploadFinished = replaceCmdUploadFinished ? replaceCmdUploadFinished : '';

		if (this.sender.prepareUpload()) {
			window.setTimeout(this.sender.sendNextFile.bind(this.sender), 100);
		} else {
			// throw default error message or implement function to evaluate reasons of failure
		}
	};

	this.checkStartUpload = function () {
		// to be overridden: available only in base type
	};

	this.cancelUpload = function () {
		this.controller.processEvents(this.controller.ON.UPLOAD_CANCEL);
	};

	this.getIsUploading = function () {
		return (this.getUploaderState() === this.controller.STATE.UPLOADING);
	};

	this.reset = function () {
		this.controller.processEvents(this.controller.ON.GUI_RESET);
	};

	this.removeItem = function (index, but) {
		this.view.removeItem(index, but);
	};

	this.getType = function () {
		return this.fileuploadType;
	};

	this.doUploadIfReady = function (callback) {
		callback();

	};

	this.reeditImage = function (index, general) {
		this.imageEdit.reeditImage(index, general);
	};

	this.openImageEditor = function (pos) {
		this.imageEdit.openImageEditor(pos);
	};

	this.getUploaderState = function () {
		return this.uploaderState;
	};

	this.uploadFromUrlAndData = function (file, cmdUploadFinished) {
		this.controller.uploadFromUrlAndData(file, cmdUploadFinished);
	};
}

function Fileupload_uploader_base(win, conf) {
	Fileupload_uploader_abstract.call(this, win);

	this.fileuploadType = 'base';

	this.controller = new Fileupload_controller_base(this);
	this.sender = new Fileupload_sender_base(this);
	this.view = new Fileupload_view_base(this);
	this.imageEdit = new Fileupload_imageEdit_base(this);
	this.utils = new Fileupload_utils_base(this);

	this.init_sub = function (conf) {};

	this.onload_sub = function () {};

	/* start upload if state === ready; display message if not, fire replaceCmdFinishUpload if upload finished*/
	this.checkStartUpload = function (replaceCmdFinishUpload) {
		switch (this.getUploaderState()) {
			case this.controller.STATE.READY:
				this.startUpload(replaceCmdFinishUpload);
				break;
			case this.controller.STATE.NOTREADY:
				WE().util.showMessage(WE().consts.g_l.fileupload.errorSelectedNotValid, WE().consts.message.WE_MESSAGE_WARNING, this.win);
				break;
			case this.controller.STATE.UPLOADING:
				WE().util.showMessage(WE().consts.g_l.fileupload.errorUploadNotFinished, WE().consts.message.WE_MESSAGE_WARNING, this.win);
				break;
			case this.controller.STATE.FINISHED:
				if (replaceCmdFinishUpload) {
					var tmp = replaceCmdFinishUpload.split(',');
					tmp.splice(1, 0);
					window.setTimeout(this.controller.fireCmd, 100, [this.win, tmp]);
				}
				break;
			case this.controller.STATE.EMPTY:
			/* falls through */
			default:
				WE().util.showMessage(WE().consts.g_l.fileupload.errorNoFileSelected, WE().consts.message.WE_MESSAGE_WARNING, this.win);
		}
	};
}
Fileupload_uploader_base.prototype = Object.create(Fileupload_uploader_abstract.prototype);
Fileupload_uploader_base.prototype.constructor = Fileupload_uploader_base;

function Fileupload_uploader_bindoc(win) {
	Fileupload_uploader_abstract.call(this, win);

	this.fileuploadType = 'binDoc';

	this.controller = new Fileupload_controller_bindoc(this);
	this.sender = new Fileupload_sender_bindoc(this);
	this.view = new Fileupload_view_bindoc(this);
	this.imageEdit = new Fileupload_imageEdit_bindoc(this);
	this.utils = new Fileupload_utils_bindoc(this);

	this.init_sub = function (conf) {
		this.fieldName = 'we_File';
	};

	this.onload_sub = function () {};
}
Fileupload_uploader_bindoc.prototype = Object.create(Fileupload_uploader_abstract.prototype);
Fileupload_uploader_bindoc.prototype.constructor = Fileupload_uploader_bindoc;

function Fileupload_uploader_editFull(win) {
	Fileupload_uploader_abstract.call(this, win);

	this.fileuploadType = 'editFull';

	this.controller = new Fileupload_controller_editFull(this);
	this.sender = new Fileupload_sender_bindoc(this);
	this.view = new Fileupload_view_editFull(this);
	this.imageEdit = new Fileupload_imageEdit_editFull(this);
	this.utils = new Fileupload_utils_bindoc(this);

	this.init_sub = function (conf) {
		this.fieldName = 'we_File';
		this.dataUrl = conf.dataUrl ? conf.dataUrl : null;
	};

	this.onload_sub = function () {};
}
Fileupload_uploader_editFull.prototype = Object.create(Fileupload_uploader_abstract.prototype);
Fileupload_uploader_editFull.prototype.constructor = Fileupload_uploader_editFull;

function Fileupload_uploader_import(win) {
	Fileupload_uploader_abstract.call(this, win);
	this.fileuploadType = 'importer';

	this.controller = new Fileupload_controller_import(this);
	this.sender = new Fileupload_sender_import(this);
	this.view = new Fileupload_view_import(this);
	this.imageEdit = new Fileupload_imageEdit_import(this);
	this.utils = new Fileupload_utils_import(this);

	this.init_sub = function (conf) {};
}
Fileupload_uploader_import.prototype = Object.create(Fileupload_uploader_abstract.prototype);
Fileupload_uploader_import.prototype.constructor = Fileupload_uploader_import;

WE().layout.fileupload.getFileUploadGlobal = WE().layout.fileupload.getFileUploadGlobal || function (type, win) {
	switch (type) {
		case 'base':
			return new Fileupload_uploader_base(win);
		case 'preview' :
		case 'wedoc' :
		case 'editor' :
			return new Fileupload_uploader_bindoc(win);
		case 'editFull' :
			return new Fileupload_uploader_editFull(win);
		case 'importer':
			return new Fileupload_uploader_import(win);
	}
};

function getFileUploadLocal(type, win) {
	switch (type) {
		case 'base':
			return new Fileupload_uploader_base(win);
		case 'preview' :
		case 'wedoc' :
		case 'editor' :
			return new Fileupload_uploader_bindoc(win);
		case 'editFull' :
			return new Fileupload_uploader_editFull(win);
		case 'importer':
			return new Fileupload_uploader_import(win);
	}
}
