/* global tinymce */
/**
 * webEdition CMS
 *
 * $Author: Marc Kraemer <marc.kraemer@aldente-it.de> $
 *
 * This source is part of webEdition CMS. webEdition CMS is
 * free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * any later version.
 *
 * The GNU General Public License can be found at
 * http://www.gnu.org/copyleft/gpl.html.
 * A copy is found in the textfile
 * webEdition/licenses/webEditionCMS/License.txt
 *
 * @license	http://www.gnu.org/copyleft/gpl.html  GPL
 */
'use strict';

(function () {
	var wexhtmlxtras = (function () {
		var global = tinymce.util.Tools.resolve('tinymce.PluginManager');

		var Commands = {register: function (editor) {
				var g_l = WE().consts.g_l.tinyMceTranslationObject;

				editor.addCommand('mceWeabbr', function () {
					editor.windowManager.openWePlugin({
						file: '/webEdition/we_cmd_frontend.php?we_cmd[0]=open_dialog_abbr&we_dialog_args[editor]=tinyMce&we_dialog_args[isFrontend]=' + editor.getParam('weIsFrontend'),
						width: 480,
						height: 240
					}, {
						tag: 'abbr'
					});
				});
				editor.addCommand('mceWeacronym', function () {
					editor.windowManager.openWePlugin({
						file: '/webEdition/we_cmd_frontend.php?we_cmd[0]=open_dialog_acronym&we_dialog_args[editor]=tinyMce&we_dialog_args[isFrontend]=' + editor.getParam('weIsFrontend'),
						width: 480,
						height: 240
					}, {
						tag: 'acronym'
					});
				});
				editor.addCommand('mceWelang', function () {
					editor.windowManager.openWePlugin({
						file: '/webEdition/we_cmd_frontend.php?we_cmd[0]=open_dialog_lang&we_dialog_args[editor]=tinyMce&we_dialog_args[isFrontend]=' + editor.getParam('weIsFrontend'),
						popup_css: false,
						width: 460,
						height: 180
					}, {});
				});
				editor.addCommand('mceCite', function () {
					editor.windowManager.openWePlugin({
						file: '/webEdition/we_cmd_frontend.php?we_cmd[0]=open_dialog_xhtmlxtras&we_dialog_args[plugin]=cite',
						width: 480,
						height: 310,
						inline: false
					}, {
						plugin_url: ''//url
					});
				});
				editor.addCommand('mceDel', function () {
					editor.windowManager.openWePlugin({
						file: '/webEdition/we_cmd_frontend.php?we_cmd[0]=open_dialog_xhtmlxtras&we_dialog_args[plugin]=del',
						width: 480,
						height: 440,
						inline: false
					}, {
						plugin_url: ''//url
					});
				});
				editor.addCommand('mceIns', function () {
					editor.windowManager.openWePlugin({
						file: '/webEdition/we_cmd_frontend.php?we_cmd[0]=open_dialog_xhtmlxtras&we_dialog_args[plugin]=ins',
						width: 480,
						height: 440,
						inline: false
					}, {
						plugin_url: ''//url
					});
				});
			}};

		var Buttons = {register: function (editor) {
				editor.ui.registry.addToggleButton('weabbr', {
					icon: 'abbr',
					tooltip: WE().consts.g_l.tinyMceTranslationObject.tt_weabbr,
					onAction: function () {
						editor.execCommand('mceWeabbr');
					},
					onSetup: function (buttonApi) {
						// this code is once executed on setup: when toolbar is loaded AND EVERY TIME when quickbar is loaded
						buttonApi.setDisabled(editor.selection.isCollapsed());
						buttonApi.setActive(false);
						if (editor.dom.getParent(editor.selection.getNode(), 'ABBR')) {
							buttonApi.setActive(true);
						}

						// this code is once executed on setup: when toolbar is loaded AND EVERY TIME when quickbar is loaded
						//editor.weControlManager.register('button', 'weabbr', buttonApi);
					}
				});
				editor.ui.registry.addToggleButton('weacronym', {
					icon: 'acronym',
					tooltip: WE().consts.g_l.tinyMceTranslationObject.tt_weacronym,
					onAction: function () {
						editor.execCommand('mceWeacronym');
					},
					onSetup: function (buttonApi) {
						buttonApi.setDisabled(editor.selection.isCollapsed());
						buttonApi.setActive(false);
						if (editor.dom.getParent(editor.selection.getNode(), 'ACRONYM')) {
							buttonApi.setActive(true);
						}
						//editor.weControlManager.register('button', 'weacronym', buttonApi);
					}
				});
				editor.ui.registry.addToggleButton('welang', {
					icon: 'language',
					tooltip: WE().consts.g_l.tinyMceTranslationObject.tt_welang,
					onAction: function () {
						editor.execCommand('mceWelang');
					},
					onSetup: function (buttonApi) {
						buttonApi.setDisabled(editor.selection.isCollapsed());
						buttonApi.setActive(false);

						var n = editor.dom.getParent(editor.selection.getNode(), 'SPAN');
						if (n) {
							do {
								if (n.nodeName && n.nodeName.toLowerCase() === 'span' && n.getAttribute('lang') && n.getAttribute('lang') !== '') {
									buttonApi.setActive(true);
									break;
								}
							} while ((n = n.parentNode && n.nodeName.toLowerCase() === 'body'));
						}
						//editor.weControlManager.register('button', 'welang', buttonApi);
					}
				});
				editor.ui.registry.addToggleButton('cite', {
					icon: 'cite',
					tooltip: WE().consts.g_l.tinyMceTranslationObject.tt_cite,
					onAction: function () {
						editor.execCommand('mceCite');
					},
					onSetup: function (buttonApi) {
						buttonApi.setDisabled(editor.selection.isCollapsed());
						buttonApi.setActive(false);
						if (editor.dom.getParent(editor.selection.getNode(), 'CITE')) {
							buttonApi.setActive(true);
						}
						//editor.weControlManager.register('button', 'cite', buttonApi);
					}
				});
				editor.ui.registry.addToggleButton('del', {
					icon: 'del',
					tooltip: WE().consts.g_l.tinyMceTranslationObject.tt_del,
					onAction: function () {
						editor.execCommand('mceDel');
					},
					onSetup: function (buttonApi) {
						buttonApi.setDisabled(editor.selection.isCollapsed());
						buttonApi.setActive(false);
						if (editor.dom.getParent(editor.selection.getNode(), 'DEL')) {
							buttonApi.setActive(true);
						}
						//editor.weControlManager.register('button', 'del', buttonApi);
					}
				});
				editor.ui.registry.addToggleButton('ins', {
					icon: 'ins',
					tooltip: WE().consts.g_l.tinyMceTranslationObject.tt_ins,
					onAction: function () {
						editor.execCommand('mceIns');
					},
					onSetup: function (buttonApi) {
						buttonApi.setDisabled(editor.selection.isCollapsed());
						buttonApi.setActive(false);
						if (editor.dom.getParent(editor.selection.getNode(), 'INS')) {
							buttonApi.setActive(true);
						}
						//editor.weControlManager.register('button', 'ins', buttonApi);
					}
				});
			}};

		global.add('wexhtmlxtras', function (editor) {
			Commands.register(editor);
			Buttons.register(editor);
			return {};
		});

		function Plugin() {}

		return Plugin;
	}());
})();